/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2003, 2009                               */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/*  KENT               TD00290  051890 PREVENT SETUP PROBLEMS        */
/*  URFER              TD00753  070897 ADD D109 DRIVER               */
/*  VALLETTA           OO00350  090497 ADD TO2D DRIVER               */
/*  LEES               P700288  111197 ADD DLCK DRIVER               */
/*  URFER              TD00775  011398 ADD PUT7 DRIVER               */
/*  JTARBY             P700255  031098 CHG DB2S DRIVER               */
/*  URFER              TD00782  031198 ADD TSIM DRIVER               */
/*  BILLHO                      021299 ADD TCPP DRIVER               */
/*  BOHALEL                     080808 ADD DFPD DRIVER               */
/*  KISHORE            19692    081309 Changed "sd0loop" from short  */
/*                                     to unsigned short so it wraps */
/*                                     to 0 rather than a negative   */
/*                                     number when an overflow       */
/*                                     occurs.                       */
/*                                                                   */
/*********************************************************************
**
** HEADER FILE NAME..C_SD0RV.H
** HEADER FILE TYPE..(STANDARD, ANSI, S/370, TPF)
** DOCUMENT NAME...  CLS-UGR, C/370 USER'S GUIDE
** DESCRIPTION.....  I-Stream Shared Driver Global Record
** LEVEL...........  VERSION 1  MODIFICATION 0
**
** FUNCTION... This header maps the non-keypointable global
**             record, @ISUDRV, used by I-Stream shared test
**             drivers.  This record is used to keep temporary
**             status information for test drivers between IPLs.
**             Each IPL will reset the contents of this record,
**             thus indicating that the drivers are no longer
**             active.  This record resides in global area 2.
**
**             Note:  global area 2 is not protected, thus a GLMOD
**             (or GLOBAL_MODIFY) is not necessary.
**
**
**********************************************************************
**
** DEPENDENCIES... Any driver using this global can be affected by
**                 changes to this header.
**
** RESTRICTIONS... To keep the header readable, keep the definitions
**                 simple.
**
** SPECIAL CONSIDERATIONS...
**
**********************************************************************
**
** #INCLUDED BY...  The following drivers include this header:
**                  DEB5
**                  TGBS
**                  MPGM
**                  FIFI
**                  CORO
**                  D109
**                  TO2D
** OTHER HEADER FILES #INCLUDED... None
**
** ACRONYMS...    API     APPLICATION PROGRAMMING INTERFACE
**
**********************************************************************
**********************************************************************
* Aggregate map for: structure SD0RV  DATED 03/11/98
**********************************************************************
|      Offset       |      Length       | Member Name
|    Bytes(Bits)    |    Bytes(Bits)    |
|===================|===================|===============================
|       0           |       2           |  SD0RID
|       2           |       1           |  SD0RCC
|       3           |       1           |  SD0CTL
|       4           |       4           |  SD0PGM
|       8           |       4           |  SD0FWD
|      12           |       4           |  SD0BACK
|      16           |      64           |  _SD0DEB4[16]
|      80           |       2           |  _SD0FISPA
|      82           |       2           |  _SD0FIMAX
|      84           |       4           |  _SD0FICUR
|      88           |      32           |  _SD0MPIF[8]
|     120           |      28           |  _SD0GDSN[7]          D18270  D18190
|     148           |       4           |  _SD0DFPD             D18270  D18190
|     152           |       8           |  _SD0TAPE[2]
|     160           |     256           |  _SD0TAPG[16]
|     416           |       4           |  _SD0UNIT[4]
|     420           |       1           |  sd0put71
|     421           |       1           |  sd0put72
|     422           |       1           |  sd0put73
|     423           |       1           |  sd0put74
|     424           |       4           |  sd0put7g
|     428           |       4           |  sd0put7c
|     432           |       4           |  sd0put7s
|     436           |       4           |  sd0put7l
|     440           |       4           |  sd0putmg
|     444           |       4           |  sd0putmc
|     448           |       4           |  sd0putms
|     452           |       4           |  sd0putml
|     456           |       4           |  sd0putmr
|     460           |       8           |  sd0putd1[2]
|     468           |       4           |  sd0putt1
|     472           |       4           |  sd0putf2
|     476           |       4           |  sd0putc1
|     480           |       4           |  sd0putc2
|     484           |       4           |  sd0putc3
|     488           |       8           |  sd0putts[2]
|     496           |       8           |  _SD0CFLF[2]
|     504           |       8           |  _SD0DFTD[8]
|     512           |       4           |  sd0cmt
|     516           |       4           |  sd0rlb
|     520           |       2           |  sd0riat[2]
|     522           |       2           |  sd0nest
|     524           |       2           |  sd0mecb
|     526           |       2           |  sd0aecb
|     528           |       1           |  sd0flg1
|     529           |       1           |  sd0flg2
|     530           |       1           |  sd0flg3
|     531           |       1           |  sd0flg4
|     532           |       2           |  sd0fcnt
|     534           |       2           |  sd0junk
|     536           |     160           |  sd0spare1[40]
|     696           |       1           |  sd0d1091
|     697           |       1           |  sd0d1092
|     698           |       1           |  sd0d1093
|     699           |       1           |  sd0d1094
|     700           |       4           |  sd0d109c
|     704           |       1           |  sd0to2dq
|     705           |       1           |  sd0to2ds
|     706           |       1           |  sd0to2dp
|     707           |       1           |  sd0to2sw
|     707           |       0(6)        |    to2drsvd
|     707(6)        |       0(1)        |    to2dstp
|     707(7)        |       0(1)        |    to2dact
|     708           |       4           |  sd0to2ct
|     712           |       4           |  sd0to2es
|     716           |       4           |  sd0to2ec
|     720           |       4           |  sd0to2mx
|     724           |       4           |  sd0to2S1
|     728           |       8           |  sd0to2lk
|     736           |       1           |  sd0ind1
|     737           |       1           |  sd0ind2
|     738           |       1           |  sd0ind3
|     739           |       1           |  sd0ind4
|     740           |       4           |  ***PADDING***
|     744           |     184           |  _SD0DB2
|     744           |       1           |    sd0db2s1
|     745           |       1           |    sd0db2s2
|     746           |       1           |    sd0db2s3
|     747           |       1           |    sd0db2s4
|     748           |       4           |    sd0db2sa
|     752           |       4           |    sd0db2sd
|     756           |       4           |    sd0db2su
|     760           |       4           |    sd0db2ss
|     764           |       4           |    sd0db2ma
|     768           |       4           |    sd0db2md
|     772           |       4           |    sd0db2mu
|     776           |       4           |    sd0db2ms
|     780           |       4           |    sd0db2mr
|     784           |      64           |    sd0dbrdb[16]
|     848           |       4           |    sd0db2c1
|     852           |       4           |    sd0db2c2
|     856           |       4           |    sd0db2c3
|     860           |       4           |    ***PADDING***
|     864           |       8           |    sd0db2ts
|     872           |       8           |    sd0db2lock
|     880           |       4           |    sd0db2mc
|     884           |      20           |    sd0db2spare[5]
|     904           |      19           |    sd0db2pack[19]
|     923           |       5           |    ***PADDING***
|     928           |       1           |  sd0tsim1
|     929           |       1           |  sd0tsim2
|     930           |       1           |  sd0tsim3
|     931           |       1           |  sd0tsim4
|     932           |       2           |  sd0tsimm
|     934           |       2           |  sd0tsimn
|     936           |       2           |  sd0tsimt
|     938           |       2           |  sd0tsima
|     940           |       4           |  sd0tsimc
|     944           |       8           |  sd0tsims[2]
|     952           |       1           |  sd0st01
|     953           |       1           |  sd0st02
|     954           |       1           |  sd0st03
|     955           |       1           |  sd0st04
|     956           |       4           |  sd0cunt
|     960           |       4           |  sd0plun
|     964           |       1           |  sd0tcpsw
|     965           |       1           |  sd0run
|     966           |       2           |  sd0from
|     968           |       2           |  sd0to
|     970           |       2           |  sd0wait
|     972           |       2           |  sd0loop
|     974           |       2           |  sd0curr
|     976           |       5           |  sd0name
|     981           |       2           |  sd0tmp02
|     983           |       1           |  sd0tcplk
========================================================================
*/
#ifndef _sd0rv__
#define _sd0rv__
typedef struct _sd0db2_struct
  {
    unsigned char sd0db2s1;     /* X  DB2S DRIVER STATUS BYTE ONE   */
    unsigned char sd0db2s2;     /* X  DB2S DRIVER STATUS BYTE TWO   */
    unsigned char sd0db2s3;     /* X  DB2S DRIVER STATUS BYTE THREE */
    unsigned char sd0db2s4;     /* X  DB2S DRIVER STATUS BYTE FOUR  */
    int    sd0db2sa;           /* F  NUM OF DB2S ADD ECBS TO START */
    int    sd0db2sd;           /* F  NUM OF DB2S DEL ECBS TO START */
    int    sd0db2su;           /* F  NUM OF DB2S UPD ECBS TO START */
    int    sd0db2ss;           /* F  NUM OF DB2S SEL ECBS TO START */
    int    sd0db2ma;           /* F  NUM OF DB2S ADD ECBS MAX      */
    int    sd0db2md;           /* F  NUM OF DB2S DEL ECBS MAX      */
    int    sd0db2mu;           /* F  NUM OF DB2S UPD ECBS MAX      */
    int    sd0db2ms;           /* F  NUM OF DB2S SEL ECBS MAX      */
    int    sd0db2mr;           /* F  NUM OF DB2S Repeats           */
    char *  sd0dbrdb[16];       /* 4F RDBNAME TO BE USED ON CONNECT */
    int    sd0db2c1;           /* F  DRIVER STAT FIELD ONE         */
    int    sd0db2c2;           /* F  DRIVER STAT FIELD TWO         */
    int    sd0db2c3;           /* F  DRIVER STAT FIELD THREE       */
    double  sd0db2ts;           /* 2F TIME STAMP OF LAST CRETC      */
    double  sd0db2lock;         /* 2F DB2S DRIVER LOCK WORDS        */
    int    sd0db2mc;           /* F  NUM OF DB2S columns           */
    int    sd0db2spare[5];     /* for future expansion             */
    char    sd0db2pack[19];     /* X19 DB2 DRIVER CURRENT PACKAGESET*/
  } SD0DB2_STRUCT;

struct SD0RV
      {
/***************************/
/* RECORD HEADER           */
/***************************/
  unsigned short    SD0RID;   /* RECORD ID - C'DR'                    */
  unsigned char     SD0RCC;   /* RECORD CODE CHECK                    */
  unsigned char     SD0CTL;   /* CONTROL BYTE - NOT USED              */

  unsigned int     SD0PGM;   /* LAST PROGRAM TO FILE THIS RECORD     */
  unsigned int     SD0FWD;   /* FORWARD CHAIN - NOT USED             */
  unsigned int     SD0BACK;  /* BACKWARD CHAIN - NOT USED            */

/***************************/
/* DEB5 driver             */
/***************************/
  unsigned int     _SD0DEB4[16];  /* DEB5 DRIVER GLOBAL AREA         */

/***************************/
/* FIFI driver             */
/***************************/
  unsigned short    _SD0FISPA;/* FIFI DRIVER SPARE                    */
  unsigned short    _SD0FIMAX;/* FIFI DRIVER MAX ORDINAL NUMBER       */
  unsigned int     _SD0FICUR;/* FIFI DRIVER CURRENT ORDINAL NUMBER   */

/***************************/
/* MPIF driver             */
/***************************/
  unsigned int     _SD0MPIF[8];   /* MPIF DRIVER GLOBAL AREA         */

/***************************/
/* GDSN driver             */
/***************************/
  unsigned int     _SD0GDSN[7];   /* GDSN DRIVER GLOBAL AREA         */

/***************************/
/* DFPD Driver             */
/***************************/
  unsigned int      _SD0DFPD;      /* DFPD CONTROL TABLE ADDR  D18270 */        //D18190

/***************************/
/* TAPE driver             */
/***************************/
  unsigned int     _SD0TAPE[2];        /* TAPE DRIVER STATUS FIELDS  */

                            /* Number of tape driver tapes configured */
#define    SD0MAXTAPE 16
  unsigned char     _SD0TAPG[SD0MAXTAPE][16]; /* TAPE DRIVER AREA 2   */

/***************************/
/* UNIT record driver      */
/***************************/
  unsigned char     _SD0UNIT[4]; /* UNIT RECORD SUPPORT STATUS FIELDS */

/***************************/
/* PUT7 driver             */
/***************************/
    unsigned char sd0put71;     /* X  STATUS BYTE 1 - active flag   */
    unsigned char sd0put72;     /* X  STATUS BYTE 2 - prt flag      */
    unsigned char sd0put73;     /* X  STATUS BYTE 3 - t/c lock      */
    unsigned char sd0put74;     /* X  STATUS BYTE 4 - cretc time    */
    int    sd0put7g;           /* F  NUM OF PUT7 GETE ECB ACTIVE   */
    int    sd0put7c;           /* F  NUM OF PUT7 CREF ECB ACTIVE   */
    int    sd0put7s;           /* F  NUM OF PUT7 STOE ECB ACTIVE   */
    int    sd0put7l;           /* F  NUM OF PUT7 LMFN ECB ACTIVE   */
    int    sd0putmg;           /* F  NUM OF PUT7 GETE ECB MAX      */
    int    sd0putmc;           /* F  NUM OF PUT7 CREF ECB MAX      */
    int    sd0putms;           /* F  NUM OF PUT7 STOE ECB MAX      */
    int    sd0putml;           /* F  NUM OF PUT7 LMFN ECB MAX      */
    int    sd0putmr;           /* F  NUM OF PUT7 Repeats           */
    int    sd0putd1[2];        /* 2F RANDOM NUMBER                 */
    int    sd0putt1;           /* F  TOKEN NUMBER                  */
    int    sd0putf2;           /* F  spare                         */
    int     sd0putc1;           /* F  total counter                 */
    int     sd0putc2;           /* F  temp counter                  */
    int     sd0putc3;           /* F  count   per second            */
    int    sd0putts[2];        /* 2F TIME STAMP LAST CRETC         */


/***************************/
/* CFLF driver             */
/***************************/
  unsigned int     _SD0CFLF[2];

/***************************/
/* DFTD driver             */
/***************************/
  char              _SD0DFTD[8];

/***************************/
/* CORO DRIVER             */
/***************************/
  int               sd0cmt;       /* number of commits               */
  int               sd0rlb;       /* number of rollback              */
#ifndef __SYSC__                                                         //Dignus
const
#endif                                                                   //Dignus
  char              sd0riat[2];   /* RID= parameter, RIAT ID         */
  short             sd0nest;      /* NEST= parameter value           */
  short             sd0mecb;      /* ECB= parameter, max num of ECB  */
  short             sd0aecb;      /* number of active ECB            */
  char              sd0flg1;      /* flags                           */
  char              sd0flg2;      /* flags                           */
  char              sd0flg3;      /* flags                           */
  char              sd0flg4;      /* flags                           */
  short             sd0fcnt;      /* number of requested file address*/
  short             sd0junk;      /* number of requested file address*/
#define SD03PRT          0x80

/***************************/
/* IEFC Driver             */
/***************************/
  int               sd0iefsw:8;   /* IEFC Switches                   */
  #define  IEFDSTP 0x04           /* ..driver stop pending           */
  #define  IEFDST  0x02           /* ..System Test active            */
  #define  IEFDFT  0x01           /* ..Function Test active          */

  unsigned int     _SD0IEFC[7];   /* Reserved for IEFC driver use    */

/***************************/
/* SPARE                   */
/***************************/
  unsigned int     sd0spare1[32];

/***************************/
/* D109 Driver             */
/***************************/
  char              sd0d1091;      /* running flag                   */
  char              sd0d1092;      /* cretc time in seconds          */
  char              sd0d1093;      /* hold output flag               */
  char              sd0d1094;      /* SPARE                          */
  int               sd0d109c;      /* current message number         */

/***************************/
/* TO2D Test Driver Area   */
/***************************/
  char              sd0to2dq;      /* Suppress Printf's (Quiet)      */
  char              sd0to2ds;      /* Collection Size Large          */
  char              sd0to2dp;      /* Send Messages to Print Log     */
  struct {
    unsigned int to2drsvd : 6;     /* BITS 1-5 RESERVED (bit 0 - prt)*/
    unsigned int to2dstp  : 1;     /* BIT 6 - TO2D stop pending      */
    unsigned int to2dact  : 1;     /* BIT 7 - TO2D active            */
  } __attribute__ ((packed)) sd0to2sw;  /* TO2D Switches             */
  int              sd0to2ct;      /* Count of active ECBs           */
  int              sd0to2es;      /* Total ECBs started             */
  int              sd0to2ec;      /* Total ECBs completed           */
  int              sd0to2mx;      /* Max active ECB target          */
  int              sd0to2S1;      /* Set of collections             */
  double            sd0to2lk;      /* Communication area lock word   */

/*******************************/
/* DEADLOCK Test Driver Area   */
/*******************************/
  char              sd0ind1;       /* Status indicator               */
  char              sd0ind2;       /* Reserved                       */
  char              sd0ind3;       /* Reserved                       */
  char              sd0ind4;       /* Reserved                       */

/***********************************************/
/* Latest and greatest TPFAR/2 DB2S driver     */
/* (complete with 4 spare fullwords)           */
/***********************************************/
  SD0DB2_STRUCT    _SD0DB2;
/*******************************/
/* TSIM     Test Driver Area   */
/*******************************/
  char              sd0tsim1;
  char              sd0tsim2;
  char              sd0tsim3;
  char              sd0tsim4;
  short             sd0tsimm;
  short             sd0tsimn;
  short             sd0tsimt;
  short             sd0tsima;
  int              sd0tsimc;
  int              sd0tsims[2];
/*******************************/
/* NULL DRIVER AREA            */
/*******************************/
  char              sd0st01;
  char              sd0st02;
  char              sd0st03;
  char              sd0st04;
  int              sd0cunt;
  int              sd0plun;
/*******************************/
/* TCPP DRIVER AREA (20 bytes) */
/*******************************/
  char              sd0tcpsw;
  unsigned char     sd0run;
  short             sd0from;
  short             sd0to;
  short             sd0wait;
  unsigned short    sd0loop;
  short             sd0curr;
  char              sd0name[5];
  char              sd0tmp02[2];
  unsigned char     sd0tcplk;
/*******************************/
/* END                         */
/*******************************/

      };



#endif
