/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2003, 2005                               */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/*                                                                   */
/*        UPDATE BY          PLOG #   DATE   DESCRIPTION             */
/*        ---------          -------  ------ ------------            */
/*        KENT               TD00301  071390 MSG SETTING ENHANCEMENTS*/
/*        KENT               TD00290  051890 PREVENT SETUP PROBLEMS  */
/*        KENT               TD00277  041290 ADD SYSTEST INTERFACE   */
/*        KENT               TD00346  022891 TEP PROJECT IMPROVEMENTS*/
/*                                                                   */
/*********************************************************************/
/*
**********************************************************************
**
** HEADER FILE NAME..C_TA0EQ.H
** HEADER FILE TYPE..(STANDARD, ANSI, S/370, TPF)
** DOCUMENT NAME...  CLS-UGR, C/370 USER'S GUIDE
** DESCRIPTION.....  HEADER FILE FOR TPF TAPE DRIVER 'C' SEGMENTS
** LEVEL...........  VERSION 1  MODIFICATION 0
**
** FUNCTION... THIS HEADER FILE DEFINES THE USAGE OF THE ECB AREAS,
**             USAGE OF THE GLOBAL AREAS, AND DEFINES VALUES USED
**             AS AN INTERFACE BETWEEN TAPE DRIVER SEGMENTS.
**
**********************************************************************
**
** DEPENDENCIES... ALL TAPE DRIVER SEGMENTS COULD BE AFFECTED BY
**                 CHANGES TO THIS HEADER FILE
**
** RESTRICTIONS... NONE
**
** SPECIAL CONSIDERATIONS...
**
**********************************************************************
**
** #INCLUDED BY...
**
** OTHER HEADER FILES #INCLUDED...
**
** ACRONYMS...    API     APPLICATION PROGRAMMING INTERFACE
**
**********************************************************************
*/
#ifndef _ta0eq__
#define _ta0eq__

#include <tpf/tpfeq.h>
#include <tpf/tpfapi.h>
#include <tpf/tpfglbl.h>
#include <c_sd0rv.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <tpf/tpftape.h>

#define _issdrv    0x01800481  //gcc

struct TA0EQ1
      {

  /*
  **   Inter-segment definition usage of EBX000-103
  */

  /*
       Values of given options
  */
  unsigned int     cntdef;   /* COUNT= value                         */
  unsigned char     actcmd;   /* Indicator for command specified      */
  unsigned char     blkdef;   /* BLOCK= value                         */
  unsigned char     chartape[4];/* Character array holding tape name  */
  unsigned char     opendef;  /* OPEN= value                          */
  unsigned char     closedef; /* CLOSE= value                         */
  unsigned char     msgdef;   /* MSG= value                           */
  unsigned char     buffdef;  /* BUFFERED/NOBUFFERED value            */
  unsigned char     filldef;  /* FILLCHAR= value                      */
  unsigned char     opcdef;   /* OPCODE= value                        */
  unsigned char     autodef;  /* AUTO/NOAUTO value                    */
  unsigned char     cycledef; /* CYCLE/NOCYCLE value                  */

  /*
       Indicators for above options
  */

  struct                           /* Option flag fields              */
  {
    unsigned valoptn:1;            /* Option is valid for this cmd    */
    unsigned valspec:1;            /* Option specified on cmd line    */
    unsigned valimpl:1;            /* Option value implied by cmd     */
    unsigned        :1;            /* Reserved for future use         */
    unsigned        :1;            /* Reserved for future use         */
    unsigned        :1;            /* Reserved for future use         */
    unsigned        :1;            /* Reserved for future use         */
    unsigned        :1;            /* Reserved for future use         */
  } __attribute__ ((packed)) blkind, tapind, openind, closeind,
    msgind, buffind, fillind, cntind,
    opcind, autoind, cycleind;

  /*
       Other indicators
  */
  unsigned char     tapefunc; /* Function to be performed by QXTK     */
  unsigned char     tapeglob; /* Global area 2 index                  */
  unsigned char     stopopt;  /* Options on the STOP command          */

  unsigned int     ta0work1;         /* General purpose work area 1  */
  unsigned int     ta0work2;         /* General purpose work area 2  */
           double   ta0work3;         /* General purpose work area 3  */
           double   ta0work4;         /* General purpose work area 4  */
      } __attribute__ ((packed));  /* End of structure TA0EQ1 */

  enum command_indicators   {
                            /* Function Test commands follow          */
                              bkspcmd=0x01, dumpcmd=0x02, inputcmd=0x03,
                              outptcmd=0x04, rewcmd=0x05, synccmd=0x06,
                              querycmd=0x07, writecmd=0x08, ccwcmd=0x09,
                              cpswcmd=0x0a, verfycmd=0x0b, abortcmd=0x0c,

                            /* System Test commands follow            */
                              startcmd=0x0d, stopcmd=0x0e, setupcmd=0x0f,
                              statcmd=0x10, pausecmd=0x11, rstrtcmd=0x12,
                              cntrlcmd=0x13};

  enum     opendef_values  { opena = 'A', openn = 'N', openi = 'I',
                             openo = 'O'};
  enum     closedef_values { closee = 'E', closer = 'R',
                             closey = 'Y'};
  enum     msgdef_values   { msgemsg_bit = 0x80,
                             msgimsg_bit = 0x40,
                             msgparm_bit = 0x20,
                             msgwtp_bit  = 0x10,
                             msgemsg     = (msgemsg_bit),
                             msgimsg     = (msgimsg_bit | msgemsg),
                             msgparm     = (msgparm_bit | msgimsg),
                             msgwtp      = (msgwtp_bit | msgimsg),
                             msgall      = (msgparm | msgwtp)};
  enum     buffdef_values  { buffyes = 'Y', buffno = 'N'};
  enum     opcdef_values   { opcwrt = 0x01, opcrdf = 0x02,
                             opcnop = 0x03, opcsns = 0x04,
                             opcrew = 0x07, opcrun = 0x0f,
                             opcrdb = 0x0c, opcrbl = 0x24,
                             opcerg = 0x13, opcwtm = 0x1f,
                             opcbid = 0x22, opcbsb = 0x27,
                             opcbsf = 0x2f, opcfsb = 0x37,
                             opcfsf = 0x3f, opcsyn = 0x43,
                             opclbl = 0x4f, opcdd0 = 0xc3,
                             opcdd2 = 0xcb, opcdd1 = 0xd3,
                             opcsid = 0xe4};
  enum     autodef_values  { autoyes = 'Y', autono = 'N'};
  enum     cycledef_values { cycleyes = 'Y', cycleno = 'N'};

  enum     tapefunc_values { funcopen = 1, funcclos, funccore,
                             funchlpf, funchlps,
                             func_loc, func_set, func_end,
                             func_ege};
  enum     tapefunc_returns { globOK = 0, globFULL, globDUPL,
                              globNULL};
  enum     stopopt_values  { stopBP = 'Y', stopNOBP = 'N'};

struct  DCTBPK             /*  Parameter list from BPKDC processing   */
  {
  int              bpkocnt;     /* Count of parameters processed     */
  __chptr32_t  bpkomsg; /* Pointer to command text           */
  __chptr32_t  bpkopm1; /* Parameter pointer                 */
  __chptr32_t  bpkopm2; /* Parameter pointer                 */
  __chptr32_t  bpkopm3; /* Parameter pointer                 */
  __chptr32_t  bpkopm4; /* Parameter pointer                 */
  __chptr32_t  bpkopm5; /* Parameter pointer                 */
  __chptr32_t  bpkopm6; /* Parameter pointer                 */
  __chptr32_t  bpkopm7; /* Parameter pointer                 */
  __chptr32_t  bpkopm8; /* Parameter pointer                 */
  __chptr32_t  bpkopm9; /* Parameter pointer                 */
  __chptr32_t  bpkopm10;/* Parameter pointer                 */
  __chptr32_t  bpkopm11;/* Parameter pointer                 */
  __chptr32_t  bpkopm12;/* Parameter pointer                 */
  __chptr32_t  bpkopm13;/* Parameter pointer                 */
  __chptr32_t  bpkopm14;/* Parameter pointer                 */
  __chptr32_t  bpkopm15;/* Parameter pointer                 */
  };
#define QTP1_tapename bpkopm2    /* Command parser parm plist entry   */
#define QTP1_bypass   bpkopm3    /* Command parser parm plist entry   */

#define QTP1_sysrta   bpkopm2    /* Command parameter plist entry     */
#define QTP1_sysmsg   bpkopm3    /* Command parameter plist entry     */
#define QTP1_syscret  bpkopm4    /* Command parameter plist entry     */
#define QTP1_sysminc  bpkopm5    /* Command parameter plist entry     */
#define QTP1_sysmaxc  bpkopm6    /* Command parameter plist entry     */

struct  Stage_List         /*  System test stage table definition */
  {
  unsigned char     stage_next; /* Index of next stage to be executed */
  unsigned char     stage_cmd ; /* Command to issue at this stage     */
  unsigned char     stage_opt1; /* Current stage cmd opts indicator 1 */
  unsigned char     stage_opt2; /* Current stage cmd opts indicator 1 */
  };

struct  TAPRCD
  {

  /*
  **   Definition of the tape record to be read/written
  */

#define    verify_recid  "TD"
#define    l_taprecid 2
  unsigned char     taprecid[l_taprecid];

#define    header_L0  "127  BLK"
#define    header_L1  "381  BLK"
#define    header_L2  "1055 BLK"
#define    header_L4  "4K   BLK"
#define    header_VA  "VAR  LEN"
#define    l_taphdr   8
  unsigned char     taphdr[l_taphdr];

#define    l_taprecn  9
  unsigned char     taprecn[l_taprecn];

#define    trailer  "*END OF RECORD*"
#define    l_trailer 15

#define    record_overhead  (l_taprecid+l_taphdr+l_taprecn+l_trailer)
#define    l_tapfill  4095-record_overhead

  unsigned char     tapfill[l_tapfill];

  };

#define tape_lethal 0x08    /* @SWITCH mask for TAPE driver          */

struct  GLOBTD1
  {

  /*
  **   Definition of the tape driver status area in global area
  */

                              /* Driver execution status              */
  unsigned _td1actv:1;        /* -- Driver is active                  */
  unsigned _td1pauz:1;        /* -- Driver is paused                  */
  unsigned _td1stop:1;        /* -- Driver stop in progress           */
  unsigned _td1wrta:1;        /* -- Include writes to the RTA tape    */
  unsigned _td1setu:1;        /* -- SETUP command started/in progress */
  unsigned         :1;        /* -- RESERVED FOR FUTURE USE           */
  unsigned         :1;        /* -- RESERVED FOR FUTURE USE           */
  unsigned         :1;        /* -- RESERVED FOR FUTURE USE           */

  unsigned char     _td1crtt; /* CRETC time between stages            */
  unsigned char     _td1msgs; /* SYSTEM TEST interface msg control    */
  unsigned char     _td1res;  /* Reserved byte                        */

  unsigned short    _td1maxc; /* Maximum random count value           */
#define MIN_MINCOUNT  1
#define MAX_MINCOUNT  32767

  unsigned short    _td1minc; /* Minimum random count value           */
#define MIN_MAXCOUNT  1
#define MAX_MAXCOUNT  32767
  } __attribute__ ((packed));

struct  GLOBTD2
  {

  /*
  **   Definition of the individual tape status fields
  */

  unsigned char     _td2tape[4]; /* Tape name of this entry           */
                              /* Tape status flags                    */
  unsigned _td2actv:1;        /* -- Tape is active                    */
  unsigned _td2stop:1;        /* -- Tape is stopping                  */
  unsigned _td2uabt:1;        /* -- User requested ABORT the tape cmd */
  unsigned _td2clos:1;        /* -- Command with CLOSE=YES issued     */
  unsigned _td2open:1;        /* -- Command with OPEN=OUTPUT issued   */
  unsigned         :1;        /* -- RESERVED FOR FUTURE USE           */
  unsigned         :1;        /* -- RESERVED FOR FUTURE USE           */
  unsigned         :1;        /* -- RESERVED FOR FUTURE USE           */

  unsigned char     _td2stcs; /* System test command issued           */
  unsigned char     _td2ftcs; /* Function test command started        */
  unsigned char     _td2ftcc; /* Function test command completed      */

  unsigned short    _td2iter; /* Tape entry iteration count           */
  unsigned short    _td2ecnt; /* Tape entry operation error count     */

  unsigned int     _td2cmds; /* Tape entry command count             */

  } __attribute__ ((packed));

#define    command_text_array "*None*", "Backspace", "Dump",         \
                              "Input", "Output", "Rewind", "Sync",   \
                              "Query", "Write", "CCW", "Switch",     \
                              "Verify", "Abort"

//gcc #pragma map(Utility, "QXTK")
#define  Utility  QXTK

  void     Utility(struct TPF_regs *);

#define utility(func)                                        \
        ta0eq1_ptr->tapefunc = func ,                        \
        Utility(&regs)

//gcc #pragma map(printmsg, "QTP4")
#define  printmsg  QTP4

#define full_msg FALSE      /* message is not chained                */
#define partial_msg TRUE    /* message should be chained to next one */

#define prefixed TRUE       /* message should be prefixed            */
#define noprefix FALSE      /* message should not be prefixed        */

  void     printmsg(unsigned char,          /* EOM flag   (0/1)      */
                    unsigned char,          /* Prefix flag (1/0)     */
                    unsigned short,         /* Message number        */
                    unsigned char,          /* Message letter        */
                    char *, ...);           /* SPRINTF data          */

/*  Define a macro to make testing of the message setting
    easier to handle.
*/

#define test_msg_setting(msgcntrl)                                \
         (ta0eq1_ptr->msgdef & msgcntrl##_bit)                    \

/*  Define macros to make testing for valid command options
    easier to handle.
*/

#define Option_is_valid(option_name)                              \
         (option_name##.valoptn || option_name##.valimpl)

#define Option_was_specified(option_name)                         \
         (option_name##.valspec)

#endif
