/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2003, 2006                               */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/*
**********************************************************************
**
** FUNCTION NAME.....QTP1
**    Load Module Name:  QTP1
** PURPOSE...........PERFORM FUNCTIONS FOR THE TAPE DRIVER
** FUNCTION TYPE.....TPF
** DOCUMENT NAME...  TAPE31 UGUIDE ON TDRIVERS DISK
** LEVEL...........  VERSION 1  MODIFICATION 0
**
** INPUT...... ALL INPUT CONTAINED IN ECB MAPPED BY C_TA0EQ
**
** OUTPUT..... NONE
**
** DESCRIPTION..... THIS SEGMENT HANDLES PROCESSING FOR SYSTEM
**                  TEST INTERFACE COMMANDS.  IT ACTS AS A FUNNEL
**                  TO OTHER SEGMENTS, AND HANDLES THE FOLLOWING
**                  COMMANDS DIRECTLY:
**
**                     CONTROL
**
**********************************************************************
**
** DEPENDENCIES... SINCE THIS SEGMENT IS NOW PART OF A DLM, IF AN
**                 EXTERNAL CALL IS ADDED FOR WHICH A STUB DOES
**                 NOT EXIST, A STUB WILL NEED TO BE CREATED.
**                                                                     0
**                 IF THIS SEGMENT IS UPDATED TO CONTAIN WRITABLE
**                 STATIC, IT THEN MUST BE COMPILED WITH THE RENT
**                 OPTION.
**
**
** RESTRICTIONS... NONE
**
** RELATED SEGMENTS...   QXTG
**                       QXTI
**                       QXTJ
**                       QTP9
**                       QTP1A
**
**********************************************************************
**
** EXTERNAL REFERENCES...
**
** HEADER FILES #INCLUDED...
**
** ACRONYMS...    API     APPLICATION PROGRAMMING INTERFACE
**
**               TPFGLBL  TPF usage of global areas
**               C_GLOBZ  TPF usage of global areas
**               C_SD0RV  Driver usage of global area 1 & 2
**               TPFTAPE  TPF Tape functions
**               C_TA0EQ  Tape driver mapping of it's use of
**                        the global area
**
***********************************************************************
** CHANGE ACTIVITY...
**
**        UPDATE BY          PLOG #   DATE   DESCRIPTION
**        ---------          -------  ------ ------------
**        KENT               TD00346  022891 TEP PROJECT IMPROVEMENTS
**        KENT               TD00362  060591 REMOVE SETUP RESTRICTIONS
**        MKENT              TD00442  072393 FIX SUBROUTINE INTERFACE
**        TCSSV              TD00652  021296 Migration to ISO-C
**                                           (LYNNEK)
**
************************************************************************/
#include <c_ta0eq64.h>

static  void  Tap_parm(__chptr32_t,       /* Pointer to parameter   */     //f7964
                       size_t,            /* Parameter value length */
                       unsigned char *);  /* Pointer to result      */

int QTP1A();       //gcc

int QTP1()                                                       /*TD00619*/
{
 struct TA0EQ1   *ta0eq1_ptr = (struct TA0EQ1 *)
                               &(ecbptr()->ebx000);
 struct DCTBPK   *bpkdc_ptr = 0;
 struct SD0RV    *sd0rv_ptr;
 struct GLOBTD1  *globtd1_ptr;

 struct {
          char *        text;
          unsigned char indicator;
        } msg_option[] = { {"EMSG", msgemsg},
                           {"IMSG", msgimsg},
                           {"PARMLIST", msgparm},
                           {"WTOPC", msgwtp},
                           {"ALL", msgall}     };
 char * msg_control_text;
 int    msg_option_index;

 if ( (sd0rv_ptr = (struct SD0RV*) (long) * (int *) glob(_issdrv) ) )   //gcc
 {
    globtd1_ptr = (struct GLOBTD1 *) &(sd0rv_ptr->_SD0TAPE);
    bpkdc_ptr = (struct DCTBPK *) ecbptr()->ce1cr0;
 }
 else if (ta0eq1_ptr->actcmd == setupcmd)
 {
    printmsg(partial_msg,prefixed,33,'W',
             "Warning:  Global area not initialized");
    printmsg(full_msg,prefixed,33,'W',
             "Multiple invocations of the SETUP command not "
             "prevented");
    globtd1_ptr = (struct GLOBTD1 *) NULL;
 }
 else
 {
    printmsg(full_msg,prefixed,33,'E',
             "Command failed due to INVALID GLOBAL ADDRESS");
    exit(0);
 }

 switch(ta0eq1_ptr->actcmd)
 {
    case startcmd:
       Tap_parm(bpkdc_ptr->QTP1_tapename,
                sizeof(ta0eq1_ptr->chartape),
                ta0eq1_ptr->chartape);

       if(!memcmp(ta0eq1_ptr->chartape,"ALL",
                  sizeof(ta0eq1_ptr->chartape)))
       {
          printmsg(full_msg,prefixed,21,'E',
                   "TAPENAME = %s not valid for START command",
                   ta0eq1_ptr->chartape);
          exit(0);
       }

       entrc("QXTJ",NULL);
    break;

    case stopcmd:
       Tap_parm(bpkdc_ptr->QTP1_tapename,
                sizeof(ta0eq1_ptr->chartape),
                ta0eq1_ptr->chartape);

       ta0eq1_ptr->stopopt = stopNOBP;
       if(bpkdc_ptr->QTP1_bypass &&
          (*(bpkdc_ptr->QTP1_bypass +1) == 'Y'))
          ta0eq1_ptr->stopopt = stopBP;

       entrc("QXTJ",NULL);
    break;

    case cntrlcmd:
       if(bpkdc_ptr->QTP1_sysmaxc)
       {
          if((*(int*)bpkdc_ptr->QTP1_sysmaxc >= MIN_MAXCOUNT) &&
             (*(int*)bpkdc_ptr->QTP1_sysmaxc <= MAX_MAXCOUNT))
             globtd1_ptr->_td1maxc = *(int *) bpkdc_ptr->QTP1_sysmaxc;
          else
          {
             printmsg(full_msg,prefixed,31,'E',
                      "MAXCOUNT must be integer value %i to %i",
                      MIN_MAXCOUNT, MAX_MAXCOUNT);
             exit(0);
          }
       }

       if(bpkdc_ptr->QTP1_sysminc)
       {
          if((*(int*)bpkdc_ptr->QTP1_sysminc >= MIN_MINCOUNT) &&
             (*(int*)bpkdc_ptr->QTP1_sysminc <= MAX_MINCOUNT))
          {
             if(*(int*) bpkdc_ptr->QTP1_sysminc >
                globtd1_ptr->_td1maxc)
             {
                printmsg(full_msg,prefixed,31,'E',
                         "MINCOUNT value cannot be larger than "
                         "MAXCOUNT value");
                exit(0);
             }
             else
                globtd1_ptr->_td1minc =
                              *(int *) bpkdc_ptr->QTP1_sysminc;
          }
          else
          {
             printmsg(full_msg,prefixed,31,'E',
                      "MINCOUNT must be integer value %i to %i",
                      MIN_MINCOUNT, MAX_MINCOUNT);
             exit(0);
          }
       }

       if(bpkdc_ptr->QTP1_syscret)
       {
          if(*(int*)bpkdc_ptr->QTP1_syscret > 0)
             globtd1_ptr->_td1crtt = *(int *) bpkdc_ptr->QTP1_syscret;
          else
          {
             printmsg(full_msg,prefixed,30,'E',
                      "CRETC value must be 1-99 seconds");
             exit(0);
          }
       }

       if(bpkdc_ptr->QTP1_sysrta)
          globtd1_ptr->_td1wrta = *(bpkdc_ptr->QTP1_sysrta+1) == 'Y';

       if(bpkdc_ptr->QTP1_sysmsg)
       {
          for(msg_option_index = 0;
              msg_option_index < (sizeof(msg_option) /
                                  sizeof(msg_option[0]));
              msg_option_index++)
          {
             if(!memcmp(bpkdc_ptr->QTP1_sysmsg +1,
                        msg_option[msg_option_index].text,
                        *(bpkdc_ptr->QTP1_sysmsg + 0)))
             {
                globtd1_ptr->_td1msgs =
                             msg_option[msg_option_index].indicator;
                break;
             }
          }
          if(!msg_option[msg_option_index].indicator)
          {
             printmsg(full_msg,prefixed,16,'E',
                      "MSG parameter must be one of "
                      "EMSG, IMSG, PARMLIST, WTOPC or ALL");
             exit(0);
          }
       }

       switch(globtd1_ptr->_td1msgs)
       {
          case msgemsg:
              msg_control_text = "EMSG";
          break;

          case msgimsg:
              msg_control_text = "IMSG";
          break;

          case msgparm:
              msg_control_text = "PARMLIST";
          break;

          case msgwtp:
              msg_control_text = "WTOPC";
          break;

          case msgall:
              msg_control_text = "ALL";
          break;

          default:
              msg_control_text = "Undefined";
       }

       /*  Display the current control settings to the user */
       printmsg(partial_msg,prefixed,93,'I',
                "Tape Driver Control Values:");

       printmsg(partial_msg,prefixed,93,'I',
                "  Write to RTA/RTL = %s",
                globtd1_ptr->_td1wrta ? "ON" : "OFF");

       printmsg(partial_msg,prefixed,93,'I',
                "  CRETC interval = %i seconds",
                globtd1_ptr->_td1crtt);

       printmsg(partial_msg,prefixed,93,'I',
                "  MSG control value = %s",
                msg_control_text);

       printmsg(partial_msg,prefixed,93,'I',
                "  Minimum Count value = %i",
                globtd1_ptr->_td1minc);

       printmsg(partial_msg,prefixed,93,'I',
                "  Maximum Count value = %i",
                globtd1_ptr->_td1maxc);

       printmsg(full_msg,prefixed,93,'I',
                "End of Display");
    break;

    case setupcmd:
       memset(ta0eq1_ptr->chartape,0,sizeof(ta0eq1_ptr->chartape));
       ta0eq1_ptr->tapeglob = 0;
       entrc("QXTI",NULL);
    break;

    case statcmd:
       memset(ta0eq1_ptr->chartape,0,sizeof(ta0eq1_ptr->chartape));
       ta0eq1_ptr->tapeglob = 0;
       QTP1A();                                          /*TD00619*/
    break;

    case pausecmd:
       memset(ta0eq1_ptr->chartape,0,sizeof(ta0eq1_ptr->chartape));
       ta0eq1_ptr->tapeglob = 0;
       entrc("QTP9",NULL);
    break;

    case rstrtcmd:
       memset(ta0eq1_ptr->chartape,0,sizeof(ta0eq1_ptr->chartape));
       ta0eq1_ptr->tapeglob = 0;
       entrc("QTP9",NULL);
    break;

    default:
    printmsg(full_msg,prefixed,5,'E',
             "Logic error in action command code: %X",
             ta0eq1_ptr->actcmd);
 }
 exit(0);
}

static void  Tap_parm(__chptr32_t parmlist_ptr,                            //f7964
                      size_t tape_parm_size,
                      unsigned char *tape_parm_dest)
{

 memcpy(tape_parm_dest,parmlist_ptr + 1, *(parmlist_ptr + 0));

 /* If parameter length is too short, or
      parameter doesn't begin with either 'ALL' or 'Z',
    then abort with an error message.
 */
 if(((*parmlist_ptr + 0) != tape_parm_size -1) ||
    (memcmp(tape_parm_dest,"Z",1) &&
     memcmp(tape_parm_dest,"ALL",tape_parm_size)))
 {
    printmsg(full_msg,prefixed,21,'E',
             "TAPENAME parameter must be 3 character "
             "tape name beginning with Z");
    exit(0);
 }

return;
}
