/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2003, 2005                               */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/*
**********************************************************************
**
** FUNCTION NAME.....QTPA
**    Load Module Name:  QTP1
** PURPOSE...........PERFORM FUNCTIONS FOR THE TAPE DRIVER
** FUNCTION TYPE.....TPF
** DOCUMENT NAME...  TAPE31 UGUIDE ON TDRIVERS DISK
** LEVEL...........  VERSION 1  MODIFICATION 0
**
** INPUT...... ALL INPUT CONTAINED IN ECB MAPPED BY C_TA0EQ
**
** OUTPUT..... NONE
**
** DESCRIPTION..... THIS SEGMENT PERFORMS COMMAND PROCESSING FOR
**                  THE FOLLOWING SYSTEM TEST INTERFACE COMMANDS:
**                      STATUS    SHOW SYSTEST ACTIVITY STATUS FOR
**                                ALL SYSTEST TAPES
**
**********************************************************************
**
**
** DEPENDENCIES... SINCE THIS SEGMENT IS NOW PART OF A DLM, IF AN
**                 EXTERNAL CALL IS ADDED FOR WHICH A STUB DOES
**                 NOT EXIST, A STUB WILL NEED TO BE CREATED.
**                                                                     0
**                 IF THIS SEGMENT IS UPDATED TO CONTAIN WRITABLE
**                 STATIC, IT THEN MUST BE COMPILED WITH THE RENT
**                 OPTION.
**
**                 QXTK MAY REQUIRE MODIFICATION IF THIS PROGRAM
**                 CHANGES HOW IT CALLS IT
**
** RESTRICTIONS... NONE
**
** RELATED SEGMENTS...   QXTG           MESSAGE PARSER
**                       QTP1           COMMAND INITIATOR
**
**********************************************************************
**
** EXTERNAL REFERENCES...
**
** HEADER FILES #INCLUDED...
**
** ACRONYMS...    API     APPLICATION PROGRAMMING INTERFACE
**
**               TPFGLBL  TPF usage of global areas
**               C_GLOBZ  TPF usage of global areas
**               C_SD0RV  Driver usage of global area 1 & 2
**               TPFTAPE  TPF Tape functions
**               C_TA0EQ  Tape driver mapping of it's use of
**                        the global area
**
***********************************************************************
** CHANGE ACTIVITY...
**
**        UPDATE BY          PLOG #   DATE   DESCRIPTION
**        ---------          -------  ------ ------------
**        KENT               TD00301  071390 MSG SETTING ENHANCEMENTS
**        KENT               TD00346  022891 TEP PROJECT IMPROVEMENTS
**        TCSSC              TD00652  021296 Migration to ISO-C
**                                           (LYNNEK)
**
************************************************************************/
#include <c_ta0eq64.h>

int QTP1A()
{
 struct TA0EQ1   *ta0eq1_ptr = (struct TA0EQ1 *)
                               &(ecbptr()->ebx000);
 struct SD0RV    *sd0rv_ptr;
 struct GLOBTD1  *globtd1_ptr = 0;
 struct GLOBTD2  *globtd2_ptr = 0, *loop_ptr;

 char *status_legend = "%-9.9s    %-10.10s    %-15.15s   "
                       "%-8.8s    %-6.6s",
      *status_text   = "%-9.9s    %-10.1i     %-15.15s   "
                       "%-8.1i    %-6.1i";

 char  *cmd_names[]  =  {command_text_array};

 char   *driver_status;

      int   glob_index;

 if( (sd0rv_ptr = (struct SD0RV*) (long) * (int *)(glob(_issdrv)))  )  //gcc
 {
    globtd1_ptr = (struct GLOBTD1*) &(sd0rv_ptr->_SD0TAPE);

    /* Situate ourselves at the first tape entry */
    globtd2_ptr = (struct GLOBTD2*) &(sd0rv_ptr->_SD0TAPG);
 }

 switch (ta0eq1_ptr->actcmd)
 {

    case statcmd:
       if(!globtd1_ptr->_td1actv)
          driver_status = "Stopped";
       else if(globtd1_ptr->_td1pauz)
          driver_status = "Paused";
       else if(globtd1_ptr->_td1stop)
          driver_status = "Stopping";
       else
          driver_status = "Active";

       printmsg(partial_msg,prefixed,94,'I',
                "Driver status: %s \n",driver_status);

       printmsg(partial_msg,noprefix,94,'I',
                status_legend,"Tape Name","Iterations",
                "Current Command","Commands","Errors");

       printmsg(partial_msg,noprefix,94,'I',
                status_legend,"---------","----------",
                "---------------","--------","------");

       for(glob_index = 1 ; glob_index < SD0MAXTAPE; glob_index++)
       {
          loop_ptr = globtd2_ptr + glob_index;
          if(loop_ptr->_td2actv)            /* Only show active tapes */
             printmsg(partial_msg,noprefix,94,'I',
                      status_text,
                      loop_ptr->_td2tape,
                      loop_ptr->_td2iter,
                      cmd_names[loop_ptr->_td2stcs],
                      loop_ptr->_td2cmds,
                      loop_ptr->_td2ecnt);
       }
       printmsg(full_msg,noprefix,94,'I',
                "End of display");
    break;

    default:
       printmsg(full_msg,prefixed,5,'E',
                "Invalid action command code");
 }

 exit(0);
}
