/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2003, 2006                               */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/*
**********************************************************************
**
** FUNCTION NAME.....QTP2
**    Load Module Name:  QTP2
** PURPOSE...........PERFORM FUNCTIONS FOR THE TAPE DRIVER
** FUNCTION TYPE.....TPF
** DOCUMENT NAME...  TAPE31 UGUIDE ON TDRIVERS DISK
** LEVEL...........  VERSION 1  MODIFICATION 0
**
** INPUT...... ALL INPUT CONTAINED IN ECB MAPPED BY C_TA0EQ
**
** OUTPUT..... ECB FIELDS POSSIBLY CHANGED BY DEFAULT VALUES ARE
**             CONTAINED WITHIN STORAGE MAPPED BY C_TA0EQ
**
** DESCRIPTION..... THIS SEGMENT PERFORMS COMMAND PROCESSING FOR
**                  FUNCTION TEST INTERFACE COMMANDS BY SUPPLYING
**                  OPTION DEFAULT VALUES AND OPTIONALLY DISPLAYING
**                  THOSE VALUES BACK TO THE USER
**
**********************************************************************
**
** DEPENDENCIES... SINCE THIS SEGMENT IS NOW PART OF A DLM, IF AN
**                 EXTERNAL CALL IS ADDED FOR WHICH A STUB DOES
**                 NOT EXIST, A STUB WILL NEED TO BE CREATED.
**                                                                     0
**                 IF THIS SEGMENT IS UPDATED TO CONTAIN WRITABLE
**                 STATIC, IT THEN MUST BE COMPILED WITH THE RENT
**                 OPTION.
**
** RESTRICTIONS... NONE
**
** RELATED SEGMENTS...   QXTG           MESSAGE PARSER
**
**********************************************************************
**
** EXTERNAL REFERENCES...
**
** HEADER FILES #INCLUDED...
**
** ACRONYMS...    API     APPLICATION PROGRAMMING INTERFACE
**
**               TPFTAPE  TPF Tape functions
**               C_TA0EQ  Tape driver mapping of it's use of
**                        the global and ECB areas
**
**********************************************************************
** CHANGE ACTIVITY...
**
**        UPDATE BY          PLOG #   DATE   DESCRIPTION
**        ---------          -------  ------ ------------
**        KENT               TD00301  071390 MSG SETTING ENHANCEMENTS
**        KENT               TD00346  022891 TEP PROJECT IMPROVEMENTS
**        KENT               TD00368  070791 FIX RBL OPCODE
**        TCSSV              TD00652  021296 Migration to ISO-C
**                                           (LYNNEK)
**
***********************************************************************/
#include <c_ta0eq64.h>

int QTP2()
{
 struct TA0EQ1   *ta0eq1_ptr = (struct TA0EQ1 *)
                               &(ecbptr()->ebx000);

 char  *command_names[] =  {command_text_array};

 struct cmd_default {
                      int count;
                     char *        tapename;
                     unsigned char blksize;
                     unsigned char openopt;
                     unsigned char closeopt;
                     unsigned char msgopt;
                     unsigned char bufferopt;
                     unsigned char fillopt;
                     unsigned char opcodeopt;
                     unsigned char autoopt;
                     unsigned char cycleopt;
                    };

 struct cmd_default default_options;

 static struct cmd_default defaults[] =  //gcc add braces around initializer
                 {   {0,NULL,0,0,0,               /* Undefined command */
                     0,0,0,0,0,0},

                     {1,"GEN" ,0,opena,closer,    /* BACKSPACE command */
                     msgparm,buffyes,0,0,0,0},

                     {1,NULL,0,0,0,               /* DUMP command      */
                     msgparm,0,0,0,0,0},

                     {1,"GEN" ,L2,openi,closey,   /* INPUT command     */
                     msgparm,buffyes,0x00,0,0,cycleno},

                     {1,"GEN" ,L2,openo,closey,   /* OUTPUT command    */
                     msgparm,buffyes,0x00,0,0,cycleno},

                     {0,"GEN" ,0,opena,closer,    /* REWIND command    */
                     msgparm,buffyes,0,0,0,0},

                     {0,"GEN" ,0,opena,closer,    /* SYNC command      */
                     msgparm,buffyes,0,0,autono,0},

                     {0,"GEN" ,0,0,0,             /* QUERY command     */
                     msgimsg,0,0,0,0,0},

                     {100,"RTA" ,0,0,0,           /* WRITE command     */
                     msgimsg,buffyes,0x00,0,0,cycleno},

                     {32,"GEN" ,0,opena,closer,   /* CCW command       */
                     msgall,buffyes,0x00,opcsns,0,cycleno},

                     {0,"GEN" ,0,0,0,             /* SWITCH command    */
                     msgparm,0,0,0,0,0},

                     {1,"GEN" ,L2,openi,closer,   /* VERIFY command    */
                     msgall,buffyes,0x00,0,0,cycleno},

                     {0,"GEN" ,0,0,0,             /* ABORT command     */
                     msgparm,0,0,0,0,0}
                    };

 char *undefined_option = "******";

 char *tape_option,
      *block_size,
      *msg_control,
      *open_option,
      *close_option,
      *buffer_option,
      *opcode_option,
      *auto_option,
      *cycle_option,
      count_option[9],
      fillchar_option[2];

 default_options = defaults[ta0eq1_ptr->actcmd];

 if (! (ta0eq1_ptr->msgind.valspec))  //gcc Option_was_specified(ta0eq1_ptr->msgind)
    ta0eq1_ptr->msgdef = default_options.msgopt;

 switch (ta0eq1_ptr->msgdef)
 {
    case msgemsg:
       msg_control = "EMSG";
    break;

    case msgimsg:
       msg_control = "IMSG";
    break;

    case msgparm:
       msg_control = "PARMLIST";
    break;

    case msgwtp:
       msg_control = "WTOPC";
    break;

    case msgall:
       msg_control = "ALL";
    break;

    default:
       msg_control =  undefined_option;
 }

 if (ta0eq1_ptr->tapind.valoptn || ta0eq1_ptr->tapind.valimpl)//gcc (Option_is_valid(ta0eq1_ptr->tapind))
 {
    if (! (ta0eq1_ptr->tapind.valspec)) //gcc Option_was_specified(ta0eq1_ptr->tapind))
    {
       memcpy(ta0eq1_ptr->chartape,default_options.tapename,
              sizeof(ta0eq1_ptr->chartape));
    }
    tape_option =(char *) ta0eq1_ptr->chartape;
 }
 else
    tape_option =  undefined_option;

 if (ta0eq1_ptr->openind.valoptn || ta0eq1_ptr->openind.valimpl)//gcc(Option_is_valid(ta0eq1_ptr->openind))
 {
    if (! (ta0eq1_ptr->openind.valspec)) //gcc Option_was_specified(ta0eq1_ptr->openind))
       ta0eq1_ptr->opendef = default_options.openopt;
 }

 switch (ta0eq1_ptr->opendef)
 {
    case opena:
       open_option = "ASSIGN";
    break;

    case openi:
       open_option = "INPUT";
    break;

    case openo:
       open_option = "OUTPUT";
    break;

    case openn:
       open_option = "NO";
    break;

    default:
       open_option =  undefined_option;
 }

 if (ta0eq1_ptr->closeind.valoptn || ta0eq1_ptr->closeind.valimpl)//gcc(Option_is_valid(ta0eq1_ptr->closeind))
 {
    if (! (ta0eq1_ptr->closeind.valspec)) //gcc Option_was_specified(ta0eq1_ptr->closeind))
       ta0eq1_ptr->closedef = default_options.closeopt;
 }

 switch (ta0eq1_ptr->closedef)
 {
    case closey:
       close_option = "YES";
    break;

    case closer:
       close_option = "RESERVE";
    break;

    case closee:
       close_option = "EXIT";
    break;

    default:
       close_option =  undefined_option;
 }

 if (ta0eq1_ptr->blkind.valoptn || ta0eq1_ptr->blkind.valimpl)//gcc(Option_is_valid(ta0eq1_ptr->blkind))
 {
    if (! (ta0eq1_ptr->blkind.valspec)) //gcc Option_was_specified(ta0eq1_ptr->blkind))
       ta0eq1_ptr->blkdef = default_options.blksize;
 }

 switch (ta0eq1_ptr->blkdef)
 {
    case L0:
       block_size = "127";
    break;

    case L1:
       block_size = "381";
    break;

    case L2:
       block_size = "1055";
    break;

    case L4:
       block_size = "4K";
    break;

    default:
       block_size =  undefined_option;
 }

 if (ta0eq1_ptr->buffind.valoptn || ta0eq1_ptr->buffind.valimpl)//gcc(Option_is_valid(ta0eq1_ptr->buffind))
 {
    if (! (ta0eq1_ptr->buffind.valspec)) //gcc Option_was_specified(ta0eq1_ptr->buffind))
       ta0eq1_ptr->buffdef = default_options.bufferopt;
 }

 switch (ta0eq1_ptr->buffdef)
 {
    case buffyes:
       buffer_option = "BUFFERED";
    break;

    case buffno:
       buffer_option = "NOBUFFERED";
    break;

    default:
       buffer_option =  undefined_option;
 }

 if (ta0eq1_ptr->autoind.valoptn || ta0eq1_ptr->autoind.valimpl)//gcc(Option_is_valid(ta0eq1_ptr->autoind))
 {
    if (! (ta0eq1_ptr->autoind.valspec)) //gcc Option_was_specified(ta0eq1_ptr->autoind))
       ta0eq1_ptr->autodef = default_options.autoopt;
 }

 switch (ta0eq1_ptr->autodef)
 {
    case autoyes:
       auto_option = "AUTO";
    break;

    case autono:
       auto_option = "NOAUTO";
    break;

    default:
       auto_option =  undefined_option;
 }

 if (ta0eq1_ptr->cycleind.valoptn || ta0eq1_ptr->cycleind.valimpl)//gcc(Option_is_valid(ta0eq1_ptr->cycleind))
 {
    if (! (ta0eq1_ptr->cycleind.valspec)) //gcc Option_was_specified(ta0eq1_ptr->cycleind))
       ta0eq1_ptr->cycledef = default_options.cycleopt;
 }

 switch (ta0eq1_ptr->cycledef)
 {
    case cycleyes:
       cycle_option = "CYCLE";
    break;

    case cycleno:
       cycle_option = "NOCYCLE";
    break;

    default:
       cycle_option =  undefined_option;
 }

 if (ta0eq1_ptr->fillind.valoptn || ta0eq1_ptr->fillind.valimpl)//gcc(Option_is_valid(ta0eq1_ptr->fillind))
 {
    if (! (ta0eq1_ptr->fillind.valspec)) //gcc Option_was_specified(ta0eq1_ptr->fillind))
       ta0eq1_ptr->filldef = default_options.fillopt;
 }

 sprintf(fillchar_option,
         (ta0eq1_ptr->fillind.valoptn || ta0eq1_ptr->fillind.valimpl) ? "%2.2X" : "**", //gccOption_is_valid(ta0eq1_ptr->fillind)
         ta0eq1_ptr->filldef);

 if (ta0eq1_ptr->cntind.valoptn || ta0eq1_ptr->cntind.valimpl)//gcc(Option_is_valid(ta0eq1_ptr->cntind))
 {
    if (! (ta0eq1_ptr->cntind.valspec)) //gcc Option_was_specified(ta0eq1_ptr->cntind))
       ta0eq1_ptr->cntdef = default_options.count;
 }

 sprintf(count_option,
       (ta0eq1_ptr->cntind.valoptn || ta0eq1_ptr->cntind.valimpl) ? //gcc Option_is_valid(ta0eq1_ptr->cntind)
                         "%-i" : undefined_option,
                                 ta0eq1_ptr->cntdef);

 if (ta0eq1_ptr->opcind.valoptn || ta0eq1_ptr->opcind.valimpl)//gcc (Option_is_valid(ta0eq1_ptr->opcind))
 {
    if (! (ta0eq1_ptr->opcind.valspec)) //gcc Option_was_specified(ta0eq1_ptr->opcind))
       ta0eq1_ptr->opcdef = default_options.opcodeopt;
 }

 switch (ta0eq1_ptr->opcdef)
 {
    case opcwrt:
       opcode_option = "WRT (Write)";
    break;

    case opcrdf:
       opcode_option = "RDF (Read Forward)";
    break;

    case opcsns:
       opcode_option = "SNS (Sense Tape Drive)";
    break;

    case opcrdb:
       opcode_option = "RDB (Read Backward)";
    break;

    case opcbid:
       opcode_option = "BID (Read Block ID)";
    break;

    case opcrbl:
       opcode_option = "RBL (Read Buffered Log)";
    break;

    case opcsyn:
       opcode_option = "SYN (Syncronize)";
    break;

    case opclbl:
       opcode_option = "LBL (Locate Block)";
    break;

    case opcsid:
       opcode_option = "SID (Sense ID)";
    break;

    case opcnop:
       opcode_option = "NOP (No-Operation)";
    break;

    case opcrew:
       opcode_option = "REW (Rewind)";
    break;

    case opcrun:
       opcode_option = "RUN (Rewind Unload)";
    break;

    case opcerg:
       opcode_option = "ERG (Erase Gap)";
    break;

    case opcwtm:
       opcode_option = "WTM (Write Tape Mark)";
    break;

    case opcfsb:
       opcode_option = "FSB (Forward Skip Block)";
    break;

    case opcbsb:
       opcode_option = "BSB (Backward Skip Block)";
    break;

    case opcfsf:
       opcode_option = "FSF (Forward Skip File)";
    break;

    case opcbsf:
       opcode_option = "BSB (Backward Skip File)";
    break;

    case opcdd0:
       opcode_option = "DD0 (System Density 0)";
    break;

    case opcdd1:
       opcode_option = "DD1 (System Density 1)";
    break;

    case opcdd2:
       opcode_option = "DD2 (System Density 2)";
    break;

    default:
       opcode_option =  undefined_option;
 }


 if(test_msg_setting(msgparm))
 {
    printmsg(partial_msg,prefixed,70,'I',
             "Tape driver option values used:  "
             "Command = %s",
             command_names[ta0eq1_ptr->actcmd]);

    printmsg(partial_msg,noprefix,70,'I',
             "  TAPE Name - %s,"
             "  AUTO - %s,"
             "  BLOCK Size - %s",
             tape_option, auto_option, block_size);

    printmsg(partial_msg,noprefix,70,'I',
             "  BUFFER Mode - %s,"
             "  COUNT - %s,"
             "  CYCLE - %s",
             buffer_option, count_option, cycle_option);

    printmsg(partial_msg,noprefix,70,'I',
             "  FILLCHAR - %s,"
             "  OPEN - %s,"
             "  CLOSE - %s",
             fillchar_option, open_option, close_option);

    printmsg(full_msg,noprefix,70,'I',
             "  MSG Control - %s,"
             "  OPCODE - %s",
             msg_control, opcode_option);
 }
 return 0;
}
