/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2003, 2008                               */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/*
**********************************************************************
**
** FUNCTION NAME.....QTP3
**    Load Module Name:  QTP3
** PURPOSE...........PERFORM FUNCTIONS FOR THE TAPE DRIVER
** FUNCTION TYPE.....TPF
** DOCUMENT NAME...  TAPE31 UGUIDE ON TDRIVERS DISK
** LEVEL...........  VERSION 1  MODIFICATION 0
**
** INPUT...... ALL INPUT CONTAINED IN ECB MAPPED BY C_TA0EQ
**
** OUTPUT..... ECB FIELDS POSSIBLY CHANGED BY DEFAULT VALUES ARE
**             CONTAINED WITHIN STORAGE MAPPED BY C_TA0EQ
**
** DESCRIPTION..... THIS SEGMENT CONSTRUCTS FUNCTION TEST COMMAND
**                  STRINGS TO BE ISSUED FOR SYSTEM TEST ACTIVITY.
**
**********************************************************************
**
** DEPENDENCIES... SINCE THIS SEGMENT IS NOW PART OF A DLM, IF AN
**                 EXTERNAL CALL IS ADDED FOR WHICH A STUB DOES
**                 NOT EXIST, A STUB WILL NEED TO BE CREATED.
**
**                 IF THIS SEGMENT IS UPDATED TO CONTAIN WRITABLE
**                 STATIC, IT THEN MUST BE COMPILED WITH THE RENT
**                 OPTION.
**
**
** RESTRICTIONS... NONE
**
** RELATED SEGMENTS...   QXTJ           SYSTEM TEST ACTIVITY INITIATOR
**
**********************************************************************
**
** EXTERNAL REFERENCES...
**
** HEADER FILES #INCLUDED...
**
** ACRONYMS...    API     APPLICATION PROGRAMMING INTERFACE
**
**               MI0MI    Input message DSECT
**               TPFTAPE  TPF Tape functions
**               C_TA0EQ  Tape driver mapping of it's use of
**                        the global and ECB areas
**               STDARG   Used to process variable length parm lists
**
**********************************************************************
** CHANGE ACTIVITY...
**
**        UPDATE BY          PLOG #   DATE   DESCRIPTION
**        ---------          -------  ------ ------------
**        KENT               TD00346  022891 TEP PROJECT IMPROVEMENTS
**        MKENT              TD00442  072393 FIX SUBROUTINE INTERFACE
**        TCSSV              TD00652  021296 Migration to ISO-C
**                                           (LYNNEK)
**        YMLI               14943f_2 041007 'ZTEST TA64 START
**                               tape_label' should exercise TA64 test
**
***********************************************************************/
#include <c_ta0eq64.h>
#include <tpf/c_mi0mi.h>
#include <stdarg.h>
#include <ctype.h>

static void  Add_Opt(char *,                   /* buffer address     */
                     char *,                   /* SPRINTF parameters */
                     ...);

int QTP3(struct Stage_List * stage_ptr)
{
 struct TA0EQ1   *ta0eq1_ptr = (struct TA0EQ1 *)
                               &(ecbptr()->ebx000);

 struct mi0mi    *mi0mi_ptr;    /* Pointer to message block           */
                                /* under construction                 */

 struct SD0RV    *sd0rv_ptr;
 struct GLOBTD1  *globtd1_ptr;
 struct GLOBTD2  *globtd2_ptr;

 struct CRETC_area {            /* CRETC interface area - set by QXTJ */
                     unsigned char tapeindex;
                     unsigned char current_stage;
                     short         operation_count;
                   } *CRETC_data = (struct CRETC_area *)
                                    &(ecbptr()->ebw000);

 struct Cmd_Options {            /* Indicators to show valid cmd opts */
                      unsigned   valoptn:1;  /* Option valid for cmd  */
                      unsigned   valspec:1;  /* Option value spec'd   */
                      unsigned   valimpl:1;  /* Option value implied  */
                      unsigned          :1;  /* Reserved              */
                      unsigned          :1;  /* Reserved              */
                      unsigned          :1;  /* Reserved              */
                      unsigned          :1;  /* Reserved              */
                      unsigned          :1;  /* Reserved              */
                    } auto_option,
                      block_option,
                      close_option,
                      count_option,
                      cycle_option,
                      fillchar_option,
                      opcode_option,
                      open_option,
                      tape_option;

 char  *command_names[] =  {command_text_array};

 char  *undefined_option = "*XXX*";

 char  *tape_parm,
       *msg_parm,
       *open_parm,
       *close_parm,
       *block_parm,
       *opcode_parm,
       *cycle_parm;

 int  count_value, count_minimum, count_maximum;

 /*  Initialize global pointer fields  */
 sd0rv_ptr = (struct SD0RV*) (long) * (int *) glob(_issdrv); //gcc

 globtd1_ptr = (struct GLOBTD1*) &(sd0rv_ptr->_SD0TAPE);
 globtd2_ptr = (struct GLOBTD2*) &(sd0rv_ptr->_SD0TAPG) +
               ta0eq1_ptr->tapeglob;

 /*  Initialize the message block by doing the following:
     1)  Release block if one exists on D0
     1)  Get a core block to build the message into
     2)  Initialize fields:
           - record id = 'MI'
           - record code check = 0
           - control byte = 0
           - filing program = 0
           - forward chain = 0
           - backward chain = 0
           - message size = 0
     3)  Initialize EBROUT field
 */
 crusa(1,D0);
 mi0mi_ptr = (struct mi0mi *) getcc(D0, GETCC_TYPE, L4);

 memcpy(&mi0mi_ptr->mi0bid,"MI",sizeof(mi0mi_ptr->mi0bid));

 memset(&mi0mi_ptr->mi0chk,0,sizeof(mi0mi_ptr->mi0chk));
 memset(&mi0mi_ptr->mi0ctl,0,sizeof(mi0mi_ptr->mi0ctl));
 memset(&mi0mi_ptr->mi0pgm,0,sizeof(mi0mi_ptr->mi0pgm));
 memset(&mi0mi_ptr->mi0fch,0,sizeof(mi0mi_ptr->mi0fch));
 memset(&mi0mi_ptr->mi0bch,0,sizeof(mi0mi_ptr->mi0bch));

 mi0mi_ptr->mi0cct = 0;

 mi0mi_ptr->mi0adr = ecbptr()->ebrout;       /* Copy current EBROUT  */

 mi0mi_ptr->mi0acc[0] = 0;                   /* Ensure empty string  */

 /*  Indicate the system test command being generated */
 globtd2_ptr->_td2stcs = stage_ptr->stage_cmd;

 /*  Start building the command string */
 Add_Opt(mi0mi_ptr->mi0acc,"ZTEST 0 TA64 %s",
         command_names[stage_ptr->stage_cmd]);

 /* Initialize indicators to all FALSE */
 memset(&auto_option,FALSE,sizeof(auto_option));
 memset(&block_option,FALSE,sizeof(block_option));
 memset(&close_option,FALSE,sizeof(close_option));
 memset(&count_option,FALSE,sizeof(count_option));
 memset(&cycle_option,FALSE,sizeof(cycle_option));
 memset(&fillchar_option,FALSE,sizeof(fillchar_option));
 memset(&opcode_option,FALSE,sizeof(opcode_option));
 memset(&open_option,FALSE,sizeof(open_option));
 memset(&tape_option,FALSE,sizeof(tape_option));

 /* Set indicators per command */
 switch (stage_ptr->stage_cmd)
 {
    case ccwcmd:
       count_option.valoptn = TRUE;
       tape_option.valoptn = TRUE;
       open_option.valoptn = TRUE;
       fillchar_option.valoptn = TRUE;
       opcode_option.valoptn = TRUE;
    break;
    case inputcmd:
       count_option.valoptn = TRUE;
       tape_option.valoptn = TRUE;
       open_option.valoptn = TRUE;
       block_option.valoptn = TRUE;
    break;
    case outptcmd:
       count_option.valoptn = TRUE;
       tape_option.valoptn = TRUE;
       open_option.valoptn = TRUE;
       block_option.valoptn = TRUE;
       fillchar_option.valoptn = TRUE;
    break;
    case rewcmd:
       tape_option.valoptn = TRUE;
       open_option.valoptn = TRUE;
    break;
    case synccmd:
       tape_option.valoptn = TRUE;
       open_option.valoptn = TRUE;
    break;
    case verfycmd:
       count_option.valoptn = TRUE;
       tape_option.valoptn = TRUE;
       open_option.valoptn = TRUE;
       block_option.valoptn = TRUE;
       fillchar_option.valoptn = TRUE;
    break;
    case writecmd:
       count_option.valoptn = TRUE;
       tape_option.valoptn = TRUE;
       fillchar_option.valoptn = TRUE;
    break;
 }

 close_option = open_option;
 cycle_option = fillchar_option;

 switch (globtd1_ptr->_td1msgs)
 {
    case msgemsg:
       msg_parm = "EMSG";
    break;

    case msgimsg:
       msg_parm = "IMSG";
    break;

    case msgparm:
       msg_parm = "PARM";
    break;

    case msgwtp:
       msg_parm = "WTOPC";
    break;

    case msgall:
       msg_parm = "ALL";
    break;

    default:
       msg_parm = undefined_option;
 }
 Add_Opt(mi0mi_ptr->mi0acc," MSG=%s",msg_parm);

 if (tape_option.valoptn || tape_option.valimpl)  //gcc (Option_is_valid(tape_option))
 {
    tape_parm =(char *) ta0eq1_ptr->chartape;
    Add_Opt(mi0mi_ptr->mi0acc," TAPE=%s",tape_parm);
 }

 if (open_option.valoptn || open_option.valimpl)   //gcc (Option_is_valid(open_option))
 {
    /* If tape isn't open yet, specify OPEN=OUTPUT to open it */
    if(globtd2_ptr->_td2open)
       open_parm = "AS";
    else
    {
       open_parm = "OUT";
       globtd2_ptr->_td2open = TRUE;
    }

    Add_Opt(mi0mi_ptr->mi0acc," OPEN=%s",open_parm);
 }

 if (close_option.valoptn || close_option.valimpl) //gcc (Option_is_valid(close_option))
 {
    /* If driver stopping, specify CLOSE=YES to close & dismount */
    if(globtd2_ptr->_td2stop)
    {
          close_parm = "YES";
          globtd2_ptr->_td2clos = TRUE;
    }
    else
          close_parm = "RES";

    Add_Opt(mi0mi_ptr->mi0acc," CLOSE=%s",close_parm);
 }

 if (block_option.valoptn || block_option.valimpl) //gcc (Option_is_valid(block_option))
 {
    switch (stage_ptr->stage_opt1)
    {
       case L0:
          block_parm = "127";
       break;

       case L1:
          block_parm = "381";
       break;

       case L2:
          block_parm = "1055";
       break;

       case L4:
          block_parm = "4K";
       break;

       default:
          block_parm = undefined_option;
    }
    Add_Opt(mi0mi_ptr->mi0acc," BLOCK=%s",block_parm);
 }

 if (cycle_option.valoptn || cycle_option.valimpl) //gcc (Option_is_valid(cycle_option))
 {
    switch (stage_ptr->stage_opt2)
    {
       case cycleyes:
          cycle_parm = " CYCLE";
       break;

       case cycleno:
          cycle_parm = " NOCYCLE";
       break;

       default:
          cycle_parm = undefined_option;
    }
    Add_Opt(mi0mi_ptr->mi0acc,cycle_parm);
 }

 if (fillchar_option.valoptn || fillchar_option.valimpl) //gcc(Option_is_valid(fillchar_option))
    Add_Opt(mi0mi_ptr->mi0acc," FILL=ED");

 if (count_option.valoptn || count_option.valimpl) //gcc (Option_is_valid(count_option))
 {
    /* Don't reset limits for input commands */
    if ((stage_ptr->stage_cmd == verfycmd) ||
        (stage_ptr->stage_cmd == inputcmd) ||
        ((stage_ptr->stage_cmd == ccwcmd) &&
         (stage_ptr->stage_opt1 == opcrdf)))
       count_value = CRETC_data->operation_count;

    else
    {
       /* Use lower limits for CCW and WRITE operations */
       if((stage_ptr->stage_cmd == ccwcmd) ||
          (stage_ptr->stage_cmd == writecmd))
       {
          count_maximum = l_tapfill + record_overhead;
          count_minimum = record_overhead;
       }
       else
       {
          count_maximum = globtd1_ptr->_td1maxc;
          count_minimum = globtd1_ptr->_td1minc;
       }

       srand((unsigned int)time(NULL));
       count_value = count_minimum + (rand() %
                     (count_maximum - count_minimum +1));
       CRETC_data->operation_count = count_value;
    }
    Add_Opt(mi0mi_ptr->mi0acc," COUNT=%-i",count_value);
 }

 if (opcode_option.valoptn || opcode_option.valimpl) //gcc (Option_is_valid(opcode_option))
 {
    switch (stage_ptr->stage_opt1)
    {
       case opcrdf:
          opcode_parm = "RDF";
       break;

       case opcwrt:
          opcode_parm = "WRT";
       break;

       case opcsns:
          opcode_parm = "SNS";
       break;

       case opcnop:
          opcode_parm = "NOP";
       break;

       default:
          opcode_parm = undefined_option;
    }
    Add_Opt(mi0mi_ptr->mi0acc," OPC=%s",opcode_parm);
 }

 Add_Opt(mi0mi_ptr->mi0acc,"%c",_EOM);

 mi0mi_ptr->mi0cct = strlen(mi0mi_ptr->mi0acc) + 3;

 /* Send the message into the input message processor */
 creec(0, NULL, CVAA,D0,CREEC_IMMEDIATE);

 return 0;
}

static void  Add_Opt(char * buffer,             /* buffer address     */
                     char * fmt_string,         /* SPRINTF parameters */
                     ...)
{
 va_list arg_ptr;
 char    *starting_location;
 int     bytes_processed, ii;

 starting_location = buffer + strlen(buffer);

 va_start(arg_ptr,fmt_string);
 bytes_processed = vsprintf(starting_location,
                            fmt_string,arg_ptr);
 va_end(arg_ptr);

 for(ii = 0; ii < bytes_processed; ii++)
    memset(starting_location + ii,
           toupper(*(starting_location + ii)),
           1);

 return;
}
