/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2003, 2006                               */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/*
**********************************************************************
**
** FUNCTION NAME.....QTP4
**    Load Module Name:  QTP4
** PURPOSE...........PERFORM OUTPUT FUNCTIONS FOR THE TAPE DRIVER
** FUNCTION TYPE.....TPF
** DOCUMENT NAME...  TAPE31 UGUIDE ON TDRIVERS DISK
** LEVEL...........  VERSION 1  MODIFICATION 0
**
** INPUT...... ALL INPUT CONTAINED IN ECB MAPPED BY C_TA0EQ
**             PARAMETERS DESCRIBED BELOW
**
** OUTPUT..... NONE
**
** DESCRIPTION..... THIS SEGMENT PERFORMS ONE FUNCTION - PROCESS
**                  AND SEND TO MESSAGES TO DISPLAY AND PRINT
**                  DEVICES FROM TAPE DRIVER SEGMENTS.
**
**********************************************************************
**
** DEPENDENCIES... ALL TAPE DRIVER SEGMENTS CALLING THIS PROGRAM
**                 MAY NEED TO CHANGE IF THIS PROGRAM CHANGES.
**
**                 SINCE THIS SEGMENT IS NOW PART OF A DLM, IF AN
**                 EXTERNAL CALL IS ADDED FOR WHICH A STUB DOES
**                 NOT EXIST, A STUB WILL NEED TO BE CREATED.
**
**                 IF THIS SEGMENT IS UPDATED TO CONTAIN WRITABLE
**                 STATIC, IT THEN MUST BE COMPILED WITH THE RENT
**                 OPTION.
**
**
** RESTRICTIONS... NONE
**
** RELATED SEGMENTS...   QXTG           MESSAGE PARSER
**
**********************************************************************
**
** EXTERNAL REFERENCES...
**
** HEADER FILES #INCLUDED...  (BUT ARE NEVER USED)
**
** ACRONYMS...    API     APPLICATION PROGRAMMING INTERFACE
**
**               TPFGLBL  TPF usage of global areas
**               C_GLOBZ  TPF usage of global areas
**               C_SD0RV  Driver usage of global area 1 & 2
**               TPFTAPE  TPF Tape functions
**               C_TA0EQ  Tape driver mapping of it's use of
**                        the global area
**
*************************************************************************
** CHANGE ACTIVITY...
**
**        UPDATE BY          PLOG #   DATE   DESCRIPTION
**        ---------          -------  ------ ------------
**        KENT               TD00346  022891 TEP PROJECT IMPROVEMENTS
**        TCSSV              TD00652  021296 Migration to ISO-C
**                                           (LYNNEK)
**        YMLI               F14612   081506 The output routing
**        YMLI               D14945   112706 'ZTEST TA64 START tape_label'
**                                         should exercise TA64 test cases
**
**************************************************************************/
#include <c_ta0eq64.h>
#include <stdarg.h>

void QTP4(unsigned char chained_msg,      /*  Message chaining flag  */
          unsigned char prefixed_msg,     /*  Message prefix flag    */
          unsigned short msg_number,      /*  Message number (0-99)  */
          unsigned char msg_letter,       /*  Message letter (I,E)   */
          char *fmtstring,                /*  PRINTF format string   */
          ...)                            /*  Variable argument list */
{
 unsigned char buffer_area[512];          /* PRINTF output area      */
 unsigned char header_buffer[12];         /* WTOPC header data       */
          int  rout_list;                 /* FSC routing list        */

 va_list arg_ptr;

 /*  Call VSPRINTF to format string before transmitting */
 va_start(arg_ptr,fmtstring);
 vsprintf((char *)buffer_area, fmtstring, arg_ptr);
 va_end(arg_ptr);

 /*  Set up standard message header, just in case it's needed */
 wtopc_insert_header(header_buffer,"TA64",
                     msg_number,msg_letter,
                     WTOPC_NO_TIME)

 /* Add terminal to the routing list */
 wtopc_routing_list(rout_list, WTOPC_EBROUT + WTOPC_TAPE)

 /* Finally, call WTOPC to send the message to the user */
 wtopc((char *)buffer_area, rout_list,
       chained_msg ? WTOPC_CHAIN : WTOPC_CHAIN_END,
       prefixed_msg ? (const void *) header_buffer : WTOPC_NO_HEADER);           //gcc

 return;
}
