/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2003, 2006                               */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/*
**********************************************************************
**
** FUNCTION NAME.....QTP5
**    Load Module Name:  QTP5
** PURPOSE...........PERFORM FUNCTIONS FOR THE TAPE DRIVER
** FUNCTION TYPE.....TPF
** DOCUMENT NAME.....TAPE31 UGUIDE ON TDRIVERS DISK
** LEVEL.............VERSION 1  MODIFICATION 0
**
** INPUT.......ALL INPUT CONTAINED IN ECB MAPPED BY C_TA0EQ
**
** OUTPUT......NONE
**
** DESCRIPTION..... THIS SEGMENT PERFORMS THE VERIFY COMMAND.
**    THE USER TAPE IS READ IN A RECORD AT A TIME, AND EACH RECORD
**    IS EXAMINED TO ENSURE THAT EACH PORTION OF THE RECORD CONTAINS
**    EXACTLY WHAT IT SHOULD.  (RECORD ID, RECORD HEADER, RECORD
**    NUMBER, FILL CHARACTERS AND RECORD TRAILER)
**
**********************************************************************
**
**
**  DEPENDENCIES... SINCE THIS SEGMENT IS NOW PART OF A DLM, IF AN
**                  EXTERNAL CALL IS ADDED FOR WHICH A STUB DOES
**                  NOT EXIST, A STUB WILL NEED TO BE CREATED.
**
**                  IF THIS SEGMENT IS UPDATED TO CONTAIN WRITABLE
**                  STATIC, IT THEN MUST BE COMPILED WITH THE RENT
**                  OPTION.
**
**                  QXTK MAY REQUIRE MODIFICATION IF THIS PROGRAM
**                  CHANGES HOW IT CALLS IT
**
** RESTRICTIONS... NONE
**
** RELATED SEGMENTS...   QXTG           MESSAGE PARSER
**                       QTP2           COMMAND PARSING/INFORMATION
**                       QXTH           COMMAND INITIATOR
**                       QXTK           COMMON FUNCTIONS
**
**********************************************************************
**
** EXTERNAL REFERENCES...
**
** HEADER FILES #INCLUDED...
**
** ACRONYMS...    API     APPLICATION PROGRAMMING INTERFACE
**
**               TPFGLBL  TPF usage of global areas
**               C_GLOBZ  TPF usage of global areas
**               C_SD0RV  Driver usage of global area 1 & 2
**               TPFTAPE  TPF Tape functions
**               C_TA0EQ  Tape driver mapping of it's use of
**                        the global area
**
***********************************************************************
** CHANGE ACTIVITY...
**
**        UPDATE BY          PLOG #   DATE   DESCRIPTION
**        ---------          -------  ------ ------------
**        KENT               TD00301  071390 MSG SETTING ENHANCEMENTS
**        KENT               TD00346  022891 TEP PROJECT IMPROVEMENTS
**        TCSSV              TD00652  021296 Migration to ISO-C (LYNNEK)
**
************************************************************************/
#include <c_ta0eq64.h>
#include <tpf/tpfio.h>

int QTP5()
{
 struct TPF_regs  regs;
 struct TA0EQ1   *ta0eq1_ptr = (struct TA0EQ1 *)
                               &(ecbptr()->ebx000);
 struct TAPRCD   *taprcd_ptr;
 struct SD0RV    *sd0rv_ptr;
 struct GLOBTD2  *globtd2_ptr = 0;

 struct snapc_list *snapdump[3];

 int   fill_index,
       original_recnum,
       record_number,
       fill_len;

 char   *verify_header, recnum_scratch[] = "999999999\n";

 unsigned char  verify_error, waitc_error;

 unsigned char  test_fill_character;

 if(ta0eq1_ptr->tapeglob)
 {
    sd0rv_ptr = (struct SD0RV*) (long) * (int *) glob(_issdrv);

    globtd2_ptr = (struct GLOBTD2 *) &(sd0rv_ptr->_SD0TAPG) +
                  ta0eq1_ptr->tapeglob;

    utility(func_set);           /*  Mark global as command starting  */
 }

 switch (ta0eq1_ptr->actcmd)
 {
    case verfycmd:
       utility(funcopen);                  /*  Open the tape  */

       switch (ta0eq1_ptr->blkdef)
       {
          case L0:
             verify_header = header_L0;
             fill_len = 127  - record_overhead;
          break;

          case L1:
             verify_header = header_L1;
             fill_len = 381  - record_overhead;
          break;

          case L2:
             verify_header = header_L2;
             fill_len = 1055 - record_overhead;
          break;

          case L4:
             verify_header = header_L4;
             fill_len = 4095 - record_overhead;
          break;

          default:
             if (test_msg_setting(msgemsg))
             {
                printmsg(full_msg,prefixed,5,'E',
                         "Invalid BLOCK option value");
             }
             exit(0);
          }

       verify_error = FALSE;
       waitc_error = FALSE;
       original_recnum = 0;

       for(record_number = 0;
           record_number != ta0eq1_ptr->cntdef;
           record_number++)
       {
          utility(funccore);               /*  Check core levels */

          /* If we have a valid tape indicator, and the abort bit
             is set, then abort the operation
          */
          if((ta0eq1_ptr->tapeglob) && (globtd2_ptr->_td2uabt))
          {
             if (test_msg_setting(msgimsg))
                printmsg(full_msg,prefixed,58,'I',
                         "User requested abort of operation to"
                         "%s tape",
                         ta0eq1_ptr->chartape);
             globtd2_ptr->_td2uabt = FALSE;
             verify_error = TRUE;
             break;
          }

          tprdc((char *)ta0eq1_ptr->chartape,D1,ta0eq1_ptr->blkdef);

          if (waitc())
          {
             if (ecbptr()->ce1sud[1] & CXSGEE)
             {
                if (test_msg_setting(msgimsg))
                   printmsg(full_msg,prefixed,68,'I',
                            "End of file or unexpected tape "
                            "mark on %s tape",
                            ta0eq1_ptr->chartape);
             }
             else if (ecbptr()->ce1sud[1] & (CXSGSE+CXSGLE))
             {
                if (test_msg_setting(msgemsg))
                   printmsg(full_msg,prefixed,8,'E',
                            "Incorrect length error on %s "
                            "tape",ta0eq1_ptr->chartape);
             }
             else
                if (test_msg_setting(msgemsg))
                   printmsg(full_msg,prefixed,3,'E',
                            "WAITC error on %s tape - check "
                            "count and blksize",
                            ta0eq1_ptr->chartape);
             verify_error = TRUE;
             waitc_error = TRUE;
             break;                           /* Leave this loop */
          }
          taprcd_ptr = ecbptr()->ce1cr1;

          if(original_recnum == 0)
            original_recnum = atol(memcpy(recnum_scratch,
                                   taprcd_ptr->taprecn,l_taprecn));

          /*  Is the record id correct?  */
          if(memcmp(taprcd_ptr->taprecid,
                    verify_recid,
                    l_taprecid))
          {
                if (test_msg_setting(msgemsg))
                   printmsg(full_msg,prefixed,2,'E',
                            "Invalid record id: %.*s"
                            " - dump follows",
                            l_taprecid,
                            taprcd_ptr->taprecid);

             verify_error = TRUE;
             break;                           /* Leave this loop */
          }

          /*  Is the record header correct?  */
          if(memcmp(taprcd_ptr->taphdr,verify_header,l_taphdr))
          {
                if (test_msg_setting(msgemsg))
                   printmsg(full_msg,prefixed,7,'E',
                            "Invalid header: %.*s - dump follows",
                            l_taphdr,
                            taprcd_ptr->taphdr);

             verify_error = TRUE;
             break;                           /* Leave this loop */
          }

          /*  Is the record number correct?  */
          if ((original_recnum - record_number) !=
               atol(memcpy(recnum_scratch,
               taprcd_ptr->taprecn,l_taprecn)))
          {
                if (test_msg_setting(msgemsg))
                   printmsg(full_msg,prefixed,6,'E',
                            "Invalid record number: %.*s"
                            " - dump follows",
                            l_taprecn,
                            taprcd_ptr->taprecn);

             verify_error = TRUE;
             break;                           /* Leave this loop */
          }

          /*  Are the fill characters correct?  */

          test_fill_character = ta0eq1_ptr->filldef;
          for(fill_index = 0; fill_index < fill_len; fill_index++)
          {
             if(taprcd_ptr->tapfill[fill_index]!=test_fill_character)
             {
                verify_error = TRUE;
                if (test_msg_setting(msgemsg))
                   printmsg(full_msg,prefixed,9,'E',
                            "Invalid FILLCHAR data - dump follows");
                break;                           /* Leave this loop */
             }
             if(ta0eq1_ptr->cycledef == cycleyes)
                test_fill_character ++;
          }

          /*  Is the record trailer correct?  */
          if(memcmp(&taprcd_ptr->tapfill[fill_len],
                    trailer,l_trailer))
          {
             if (test_msg_setting(msgemsg))
                printmsg(full_msg,prefixed,7,'E',
                         "Invalid record trailer - dump follows");

             verify_error = TRUE;
             break;                           /* Leave this loop */
          }

          if(verify_error)
             break;                           /* Leave this loop */

          relcc(D1);
       }

       if ((verify_error || waitc_error) && (ta0eq1_ptr->tapeglob))
          globtd2_ptr->_td2ecnt++;

       if(!verify_error)
       {
          if (test_msg_setting(msgimsg))
             printmsg(full_msg,prefixed,66,'I',
                      "Data verified on %s tape: %i of %i "
                      "total records",
                      ta0eq1_ptr->chartape,
                      ta0eq1_ptr->cntdef,
                      original_recnum);
       }
       else if(!waitc_error)
       {
          snapdump[0] = (struct snapc_list *) &ecbptr()->ebw000;
          snapdump[0]->snapc_len = sizeof(struct TA0EQ1);
          snapdump[0]->snapc_name = "ECB Ctrl";
          snapdump[0]->snapc_tag = ta0eq1_ptr;
          snapdump[0]->snapc_indir = SNAPC_NOINDIR;

          snapdump[1] = (struct snapc_list *) &ecbptr()->ebw016;
          snapdump[1]->snapc_len = ecbptr()->ce1cc1;
          snapdump[1]->snapc_name = "Tape Rcd";
          snapdump[1]->snapc_tag = ecbptr()->ce1cr1;
          snapdump[1]->snapc_indir = SNAPC_NOINDIR;

          snapdump[2] = (struct snapc_list *) &ecbptr()->ebw032;
          snapdump[2]->snapc_len = 0;

          snapc(SNAPC_RETURN,                   /* return to nsi     */
                0xC0DE01,                       /* snapc id number   */
                "VERIFY ERROR DUMP",            /* appended message  */
                snapdump,                       /* snapc tag         */
                'T',                            /* snapc prefix      */
                SNAPC_NOREGS,                   /* display regs      */
                SNAPC_ECB,                      /* use ecb ss and ta */
                NULL);                          /* snapc pgm name    */
       }

       utility(funcclos);                  /*  Close the tape  */
    break;

    default:
       if (test_msg_setting(msgemsg))
          printmsg(full_msg,prefixed,5,'E',
                   "Invalid action command code");
 }

 if(ta0eq1_ptr->tapeglob)
    utility(func_end);           /*  Mark global as command complete  */

 if (test_msg_setting(msgimsg))
    printmsg(full_msg,prefixed,67,'I',
             "%s tape operation complete",
             ta0eq1_ptr->chartape);
 exit(0);
}
