/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2003, 2006                               */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/*
**********************************************************************
**
** FUNCTION NAME.....QTP6
**    Load Module Name:  QTP6
** PURPOSE...........PERFORM FUNCTIONS FOR THE TAPE DRIVER
** FUNCTION TYPE.....TPF
** DOCUMENT NAME...  TAPE31 UGUIDE ON TDRIVERS DISK
** LEVEL...........  VERSION 1  MODIFICATION 0
**
** INPUT...... ALL INPUT CONTAINED IN ECB MAPPED BY C_TA0EQ
**
** OUTPUT..... NONE
**
** DESCRIPTION..... THIS SEGMENT PERFORMS THE FOLLOWING FUNCTIONS:
**                      BACKSPACE tape some number of records
**                      REWIND    tape
**                      INPUT     some number of records from
**                                the tape
**
**********************************************************************
**
** DEPENDENCIES... SINCE THIS SEGMENT IS NOW PART OF A DLM, IF AN
**                 EXTERNAL CALL IS ADDED FOR WHICH A STUB DOES
**                 NOT EXIST, A STUB WILL NEED TO BE CREATED.
**
**                 IF THIS SEGMENT IS UPDATED TO CONTAIN WRITABLE
**                 STATIC, IT THEN MUST BE COMPILED WITH THE RENT
**                 OPTION.
**
**                 QXT9 MAY REQUIRE MODIFICATION IF THIS PROGRAM
**                 CHANGES HOW IT CALLS IT
**
** RESTRICTIONS... NONE
**
** RELATED SEGMENTS...   QXTG           MESSAGE PARSER
**                       QTP2           COMMAND PARSING/INFORMATION
**                       QXTH           COMMAND INITIATOR
**                       QXTK           COMMON FUNCTIONS
**
**********************************************************************
**
** EXTERNAL REFERENCES...
**
** HEADER FILES #INCLUDED...
**
** ACRONYMS...    API     APPLICATION PROGRAMMING INTERFACE
**
**               TPFGLBL  TPF usage of global areas
**               C_GLOBZ  TPF usage of global areas
**               C_SD0RV  Driver usage of global area 1 & 2
**               TPFTAPE  TPF Tape functions
**               C_TA0EQ  Tape driver mapping of it's use of
**                        the global area
**
***********************************************************************
** CHANGE ACTIVITY...
**
**        UPDATE BY          PLOG #   DATE   DESCRIPTION
**        ---------          -------  ------ ------------
**        KENT               TD00301  071390 MSG SETTING ENHANCEMENTS
**        KENT               TD00346  022891 TEP PROJECT IMPROVEMENTS
**        TCSSV              TD00652  021296 Migration to ISO-C
**                                           (LYNNEK)
**                           d14351   04212006  Added a check on 
**                                              backspace count for
**                                              tbspc( ).
**                                              If the count is larger
**                                              than 65535, the driver
**                                              will send an error message
**                                              and exit.        
**
************************************************************************/
#include <c_ta0eq64.h>
#include <tpf/tpfio.h>

int QTP6()
{

 struct TPF_regs  regs;
 struct TA0EQ1   *ta0eq1_ptr = (struct TA0EQ1 *)
                               &(ecbptr()->ebx000);
 struct TAPRCD   *taprcd_ptr;
 struct SD0RV    *sd0rv_ptr;
 struct GLOBTD2  *globtd2_ptr = 0;

      int   record_number;

 if(ta0eq1_ptr->tapeglob)
 {
    sd0rv_ptr = (struct SD0RV*) (long) * (int *) glob(_issdrv); //gcc

    globtd2_ptr = (struct GLOBTD2 *) &(sd0rv_ptr->_SD0TAPG) +
                  ta0eq1_ptr->tapeglob;

    utility(func_set);           /*  Mark global as command starting  */
 }

 switch (ta0eq1_ptr->actcmd)
 {
    case bkspcmd:
       utility(funcopen);                  /*  Open the tape  */

       if (ta0eq1_ptr->cntdef > 65535)                             /*d14351*/
       {                                                           /*d14351*/
          if (test_msg_setting(msgemsg))                           /*d14351*/
             printmsg(full_msg,prefixed,20,'E',                    /*d14351*/
                      "Count value of greater than 65535 invalid " /*d14351*/
                      "for this operation");                       /*d14351*/
          utility(funcclos);               /*  Close the tape  */  /*d14351*/
          break;                                                   /*d14351*/
       }                                                           /*d14351*/
       (*((short *)&ecbptr()->ce1fh1)) = ta0eq1_ptr->cntdef;
                                           /* operation count */

       if(tbspc((const char *)ta0eq1_ptr->chartape,D1,FALLBACK))
       {
          if (test_msg_setting(msgemsg))
             printmsg(full_msg,prefixed,1,'E',
                      "WAITC error on %s tape during "
                      "BACKSPACE", ta0eq1_ptr->chartape);

          if (ta0eq1_ptr->tapeglob)
             globtd2_ptr->_td2ecnt++;
       }

       utility(funcclos);                  /*  Close the tape  */
    break;

    case rewcmd:
       utility(funcopen);                  /*  Open the tape  */
       trewc((const char *)ta0eq1_ptr->chartape,FALLBACK);

       if (ecbptr()->ce1sug)               /* Non-zero return code */
       {
          if (test_msg_setting(msgemsg))
             printmsg(full_msg,prefixed,4,'E',
                      "WAITC error on %s tape during REWIND",
                      ta0eq1_ptr->chartape);

          if (ta0eq1_ptr->tapeglob)
             globtd2_ptr->_td2ecnt++;
       }

       utility(funcclos);                  /*  Close the tape  */
    break;

    case inputcmd:
       utility(funcopen);                  /*  Open the tape  */

       for(record_number = 0;
           record_number != ta0eq1_ptr->cntdef;
           record_number++)
       {
          utility(funccore);               /*  Check core levels */

          /* If we have a valid tape indicator, and the abort bit
             is set, then abort the operation
          */
          if((ta0eq1_ptr->tapeglob) &&
             (globtd2_ptr->_td2uabt))
          {
             if (test_msg_setting(msgimsg))
                printmsg(full_msg,prefixed,58,'I',
                         "User requested abort of operation to "
                         "%s tape",
                         ta0eq1_ptr->chartape);
             globtd2_ptr->_td2uabt = FALSE;
             break;
          }

          tprdc((const char *)ta0eq1_ptr->chartape,D1,ta0eq1_ptr->blkdef);

          if (waitc())
          {
             if (ta0eq1_ptr->tapeglob)
                globtd2_ptr->_td2ecnt++;

             if (ecbptr()->ce1sud[1] & CXSGEE)
             {
                if (test_msg_setting(msgimsg))
                   printmsg(full_msg,prefixed,68,'I',
                            "End of file or unexpected tape "
                            "mark on %s tape",
                            ta0eq1_ptr->chartape);
             }
             else if (ecbptr()->ce1sud[1] & (CXSGSE+CXSGLE))
             {
                if (test_msg_setting(msgemsg))
                   printmsg(full_msg,prefixed,8,'E',
                            "Incorrect length error on"
                            " %s tape",
                           ta0eq1_ptr->chartape);
             }
             else
                if (test_msg_setting(msgemsg))
                   printmsg(full_msg,prefixed,3,'E',
                            "WAITC error on %s tape - check "
                            "count and blksize",
                            ta0eq1_ptr->chartape);
             break;                           /* Leave this loop */
          }
          else
          {
             taprcd_ptr = ecbptr()->ce1cr1;

             if (test_msg_setting(msgwtp))
                printmsg(partial_msg,prefixed,63,'I',
                         "Macro: TPRDC  Tape: %s Data: %.*s",
                         ta0eq1_ptr->chartape,
                         l_taprecid+l_taphdr+l_taprecn,
                         taprcd_ptr->taprecid);
          }
          relcc(D1);
       }

    utility(funcclos);                  /*  Close the tape  */

    break;

    default:
       if (test_msg_setting(msgemsg))
          printmsg(full_msg,prefixed,5,'E',
                   "Invalid action command code");
 }

 if(ta0eq1_ptr->tapeglob)
    utility(func_end);           /*  Mark global as command complete  */

 if (test_msg_setting(msgimsg))
    printmsg(full_msg,prefixed,67,'I',
             "%s tape operation complete",
             ta0eq1_ptr->chartape);
 exit(0);
}
