/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2003, 2006                               */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/*
**********************************************************************
**
** FUNCTION NAME.....QTP7
**    Load Module Name:  QTP7
** PURPOSE...........PERFORM FUNCTIONS FOR THE TAPE DRIVER
** FUNCTION TYPE.....TPF
** DOCUMENT NAME...  TAPE31 UGUIDE ON TDRIVERS DISK
** LEVEL...........  VERSION 1  MODIFICATION 0
**
** INPUT...... ALL INPUT CONTAINED IN ECB MAPPED BY C_TA0EQ
**
** OUTPUT..... NONE
**
** DESCRIPTION..... THIS SEGMENT PERFORMS THE FOLLOWING FUNCTIONS:
**                      OUTPUT    write to either a general or
**                                realtime tape
**                      WRITE     write to the realtime tape
**
**********************************************************************
**
**
** DEPENDENCIES... SINCE THIS SEGMENT IS NOW PART OF A DLM, IF AN
**                 EXTERNAL CALL IS ADDED FOR WHICH A STUB DOES
**                 NOT EXIST, A STUB WILL NEED TO BE CREATED.
**                                                                     0
**                 IF THIS SEGMENT IS UPDATED TO CONTAIN WRITABLE
**                 STATIC, IT THEN MUST BE COMPILED WITH THE RENT
**                 OPTION.
**
**                 QXTK MAY REQUIRE MODIFICATION IF THIS PROGRAM
**                 CHANGES HOW IT CALLS IT
**
** RESTRICTIONS... NONE
**
** RELATED SEGMENTS...   QXTG           MESSAGE PARSER
**                       QTP2           COMMAND PARSING/INFORMATION
**                       QXTH           COMMAND INITIATOR
**                       QXTK           COMMON FUNCTIONS
**
**********************************************************************
**
** EXTERNAL REFERENCES...
**
** HEADER FILES #INCLUDED...
**
** ACRONYMS...    API     APPLICATION PROGRAMMING INTERFACE
**
**               TPFGLBL  TPF usage of global areas
**               C_GLOBZ  TPF usage of global areas
**               C_SD0RV  Driver usage of global area 1 & 2
**               TPFTAPE  TPF Tape functions
**               C_TA0EQ  Tape driver mapping of it's use of
**                        the global area
**
***********************************************************************
** CHANGE ACTIVITY...
**
**        UPDATE BY          PLOG #   DATE   DESCRIPTION
**        ---------          -------  ------ ------------
**        KENT               TD00301  071390 MSG SETTING ENHANCEMENTS
**        KENT               TD00346  022891 TEP PROJECT IMPROVEMENTS
**        MKENT              TD00442  072393 FIX SUBROUTINE INTERFACE
**        TCSSV              TD00652  021296 Migration to ISO-C
**                                           (LYNNEK)
**
************************************************************************/
#include <c_ta0eq64.h>
#include <tpf/tpfio.h>

static  void Fill_BLK(char *,                    /* buffer address    */
                      int,                       /* fill data length  */
                      unsigned char,             /* fill character    */
                      unsigned char);            /* char cycling flag */

int QTP7()
{

 struct TPF_regs  regs;
 struct TA0EQ1   *ta0eq1_ptr = (struct TA0EQ1 *)
                               &(ecbptr()->ebx000);
 struct TAPRCD   *taprcd_ptr;
 struct SD0RV    *sd0rv_ptr;
 struct GLOBTD2  *globtd2_ptr = 0;

      int   record_number, fill_len;

 char     *record_header;

 if(ta0eq1_ptr->tapeglob)
 {
    sd0rv_ptr = (struct SD0RV*) (long) * (int *) glob(_issdrv); //gcc

    globtd2_ptr = (struct GLOBTD2 *) &(sd0rv_ptr->_SD0TAPG) +
                  ta0eq1_ptr->tapeglob;

    utility(func_set);           /*  Mark global as command starting  */
 }

 switch (ta0eq1_ptr->actcmd)
 {
    case outptcmd:
       /* If globals are valid, and the abort bit is set,
          then abort the operation
       */
       if((ta0eq1_ptr->tapeglob) && (globtd2_ptr->_td2uabt))
       {
          if (test_msg_setting(msgimsg))
             printmsg(full_msg,prefixed,68,'I',
                      "User requested abort of operation to "
                      "%s tape",
                      ta0eq1_ptr->chartape);
          globtd2_ptr->_td2uabt = FALSE;
          break;
       }

       utility(funcopen);                  /*  Open the tape  */

       switch (ta0eq1_ptr->blkdef)
       {
          case L0:
             record_header = header_L0;
             fill_len = 127  - record_overhead;
          break;

          case L1:
             record_header = header_L1;
             fill_len = 381  - record_overhead;

          break;

          case L2:
             record_header = header_L2;
             fill_len = 1055 - record_overhead;

          break;

          case L4:
             record_header = header_L4;
             fill_len = 4095 - record_overhead;
          break;

          default:
             if (test_msg_setting(msgemsg))
                printmsg(full_msg,prefixed,5,'E',
                         "Invalid BLOCK option value");
             exit(0);
       }

       for(record_number = ta0eq1_ptr->cntdef;
           record_number > 0;
           record_number--)
       {
          utility(funccore);               /*  Check core levels */

          getcc(D1,GETCC_TYPE,ta0eq1_ptr->blkdef);
          taprcd_ptr = ecbptr()->ce1cr1;

          /*  Add record id  */
          memcpy(taprcd_ptr->taprecid,verify_recid,l_taprecid);

          /*  Add record header  */
          memcpy(taprcd_ptr->taphdr,record_header,l_taphdr);

          /*  Add record number  */
          sprintf((char *)taprcd_ptr->taprecn,"%*.*i",
                  l_taprecn,l_taprecn,record_number);

          /*  Add fill characters  */
          Fill_BLK((char *)taprcd_ptr->tapfill,     /* area to fill      */
                   fill_len,                /* fill data length  */
                   ta0eq1_ptr->filldef,     /* fill character    */
                   ta0eq1_ptr->cycledef);   /* char cycling flag */

          /*  Add record trailer  */
          memcpy(&taprcd_ptr->tapfill[fill_len],
                 trailer,l_trailer);

          if((ta0eq1_ptr->tapeglob) && (globtd2_ptr->_td2uabt))
          {
             if (test_msg_setting(msgimsg))
                printmsg(full_msg,prefixed,68,'I',
                         "User requested abort of operation to "
                         "%s tape",
                         ta0eq1_ptr->chartape);
             globtd2_ptr->_td2uabt = FALSE;
             break;
          }

          if (test_msg_setting(msgwtp))
          {
             if(!memcmp(ta0eq1_ptr->chartape,"RT",2))
                printmsg(partial_msg,prefixed,62,'I',
                         "Macro: TOURC  Tape: %s Data: %.*s",
                         ta0eq1_ptr->chartape,
                         l_taprecid+l_taphdr+l_taprecn,
                         taprcd_ptr->taprecid);
             else
                printmsg(partial_msg,prefixed,60,'I',
                         "Macro: TWRTC  Tape: %s Data: %.*s",
                         ta0eq1_ptr->chartape,
                         l_taprecid+l_taphdr+l_taprecn,
                         taprcd_ptr->taprecid);
          }

          if(!memcmp(ta0eq1_ptr->chartape,"RT",2))
             tourc((char *)ta0eq1_ptr->chartape,D1);
          else
             twrtc((char *)ta0eq1_ptr->chartape,D1);

          if (waitc())
          {
             if(ta0eq1_ptr->tapeglob)
                globtd2_ptr->_td2ecnt++;

             if (test_msg_setting(msgemsg))
                printmsg(full_msg,prefixed,3,'E',
                         "WAITC error on %s tape - "
                         "check count and blksize",
                         ta0eq1_ptr->chartape);
             break;                           /* Leave this loop */
          }
       }

       utility(funcclos);                  /*  Close the tape  */
    break;

    case writecmd:
       if (ta0eq1_ptr->cntdef > 4095)
       {
          if (test_msg_setting(msgemsg))
             printmsg(full_msg,prefixed,20,'E',
                      "Count value of greater than 4095 invalid "
                      "for this operation");
          break;
       }

       /* Set up block address and byte count */
       record_header = header_VA;
       getcc(D1,GETCC_TYPE,L4);
       taprcd_ptr = ecbptr()->ce1cr1;

       if (ta0eq1_ptr->cntdef > record_overhead)
          fill_len = ta0eq1_ptr->cntdef - record_overhead;
       else
          fill_len = 0;

       /* Set block address and length for TOUTC macro call */
       ecbptr()->ce1fa1 = ( int)ecbptr()->ce1cr1;
       (*((short *)&ecbptr()->ce1fh1)) = ta0eq1_ptr->cntdef;

       /*  Add record id  */
       memcpy(taprcd_ptr->taprecid,verify_recid,l_taprecid);

       /*  Add record header  */
       memcpy(taprcd_ptr->taphdr,record_header,l_taphdr);

       /*  Add record number (record length)  */
       sprintf((char *)taprcd_ptr->taprecn,"%*.*i",
               l_taprecn,l_taprecn,ta0eq1_ptr->cntdef);

       /*  Add fill characters  */
       Fill_BLK((char *)taprcd_ptr->tapfill,     /* area to fill      */
                fill_len,                /* fill data length  */
                ta0eq1_ptr->filldef,     /* fill character    */
                ta0eq1_ptr->cycledef);   /* char cycling flag */

       /*  Add record trailer  */
       memcpy(&taprcd_ptr->tapfill[fill_len],trailer,l_trailer);

       if (test_msg_setting(msgwtp))
          printmsg(partial_msg,prefixed,61,'I',
                   "Macro: TOUTC  Tape: %s Data: %.*s",
                   ta0eq1_ptr->chartape,
                   l_taprecid+l_taphdr+l_taprecn,
                   taprcd_ptr->taprecid);

       if(ta0eq1_ptr->buffdef == buffyes)
          toutc((char *)ta0eq1_ptr->chartape,D1,BUFFERED);
       else
          toutc((char *)ta0eq1_ptr->chartape,D1,NOBUFF);

       if (waitc())
       {
          if(ta0eq1_ptr->tapeglob)
             globtd2_ptr->_td2ecnt++;

          if (test_msg_setting(msgemsg))
             printmsg(full_msg,prefixed,3,'E',
                      "WAITC error on %s tape - check count "
                      "and blksize",
                      ta0eq1_ptr->chartape);
       }
       crusa(1,D1);
    break;

    default:
       if (test_msg_setting(msgemsg))
          printmsg(full_msg,prefixed,5,'E',
                   "Invalid action command code");
 }

 if(ta0eq1_ptr->tapeglob)
    utility(func_end);           /*  Mark global as command complete  */

 if (test_msg_setting(msgimsg))
    printmsg(full_msg,prefixed,67,'I',
             "%s tape operation complete",
             ta0eq1_ptr->chartape);
 exit(0);
}

 static void Fill_BLK(char * buffer,             /* buffer address    */
                       int fill_length,      /* fill data length  */
                      unsigned char fillbyte,    /* fill character    */
                      unsigned char cycleflag)   /* char cycling flag */

/* Fill the block with the indicated fill byte.  If the 'cycle'
   flag is set, increment the fill byte by 1 (mod 256) for each
   sucessive byte
*/
{
  int buffer_index;

 for (buffer_index = 0; buffer_index < fill_length; buffer_index++)
 {
    buffer[buffer_index] = fillbyte;
    if (cycleflag == cycleyes)
       fillbyte++;
 }

 return;
}
