/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2003, 2006                               */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/*
**********************************************************************
**
** FUNCTION NAME.....QTP8
**    Load Module Name:  QTP8
** PURPOSE...........PERFORM FUNCTIONS FOR THE TAPE DRIVER
** FUNCTION TYPE.....TPF
** DOCUMENT NAME...  TAPE31 UGUIDE ON TDRIVERS DISK
** LEVEL...........  VERSION 1  MODIFICATION 0
**
** INPUT...... ALL INPUT CONTAINED IN ECB MAPPED BY C_TA0EQ
**
** OUTPUT..... NONE
**
** DESCRIPTION..... THIS SEGMENT PERFORMS THE FOLLOWING FUNCTIONS:
**                      QUERY     Display tape indicator fields
**
**********************************************************************
**
** DEPENDENCIES... SINCE THIS SEGMENT IS NOW PART OF A DLM, IF AN
**                 EXTERNAL CALL IS ADDED FOR WHICH A STUB DOES
**                 NOT EXIST, A STUB WILL NEED TO BE CREATED.
**                                                                     0
**                 IF THIS SEGMENT IS UPDATED TO CONTAIN WRITABLE
**                 STATIC, IT THEN MUST BE COMPILED WITH THE RENT
**                 OPTION.
**
**
** RESTRICTIONS... NONE
**
** RELATED SEGMENTS...   QXTG           MESSAGE PARSER
**                       QTP2           COMMAND PARSING/INFORMATION
**                       QXTH           COMMAND INITIATOR
**
**********************************************************************
**
** EXTERNAL REFERENCES...
**
** HEADER FILES #INCLUDED...
**
** ACRONYMS...    API     APPLICATION PROGRAMMING INTERFACE
**
**               TPFGLBL  TPF usage of global areas
**               C_GLOBZ  TPF usage of global areas
**               C_SD0RV  Driver usage of global area 1 & 2
**               TPFTAPE  TPF Tape functions
**               C_TA0EQ  Tape driver mapping of it's use of
**                        the global area
**
***********************************************************************
** CHANGE ACTIVITY...
**
**        UPDATE BY          PLOG #   DATE   DESCRIPTION
**        ---------          -------  ------ ------------
**        KENT               TD00301  071390 MSG SETTING ENHANCEMENTS
**        KENT               TD00346  022891 TEP PROJECT IMPROVEMENTS
**        TCSSV              TD00652  021296 Migration to ISO-C
**                                           (LYNNEK)
**
************************************************************************/
#include <c_ta0eq64.h>

#define on_off(flag)   flag ? "On" : "--"

int QTP8()
{
 struct TPF_regs  regs;
 struct TA0EQ1   *ta0eq1_ptr = (struct TA0EQ1 *)
                               &(ecbptr()->ebx000);
 struct SD0RV    *sd0rv_ptr;
 struct GLOBTD2  *globtd2_ptr;

 struct tpstat   *tdspc_ptr1, *tdspc_ptr2, tdspc_null;

#define status_fmt1  "%-30.30s %-10s  %s "

#define status_fmt2  status_fmt1 "\n" status_fmt1

 if((ta0eq1_ptr->tapeglob) && (ta0eq1_ptr->actcmd != querycmd))
 {
    sd0rv_ptr = (struct SD0RV*) (long) * (int *) glob(_issdrv);  //gcc

    globtd2_ptr = (struct GLOBTD2 *) &(sd0rv_ptr->_SD0TAPG) +
                  ta0eq1_ptr->tapeglob;

    utility(func_set);           /*  Mark global as command starting  */
 }

 switch (ta0eq1_ptr->actcmd)
 {
    case querycmd:
       if ( (tdspc_ptr1 = tdspc((char *)ta0eq1_ptr->chartape,ACTIVE,D1))  )  //gcc
       {
          if(test_msg_setting(msgimsg))
             printmsg(partial_msg,prefixed,65,'I',
                      "Active %s on drive %4.4X",
                      tdspc_ptr1->tpname,
                      tdspc_ptr1->cuu);
       }
       else
       {
          if(test_msg_setting(msgimsg))
             printmsg(partial_msg,prefixed,65,'I',
                      "Active %s tape not mounted",
                      ta0eq1_ptr->chartape);
          break;  /* Don't do anything if no active tape */
       }

       if ( (tdspc_ptr2 = tdspc((char *)ta0eq1_ptr->chartape,STANDBY,D2)) )   //gcc
       {
          if(test_msg_setting(msgimsg))
             printmsg(partial_msg,prefixed,65,'I',
                      "Standby %s on drive %4.4X",
                      tdspc_ptr2->tpname,
                      tdspc_ptr2->cuu);
       }
       else
       {
          if(test_msg_setting(msgimsg))
             printmsg(partial_msg,prefixed,65,'I',
                      "Standby %s tape not mounted",
                      ta0eq1_ptr->chartape);

          /* Ensure standby indicators all show as 0 - point to
             zeroed out copy of TDSPC structure
          */
          tdspc_ptr2 = (struct tpstat *) memset(&tdspc_null,0,
                                                sizeof(tdspc_null));
       }

       if(test_msg_setting(msgimsg))
       {
          printmsg(partial_msg,noprefix,65,'I',
                   status_fmt1,"Field Name","Active","Standby");

          printmsg(partial_msg,noprefix,65,'I',
                   status_fmt1,
                   "Hold I/O requests",
                   on_off(tdspc_ptr1->holdio),
                   on_off(tdspc_ptr2->holdio));

          printmsg(partial_msg,noprefix,65,'I',
                   status_fmt2,
                   "Inhibit Queue restart",
                   on_off(tdspc_ptr1->inhqueue),
                   on_off(tdspc_ptr2->inhqueue),
                   "Auxilary queued",
                   on_off(tdspc_ptr1->auxqueue),
                   on_off(tdspc_ptr2->auxqueue));

          printmsg(partial_msg,noprefix,65,'I',
                   status_fmt2,
                   "Standby tape",
                   on_off(tdspc_ptr1->standby),
                   on_off(tdspc_ptr2->standby),
                   "Tape is reserved",
                   on_off(tdspc_ptr1->reserved),
                   on_off(tdspc_ptr2->reserved));

          printmsg(partial_msg,noprefix,65,'I',
                   status_fmt2,
                   "Tape is offline",
                   on_off(tdspc_ptr1->offline),
                   on_off(tdspc_ptr2->offline),
                   "Tape is closed",
                   on_off(tdspc_ptr1->closed),
                   on_off(tdspc_ptr2->closed));

          printmsg(partial_msg,noprefix,65,'I',
                   status_fmt2,
                   "Standard user labels present",
                   on_off(tdspc_ptr1->userlabl),
                   on_off(tdspc_ptr2->userlabl),
                   "Backward tape switch",
                   on_off(tdspc_ptr1->btswitch),
                   on_off(tdspc_ptr2->btswitch));

          printmsg(partial_msg,noprefix,65,'I',
                   status_fmt1,
                   "Multivolume - not first tape",
                   on_off(tdspc_ptr1->notfirst),
                   on_off(tdspc_ptr2->notfirst));

          printmsg(partial_msg,noprefix,65,'I',
                   status_fmt2,
                   "Input tape",
                   on_off(tdspc_ptr1->input),
                   on_off(tdspc_ptr2->input),
                   "Labeled tape",
                   on_off(tdspc_ptr1->labeled),
                   on_off(tdspc_ptr2->labeled));

          printmsg(partial_msg,prefixed,65,'I',
                   "End of display");
       }
    break;

    default:
       if(test_msg_setting(msgemsg))
          printmsg(full_msg,prefixed,5,'E',
                   "Invalid action command code");
 }

 if((ta0eq1_ptr->tapeglob) && (ta0eq1_ptr->actcmd != querycmd))
    utility(func_end);           /*  Mark global as command complete  */

 if(test_msg_setting(msgimsg))
    printmsg(full_msg,prefixed,67,'I',
             "%s tape operation complete",
             ta0eq1_ptr->chartape);
 exit(0);
}
