/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2003, 2006                               */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/*
**********************************************************************
**
** FUNCTION NAME.....QTP9
**    Load Module Name:  QTP9
** PURPOSE...........PERFORM FUNCTIONS FOR THE TAPE DRIVER
** FUNCTION TYPE.....TPF
** DOCUMENT NAME...  TAPE31 UGUIDE ON TDRIVERS DISK
** LEVEL...........  VERSION 1  MODIFICATION 0
**
** INPUT...... ALL INPUT CONTAINED IN ECB MAPPED BY C_TA0EQ
**
** OUTPUT..... NONE
**
** DESCRIPTION..... THIS SEGMENT PERFORMS COMMAND PROCESSING FOR
**                  THE FOLLOWING SYSTEM TEST INTERFACE COMMANDS:
**                      PAUSE     PAUSE SYSTEST ACTIVITY
**                      RESTART   RE-START SYSTEST ACTIVITY
**
**                  THIS SEGMENT ALSO PERFORMS THE FOLLOWING
**                  FUNCTION TEST INTERFACE COMMANDS:
**                      SWITCH    PERFORM AN APPLICATION INITIATED
**                                TAPE SWITCH
**
**********************************************************************
**
** DEPENDENCIES... QXTK MAY REQUIRE MODIFICATION IF THIS PROGRAM
**                 CHANGES HOW IT CALLS IT
**
**                 SINCE THIS SEGMENT IS NOW PART OF A DLM, IF AN
**                 EXTERNAL CALL IS ADDED FOR WHICH A STUB DOES
**                 NOT EXIST, A STUB WILL NEED TO BE CREATED.
**                                                                     0
**                 IF THIS SEGMENT IS UPDATED TO CONTAIN WRITABLE
**                 STATIC, IT THEN MUST BE COMPILED WITH THE RENT
**                 OPTION.
**
**
** RESTRICTIONS... NONE
**
** RELATED SEGMENTS...   QXTG           MESSAGE PARSER
**                       QTP1           COMMAND PARSING/INFORMATION
**                       QTP2           COMMAND INITIATOR
**                       QXTH           COMMAND INITIATOR
**                       QXTJ           SYSTEST ACTIVITY DRIVER
**                       QXTK           COMMON FUNCTIONS
**
**********************************************************************
**
** EXTERNAL REFERENCES...
**
** HEADER FILES #INCLUDED...
**
** ACRONYMS...    API     APPLICATION PROGRAMMING INTERFACE
**
**               TPFGLBL  TPF usage of global areas
**               C_GLOBZ  TPF usage of global areas
**               C_SD0RV  Driver usage of global area 1 & 2
**               TPFTAPE  TPF Tape functions
**               C_TA0EQ  Tape driver mapping of it's use of
**                        the global area
**
***********************************************************************
** CHANGE ACTIVITY...
**
**        UPDATE BY          PLOG #   DATE   DESCRIPTION
**        ---------          -------  ------ ------------
**        KENT               TD00301  071390 MSG SETTING ENHANCEMENTS
**        KENT               TD00346  022891 TEP PROJECT IMPROVEMENTS
**        TCSSV              TD00652  021296 Migration to ISO-C
**                                           (LYNNEK)
**
************************************************************************/
#include <c_ta0eq64.h>

int QTP9()  //gcc
{
 struct TA0EQ1   *ta0eq1_ptr = (struct TA0EQ1 *)
                               &(ecbptr()->ebx000);
 struct SD0RV    *sd0rv_ptr;
 struct GLOBTD1  *globtd1_ptr;

 struct tpstat   *tdspc_ptr;

#                                                            /*TD00619*/
#ifndef _TARGET_TPF /* TPF ISO-C */                          /*TD00619*/
#                                                            /*TD00619*/
extern void COS0();                                          /*TD00619*/
#                                                            /*TD00619*/
#endif                                                       /*TD00619*/
#                                                            /*TD00619*/


 sd0rv_ptr = (struct SD0RV*) (long) * (int *) glob(_issdrv); //gcc

 globtd1_ptr = (struct GLOBTD1 *) &(sd0rv_ptr->_SD0TAPE);

 switch (ta0eq1_ptr->actcmd)
 {
    case pausecmd:
       if(!globtd1_ptr->_td1actv)
       {
          printmsg(full_msg,prefixed,22,'E',
                   "Command ignored - driver has not been START'ed");
       }
       else if(!globtd1_ptr->_td1pauz)
       {
          globtd1_ptr->_td1pauz = TRUE;
          printmsg(full_msg,prefixed,82,'I',
                   "Driver now PAUSE'd");
       }
       else
       {
          printmsg(full_msg,prefixed,23,'E',
                   "Command ignored - driver is PAUSE'd");
       }

    break;

    case rstrtcmd:
       if(!globtd1_ptr->_td1actv)
       {
          printmsg(full_msg,prefixed,22,'E',
                   "Command ignored - driver has not been START'ed");
       }
       else if(globtd1_ptr->_td1pauz)
       {
          globtd1_ptr->_td1pauz = FALSE;
          printmsg(full_msg,prefixed,83,'I',
                   "Driver now RESTART'ed");
       }
       else
       {
          printmsg(full_msg,prefixed,24,'E',
                   "Command ignored - driver not PAUSE'd");
       }
    break;

    case cpswcmd:
       tdspc_ptr = tdspc((char *)ta0eq1_ptr->chartape,
                          ACTIVE,D1);

       if(tdspc_ptr->cuu == 0)   /* Exit if tape not mounted */
       {
          if(test_msg_setting(msgimsg))
             printmsg(full_msg,prefixed,65,'I',
                      "Active %s tape not mounted",
                      ta0eq1_ptr->chartape);
          break;
       }

       cremc(strlen((char *)ta0eq1_ptr->chartape),
             ta0eq1_ptr->chartape,
             COS0);
    break;

    default:
       printmsg(full_msg,prefixed,5,'E',
                "Invalid action command code");
 }

 exit(0);
}
