         PRINT NOGEN
         BEGIN NAME=QXTG,AMODE=64
***********************************************************************
*       Licensed Materials - Property of IBM                          *
*       "Restricted Materials of IBM"                                 *
*       5748-T15                                                      * 
*       Copyright IBM Corp. 2003, 2006                                *
*       All Rights Reserved                                           *
*       US Government Users Restricted Rights - Use, duplication      *
*       or dislosure restricted by GSA ADP Schedule Contract          *
*       with IBM Corp.                                                *
***********************************************************************
*  This file is provided on an "as is" basis.  There are no           *
*  warranties, express or implied, including the implied warranties   *
*  of merchantability and fitness for a particular purpose.           *
*  IBM has no obligation to provide service, defect correction, or    *
*  any maintenance for the file.  IBM has no obligation to supply     *
*  any updates or enhancements for the file to you even if such are   *
*  or later become available.                                         *
***********************************************************************
***********************************************************************
*
* MODULE NAME..... QXTG
* DESCRIPTION..... GENERAL TAPE DRIVER, 64-BIT
* RELATED MODULE.. QTP1
*                  QTP1A
*                  QTP2
*                  QTP3
*                  QTP4
*                  QTP5
*                  QTP6
*                  QTP7
*                  QTP8
*                  QTP9
*                  QTPA
*                  QXTG
*                  QXTH
*                  QXTI
*                  QXTJ
*                  QXTK
*
* DOCUMENT NAME... GENERAL TAPE DRIVER USERS GUIDE
* LEVEL........... N/A
*
* FUNCTION...      THIS IS A GENERAL PURPOSE TAPE DRIVER TO READ AND
*                  WRITE TO A GENERAL TAPE (TAPE NAME GEN) AND TO
*                  WRITE TO THE REALTIME TAPE RTA. THE FOLLOWING
*                  TAPE MACROS CAN BE EXERCISED:
*
*                          TASNC - ASSIGN GENERAL TAPE
*                          TBSPC - BACKSPACE GENERAL TAPE
*                          TCLSC - CLOSE GENERAL TAPE
*                          TDSPC - DISPLAY TAPE STATUS
*                          TOPNC - OPEN GENERAL TAPE
*                          TOURC - WRITE REALTIME AND REL CORE
*                          TOUTC - WRITE REALTIME
*                          TPRDC - READ FROM A GENERAL TAPE
*                          TREWC - REWIND A GENERAL TAPE
*                          TRSVC - RESERVE A GENERAL TAPE
*                          TSYNC - SYN TAPE
*                          TDTAC - USE USER CCW  (READ/WRITE)
*                          TPCNC - USE USER CCW  (TAPE CONTROL)
*                          TWRTC - WRITE A GENERAL TAPE
*
*                  THERE IS ALSO A SYSTEM TEST INTERFACE WHICH MAKES
*                  CALLS USING THE FUNCTION TEST COMMANDS.  THIS
*                  INTERFACE IS IMPLEMENTED VIA SEGMENTS QXTJ,
*                  QTP3, QTP9, AND QTP1A
*
* MODULE ATTRIBUTES..
*   TYPE.......... 'E' (ECB CONTROLLED)
*   DEPENDENCY.... NONE (OBJECT SHIPPABLE)
*   ALLOCATION.... 4K FILE RESIDENT
*   ENVIRONMENT... BASE, BSS
*   ENTRY POINT... QXTG  (ENTNC FROM CVZZ)
*
* ECB-CONVENTIONS.......
*
*   WORK AREA
*         SEE EQUATE MACRO TA0EQ
*
* DATA LEVELS
*         NONE USED
*
* REGISTER-CONVENTIONS..
*         R0 -
*         R1 - BAS REGISTER
*         R2 - POINTER TO WTOPC MESSAGE
*         R3 - BAS/WORK REG
*         R4 - BAS REGISTER
*         R5 -
*         R6 - BASE REG FOR BPKDC PROCESSING
*         R7 - WORK REG
*         R14 -
*         R15 - WORK
*
***********************************************************************
** CHANGE ACTIVITY...
**
**        UPDATE BY          PLOG #   DATE   DESCRIPTION
**        ---------          -------  ------ ------------
**        KENT               TD00301  071390 MSG SETTING ENHANCEMENTS
**        KENT               TD00346  022891 TEP PROJECT IMPROVEMENTS
**        KENT               TD00368  070791 FIX RBL OPCODE
**        YMLI               F14612   081506 The output routing
***********************************************************************
*
* TO USE...... QXTG IS INVOKED VIA ZTEST FUNCTIONAL MESSAGE.
*
*        ZTEST TAPE aaaaa  options
*
*              aaaaa=ACTION CODE
*                       ABORT
*                       BACKSPACE      (GEN TAPE ONLY)
*                       CONTROL
*                       DUMP
*                       INPUT          (GEN TAPE ONLY)
*                       OUTPUT         (TOURC -- RTA/RTL TWRTC -- GEN)
*                       REWIND         (GEN TAPE ONLY)
*                       SYNC
*                       QUERY
*                       WRITE          (TOUTC)
*                       CCW
*                       SWITCH
*                       VERIFY
*                       START
*                       STOP
*                       PAUSE
*                       RESTART
*                       SETUP
*
*               options =
*                       BLOCK- (127,381,1055,4K)
*
*                       COUNT-NNNNNNNNN
*                           NINE DIGIT DECIMAL NUMBER FOR COUNT
*                           OF RECORDS TO READ OR WRITE
*
*                       TAPE- (ANY VALID TPF TAPE NAME)
*
*                       OPEN- (ASSIGN, INPUT, OUTPUT, NO)
*                           INPUT - OPEN THE GENERAL TAPE FOR INPUT
*                           OUTPUT - OPEN THE GENERAL TAPE FOR OUTPUT
*                           NO  - DON'T OPEN THE GENERAL TAPE (ERR)
*                           ASSIGN - ASSIGN TAPE FROM PREVIOUS ECB
*
*                       CLOSE- (EXIT, RESERVE, YES)
*                           YES - CLOSE THE GENERAL TAPE
*                           EXIT - JUST EXITC (CTL-13)
*                           RESERVE - RESERVE THE TAPE FOR NEXT ECB
*
*                       FILLCHAR-cc
*                           CHARACTER TO FILL BLOCK WITH (HEX NUM)
*                           VALID VALUES ARE X'00' TO X'FF'
*
*                       CYCLE / NOCYCLE
*                           PROVIDE CYCLING OF THE FILLCHAR VALUE
*
*                       OPCODE-aaa
*                           3 CHARACTER TAPE CCW OP-CODE
*
*                       BUFFERED/NOBUFFERED (3480/3490 ONLY)
*                           BUFFERED - BUFFERRED OUTPUT
*                           NOBUFFERED - WRITE IMMEDIATE (TWI)
*
*                       MSG = EMSG | IMSG | PARMLIST | WTOPC | ALL
*                           WTOPC - NORMAL WTOPC READS/WRITES TO OPER
*                           PARMLIST - SHOW PARAMETERS - NO WTOPC MSGS
*                           EMSG - ERROR MESSAGES ONLY
*                           IMSG - INFORMATIVE/COMPLETION MESSAGES
*                           ALL - SHOW BOTH WTOPC AND DEFAULTS MSGS
*
*                       AUTO/NOAUTO (SYNC COMMAND ONLY)
*                           AUTO - INITIATE AUTO TAPE SWITCH IF ERR
*                           NOAUTO - DON'T AUTO TAPE SWITCH IF ERR
*
*
         SPACE 2
BLK$LIST DSECT
BLK$PARM DS    CL4                CHARACTER PARAMETER NAME
BLK$VAL  DS    AL1                INDICATOR FOR PARAMETER VALUE
LBLK$TBL EQU   *-BLK$LIST         LENGTH OF AN ENTRY
$IS$     CSECT
         SPACE 2
OPN$LIST DSECT
OPN$PARM DS    CL6                CHARACTER PARAMETER VALUE
OPN$VAL  DS    AL1                INDICATOR FOR PARAMETER VALUE
LOPN$TBL EQU   *-OPN$LIST         LENGTH OF AN ENTRY
$IS$     CSECT
         SPACE 2
CLS$LIST DSECT
CLS$PARM DS    CL7                CHARACTER PARAMETER VALUE
CLS$VAL  DS    AL1                INDICATOR FOR PARAMETER VALUE
LCLS$TBL EQU   *-CLS$LIST         LENGTH OF AN ENTRY
$IS$     CSECT
         SPACE 2
MSG$LIST DSECT
MSG$PARM DS    CL8                CHARACTER PARAMETER VALUE
MSG$VAL  DS    AL1                INDICATOR FOR PARAMETER VALUE
LMSG$TBL EQU   *-MSG$LIST         LENGTH OF AN ENTRY
$IS$     CSECT
         SPACE 2
OPC$LIST DSECT
OPC$PARM DS    CL3                CHARACTER OPCODE NAME
OPC$VAL  DS    AL1                INDICATOR FOR OPCODE NAME
LOPC$TBL EQU   *-OPC$LIST         LENGTH OF AN ENTRY
$IS$     CSECT
         DCTBPK REG=R6            MAP BPKDC OUTPUT LIST
*
         J     QXTG0100           BRANCH AROUND EYE-CATCHER             64MODE
         DC    CL8'&SYSDATE',CL1' ',CL8'&SYSTIME'
*
QXTG0100 DS    0H
         BPKDC EXECUTE=YES,BPKD=MAINBPK,HELP=QXTG9995,ERROR=QXTG9995,  X
               EMSG=Y
*
         XC    TAPEINDC,TAPEINDC  CLEAR INDICATORS AND FLAGS
         LLGF  R6,CE1CR0          PROCESS MAINLINE COMMAND              64MODE
         LLGF  R7,ACTION                                                64MODE
         B     QXTG0110(R7)       GO TO INDICATED ROUTINE
         SPACE 2
*
QXTG0110 DS    0H                 BRANCH TABLE FOR CMDS
         J     QXTG9100           INVALID INDEX GENERATED               64MODE
         J     QXTG9995           REQUESTED SYSTEM TEST HELP            64MODE
         J     QXTG9990           REQUESTED FUNCTION TEST HELP          64MODE
         J     QXTG9985           REQUESTED OTHER TEST HELP   @TEDMLIN  64MODE
         J     QXTG0200           ACTION IS 'BACKSPACE'                 64MODE
         J     QXTG0300           ACTION IS 'INPUT'                     64MODE
         J     QXTG0400           ACTION IS 'OUTPUT'                    64MODE
         J     QXTG0500           ACTION IS 'REWIND'                    64MODE
         J     QXTG0600           ACTION IS 'SYNC'                      64MODE
         J     QXTG0700           ACTION IS 'QUERY'                     64MODE
         J     QXTG0800           ACTION IS 'WRITE'                     64MODE
         J     QXTG0900           ACTION IS 'DUMP'                      64MODE
         J     QXTG1000           ACTION IS 'CCW'                       64MODE
         J     QXTG1100           ACTION IS 'SWITCH'                    64MODE
         J     QXTG1200           ACTION IS 'VERIFY'                    64MODE
         J     QXTG1300           ACTION IS 'SETUP'                     64MODE
         J     QXTG1400           ACTION IS 'START'                     64MODE
         J     QXTG1500           ACTION IS 'STOP'                      64MODE
         J     QXTG1600           ACTION IS 'STATUS'                    64MODE
         J     QXTG1700           ACTION IS 'PAUSE'                     64MODE
         J     QXTG1800           ACTION IS 'RESTART'                   64MODE
         J     QXTG1900           ACTION IS 'ABORT'                     64MODE
         J     QXTG2000           ACTION IS 'CONTROL'                   64MODE
         J     QXTG2100           ACTION IS 'OTEST1'          @TEDMLIN  64MODE
         SPACE 2
*
***************************************************************
         EJECT 1
QXTG0200 DS    0H                 TAPE 'BACKSPACE' COMMAND
         MVI   ACTCMD,BKSPCMD     SET COMMAND INDICATOR
         OI    OPENIND,VALOPTN    SET VALID OPTS INDICATOR
         OI    CLOSEIND,VALOPTN   SET VALID OPTS INDICATOR
         OI    BUFFIND,VALOPTN    SET VALID OPTS INDICATOR
         OI    CNTIND,VALOPTN     SET VALID OPTS INDICATOR
         OI    TAPIND,VALOPTN     SET VALID OPTS INDICATOR
         J     QXTG3000           CONTINUE ON OTHERWISE                 64MODE
         SPACE 2
QXTG0300 DS    0H                 TAPE 'INPUT' COMMAND
         MVI   ACTCMD,INPUTCMD    SET COMMAND INDICATOR
         OI    BLKIND,VALOPTN     SET VALID OPTS INDICATOR
         OI    TAPIND,VALOPTN     SET VALID OPTS INDICATOR
         OI    OPENIND,VALOPTN    SET VALID OPTS INDICATOR
         OI    CLOSEIND,VALOPTN   SET VALID OPTS INDICATOR
         OI    CNTIND,VALOPTN     SET VALID OPTS INDICATOR
         J     QXTG3000           CONTINUE ON OTHERWISE                 64MODE
         SPACE 2
QXTG0400 DS    0H                 TAPE 'OUTPUT' COMMAND
         MVI   ACTCMD,OUTPTCMD    SET COMMAND INDICATOR
         OI    BLKIND,VALOPTN     SET VALID OPTS INDICATOR
         OI    TAPIND,VALOPTN     SET VALID OPTS INDICATOR
         OI    OPENIND,VALOPTN    SET VALID OPTS INDICATOR
         OI    CLOSEIND,VALOPTN   SET VALID OPTS INDICATOR
         OI    BUFFIND,VALOPTN    SET VALID OPTS INDICATOR
         OI    CNTIND,VALOPTN     SET VALID OPTS INDICATOR
         OI    FILLIND,VALOPTN    SET VALID OPTS INDICATOR
         OI    CYCLEIND,VALOPTN   SET VALID OPTS INDICATOR
         J     QXTG3000           CONTINUE ON OTHERWISE                 64MODE
         SPACE 2
QXTG0500 DS    0H                 TAPE 'REWIND' COMMAND
         MVI   ACTCMD,REWCMD      SET COMMAND INDICATOR
         OI    TAPIND,VALOPTN     SET VALID OPTS INDICATOR
         OI    OPENIND,VALOPTN    SET VALID OPTS INDICATOR
         OI    CLOSEIND,VALOPTN   SET VALID OPTS INDICATOR
         OI    BUFFIND,VALOPTN    SET VALID OPTS INDICATOR
         J     QXTG3000           CONTINUE ON OTHERWISE                 64MODE
         SPACE 2
QXTG0600 DS    0H                 TAPE 'SYNC' COMMAND
         MVI   ACTCMD,SYNCCMD     SET COMMAND INDICATOR
         OI    TAPIND,VALOPTN     SET VALID OPTS INDICATOR
         OI    OPENIND,VALOPTN    SET VALID OPTS INDICATOR
         OI    CLOSEIND,VALOPTN   SET VALID OPTS INDICATOR
         OI    BUFFIND,VALOPTN    SET VALID OPTS INDICATOR
         OI    AUTOIND,VALOPTN    SET VALID OPTS INDICATOR
         J     QXTG3000           CONTINUE ON OTHERWISE                 64MODE
         SPACE 2
QXTG0700 DS    0H                 TAPE 'QUERY' COMMAND
         MVI   ACTCMD,QUERYCMD    SET COMMAND INDICATOR
         OI    TAPIND,VALOPTN     SET VALID OPTS INDICATOR
         J     QXTG3000           CONTINUE ON OTHERWISE                 64MODE
         SPACE 2
QXTG0800 DS    0H                 TAPE 'WRITE' COMMAND
         MVI   ACTCMD,WRITECMD    SET COMMAND INDICATOR
         OI    TAPIND,VALOPTN     SET VALID OPTS INDICATOR
         OI    BUFFIND,VALOPTN    SET VALID OPTS INDICATOR
         OI    CNTIND,VALOPTN     SET VALID OPTS INDICATOR
         OI    FILLIND,VALOPTN    SET VALID OPTS INDICATOR
         OI    CYCLEIND,VALOPTN   SET VALID OPTS INDICATOR
         J     QXTG3000           CONTINUE ON OTHERWISE                 64MODE
         SPACE 2
QXTG0900 DS    0H                 TAPE 'DUMP' COMMAND
         MVI   ACTCMD,DUMPCMD     SET COMMAND INDICATOR
         OI    CNTIND,VALOPTN     SET VALID OPTS INDICATOR
         J     QXTG3000           CONTINUE ON OTHERWISE                 64MODE
         SPACE 2
QXTG1000 DS    0H                 TAPE 'CCW' COMMAND
         MVI   ACTCMD,CCWCMD      SET COMMAND INDICATOR
         OI    TAPIND,VALOPTN     SET VALID OPTS INDICATOR
         OI    OPENIND,VALOPTN    SET VALID OPTS INDICATOR
         OI    CLOSEIND,VALOPTN   SET VALID OPTS INDICATOR
         OI    BUFFIND,VALOPTN    SET VALID OPTS INDICATOR
         OI    CNTIND,VALOPTN     SET VALID OPTS INDICATOR
         OI    FILLIND,VALOPTN    SET VALID OPTS INDICATOR
         OI    OPCIND,VALOPTN     SET VALID OPTS INDICATOR
         OI    CYCLEIND,VALOPTN   SET VALID OPTS INDICATOR
         J     QXTG3000           CONTINUE ON OTHERWISE                 64MODE
         SPACE 2
QXTG1100 DS    0H                 TAPE 'SWITCH' COMMAND
         MVI   ACTCMD,CPSWCMD     SET COMMAND INDICATOR
         OI    TAPIND,VALOPTN     SET VALID OPTS INDICATOR
         J     QXTG3000           CONTINUE ON OTHERWISE                 64MODE
         SPACE 2
QXTG1200 DS    0H                 TAPE 'VERIFY' COMMAND
         MVI   ACTCMD,VERFYCMD    SET COMMAND INDICATOR
         OI    BLKIND,VALOPTN     SET VALID OPTS INDICATOR
         OI    TAPIND,VALOPTN     SET VALID OPTS INDICATOR
         OI    OPENIND,VALOPTN    SET VALID OPTS INDICATOR
         OI    CLOSEIND,VALOPTN   SET VALID OPTS INDICATOR
         OI    BUFFIND,VALOPTN    SET VALID OPTS INDICATOR
         OI    CNTIND,VALOPTN     SET VALID OPTS INDICATOR
         OI    FILLIND,VALOPTN    SET VALID OPTS INDICATOR
         OI    CYCLEIND,VALOPTN   SET VALID OPTS INDICATOR
         J     QXTG3000           CONTINUE ON OTHERWISE                 64MODE
         SPACE 2
QXTG1300 DS    0H                 TAPE 'SETUP' COMMAND
         MVI   ACTCMD,SETUPCMD    INDICATE SETUP COMMAND
         J     QXTG3500           PROCEED TO SYSTEST PROCESSING         64MODE
         SPACE 2
QXTG1400 DS    0H                 TAPE 'START' COMMAND
         MVI   ACTCMD,STARTCMD    INDICATE START COMMAND
         J     QXTG3500           PROCEED TO SYSTEST PROCESSING         64MODE
         SPACE 2
QXTG1500 DS    0H                 TAPE 'STOP' COMMAND
         MVI   ACTCMD,STOPCMD     INDICATE STOP COMMAND
         J     QXTG3500           PROCEED TO SYSTEST PROCESSING         64MODE
         SPACE 2
QXTG1600 DS    0H                 TAPE 'STATUS' COMMAND
         MVI   ACTCMD,STATCMD     INDICATE STATUS COMMAND
         J     QXTG3500           PROCEED TO SYSTEST PROCESSING         64MODE
         SPACE 2
QXTG1700 DS    0H                 TAPE 'PAUSE' COMMAND
         MVI   ACTCMD,PAUSECMD    INDICATE PAUSE COMMAND
         J     QXTG3500           PROCEED TO SYSTEST PROCESSING         64MODE
         SPACE 2
QXTG1800 DS    0H                 TAPE 'RESTART' COMMAND
         MVI   ACTCMD,RSTRTCMD    INDICATE RESTART COMMAND
         J     QXTG3500           PROCEED TO SYSTEST PROCESSING         64MODE
         SPACE 2
QXTG1900 DS    0H                 TAPE 'ABORT' COMMAND
         MVI   ACTCMD,ABORTCMD    INDICATE ABORT COMMAND
         OI    TAPIND,VALOPTN     SET VALID OPTS INDICATOR
         J     QXTG3000           CONTINUE ON OTHERWISE                 64MODE
         SPACE 2
QXTG2000 DS    0H                 TAPE 'CONTROL' COMMAND
         MVI   ACTCMD,CNTRLCMD    INDICATE CONTROL COMMAND
         J     QXTG3500           PROCEED TO SYSTEST PROCESSING         64MODE
         EJECT 1
         SPACE 2
QXTG2100 DS    0H                                             @TEDMLIN
         ENTNC QTPA               ENTER NEXT SEGMENT          @TEDMLIN
         SPACE 5                                              @TEDMLIN
QXTG3000 DS    0H                 GO ON TO NEXT STEP
         BRAS  R1,QXTG7000        EXAMINE AND SET OPTION VALUES         64MODE
         BRAS  R3,QXTG9000        DIAGNOSE AND WARN                     64MODE
         ENTRC QTP2               SET COMMAND SPECIFIC DEFAULTS
         ENTNC QXTH               ENTER NEXT SEGMENT FOR PROCESSING
         SPACE 5
QXTG3500 DS    0H                 GO ON TO NEXT STEP
         ENTNC QTP1               ENTER NEXT SEGMENT FOR PROCESSING
         EJECT 1
***********
*  EXIT PROCESSING - CHECK FOR EXTRA OPTS SPECIFIED ON COMMAND
***********
QXTG9000 DS    0H                      TELL USER OF IGNORED OPTS
         TM    BLKIND,VALSPEC          USER SPECIFIED OPTION?
         JNO   QXTG9010                NO, SKIP TO NEXT CHECK           64MODE
         TM    BLKIND,VALOPTN          VALID OPTION?
         JO    QXTG9010                OK, SKIP TO NEXT CHECK           64MODE
         NI    BLKIND,X'FF'-VALSPEC    BAD VALUE - TURN OFF INDICATOR
         XC    BLKDEF,BLKDEF           CLEAR OUT BAD VALUE
         LA    R2,QXTGM72
         BRAS  R4,QXTGWTPC             SEND ERROR MESSAGE               64MODE
QXTG9010 DS    0H
         TM    TAPIND,VALSPEC          USER SPECIFIED OPTION?
         JNO   QXTG9020                NO, SKIP TO NEXT CHECK           64MODE
         TM    TAPIND,VALOPTN          VALID OPTION?
         JO    QXTG9020                OK, SKIP TO NEXT CHECK           64MODE
         NI    TAPIND,X'FF'-VALSPEC    BAD VALUE - TURN OFF INDICATOR
         XC    CHARTAPE,CHARTAPE       CLEAR OUT BAD VALUE
         LA    R2,QXTGM73
         BRAS  R4,QXTGWTPC             SEND ERROR MESSAGE               64MODE
QXTG9020 DS    0H
         TM    OPENIND,VALSPEC         USER SPECIFIED OPTION?
         JNO   QXTG9030                NO, SKIP TO NEXT CHECK           64MODE
         TM    OPENIND,VALOPTN         VALID OPTION?
         JO    QXTG9030                OK, SKIP TO NEXT CHECK           64MODE
         NI    OPENIND,X'FF'-VALSPEC   BAD VALUE - TURN OFF INDICATOR
         XC    OPENDEF,OPENDEF         CLEAR OUT BAD VALUE
         LA    R2,QXTGM74
         BRAS  R4,QXTGWTPC             SEND ERROR MESSAGE               64MODE
QXTG9030 DS    0H
         TM    CLOSEIND,VALSPEC        USER SPECIFIED OPTION?
         JNO   QXTG9040                NO, SKIP TO NEXT CHECK           64MODE
         TM    CLOSEIND,VALOPTN        VALID OPTION?
         JO    QXTG9040                OK, SKIP TO NEXT CHECK           64MODE
         NI    CLOSEIND,X'FF'-VALSPEC  BAD VALUE - TURN OFF INDICATOR
         XC    CLOSEDEF,CLOSEDEF       CLEAR OUT BAD VALUE
         LA    R2,QXTGM75
         BRAS  R4,QXTGWTPC             SEND ERROR MESSAGE               64MODE
QXTG9040 DS    0H
         TM    BUFFIND,VALSPEC         USER SPECIFIED OPTION?
         JNO   QXTG9060                NO, SKIP TO NEXT CHECK           64MODE
         TM    BUFFIND,VALOPTN         VALID OPTION?
         JO    QXTG9060                OK, SKIP TO NEXT CHECK           64MODE
         NI    BUFFIND,X'FF'-VALSPEC   BAD VALUE - TURN OFF INDICATOR
         XC    BUFFDEF,BUFFDEF         CLEAR OUT BAD VALUE
         LA    R2,QXTGM77
         BRAS  R4,QXTGWTPC             SEND ERROR MESSAGE               64MODE
QXTG9060 DS    0H
         TM    CNTIND,VALSPEC          USER SPECIFIED OPTION?
         JNO   QXTG9070                NO, SKIP TO NEXT CHECK           64MODE
         TM    CNTIND,VALOPTN          VALID OPTION?
         JO    QXTG9070                OK, SKIP TO NEXT CHECK           64MODE
         NI    CNTIND,X'FF'-VALSPEC    BAD VALUE - TURN OFF INDICATOR
         XC    CNTDEF,CNTDEF           CLEAR OUT BAD VALUE
         LA    R2,QXTGM78
         BRAS  R4,QXTGWTPC             SEND ERROR MESSAGE               64MODE
QXTG9070 DS    0H
         TM    FILLIND,VALSPEC         USER SPECIFIED OPTION?
         JNO   QXTG9080                NO, SKIP TO NEXT CHECK           64MODE
         TM    FILLIND,VALOPTN         VALID OPTION?
         JO    QXTG9080                OK, SKIP TO NEXT CHECK           64MODE
         NI    FILLIND,X'FF'-VALSPEC   BAD VALUE - TURN OFF INDICATOR
         XC    FILLDEF,FILLDEF         CLEAR OUT BAD VALUE
         LA    R2,QXTGM79
         BRAS  R4,QXTGWTPC             SEND ERROR MESSAGE               64MODE
QXTG9080 DS    0H
         TM    OPCIND,VALSPEC          USER SPECIFIED OPTION?
         JNO   QXTG9090                NO, SKIP TO NEXT CHECK           64MODE
         TM    OPCIND,VALOPTN          VALID OPTION?
         JO    QXTG9090                OK, SKIP TO NEXT CHECK           64MODE
         NI    OPCIND,X'FF'-VALSPEC    BAD VALUE - TURN OFF INDICATOR
         XC    OPCDEF,OPCDEF           CLEAR OUT BAD VALUE
         LA    R2,QXTGM80
         BRAS  R4,QXTGWTPC             SEND ERROR MESSAGE               64MODE
QXTG9090 DS    0H
         TM    AUTOIND,VALSPEC         USER SPECIFIED OPTION?
         JNO   QXTG9100                NO, SKIP TO NEXT CHECK           64MODE
         TM    AUTOIND,VALOPTN         VALID OPTION?
         JO    QXTG9100                OK, SKIP TO NEXT CHECK           64MODE
         NI    AUTOIND,X'FF'-VALSPEC   BAD VALUE - TURN OFF INDICATOR
         XC    AUTODEF,AUTODEF         CLEAR OUT BAD VALUE
         LA    R2,QXTGM71
         BRAS  R4,QXTGWTPC             SEND ERROR MESSAGE               64MODE
QXTG9100 DS    0H
         TM    CYCLEIND,VALSPEC        USER SPECIFIED OPTION?
         JNO   QXTG9110                NO, SKIP TO NEXT CHECK           64MODE
         TM    CYCLEIND,VALOPTN        VALID OPTION?
         JO    QXTG9110                OK, SKIP TO NEXT CHECK           64MODE
         NI    CYCLEIND,X'FF'-VALSPEC  BAD VALUE - TURN OFF INDICATOR
         XC    CYCLEDEF,CYCLEDEF       CLEAR OUT BAD VALUE
         LA    R2,QXTGM71
         BRAS  R4,QXTGWTPC             SEND ERROR MESSAGE               64MODE
QXTG9110 DS    0H                      ALL DONE
         BR    R3
**
**  END OF MAINLINE CODE
**
         EJECT 1
**
**  EXAMINE AND SET OPTION VALUES
**
QXTG7000 DS    0H                 'BLOCK' OPTION
         ICM   R7,X'F',BLKTYPE    GET ADDR OF BLKTYPE PARM
         JZ    QXTG7100           NO VALUE, CONTINUE                    64MODE
         SPACE 2
         OI    BLKIND,VALSPEC     INDICATE OPTION WAS SPECIFIED
         LA    R3,BLK$TBL         SEARCH BLOCK TABLE FOR VALUE
         USING BLK$LIST,R3        ADDRESS DSECT FOR BLOCK VALUE LIST
         SGR   R15,R15                                                  64MODE
         IC    R15,0(R7)          GET LENGTH OF SPECIFIED VALUE
         BCTGR R15,0              SUBTRACT FOR EXECUTE COMPARE          64MODE
QXTG7010 DS    0H
         CLC   BLK$PARM,ENDLIST   END OF LIST YET?
         JE    QXTG7030           YES, VALUE NOT FOUND                  64MODE
         EX    R15,CLC$BLK        IS ENTERED NAME SAME AS THIS ENTRY
         JE    QXTG7020           YES, SET INDICATOR FROM TABLE         64MODE
         LA    R3,LBLK$TBL(R3)    INCREMENT TO NEXT ENTRY IN TABLE
         J     QXTG7010           AND ITERATE                           64MODE
QXTG7020 DS    0H
         MVC   BLKDEF,BLK$VAL     SET INDICATOR FOR BLOCK VALUE
         J     QXTG7100           CONTINUE TO NEXT OPTION CHECK         64MODE
QXTG7030 DS    0H
         LA    R2,QXTGM10         ERROR MESSAGE 10
         BRAS  R4,QXTGWTPC        SEND ERROR MESSAGE                    64MODE
         EXITC                    AND EXIT
CLC$BLK  CLC   BLK$PARM(0),1(R7)  IS ENTERED NAME SAME AS THIS ENTRY
         DROP  R3                 NO LONGER NEED R3 FOR DSECT
         EJECT 1
QXTG7100 DS    0H                 PROCESS TAPE NAME OPTION
         ICM   R7,X'F',TAPNAME    GET ADDR OF PARAMETER
         JZ    QXTG7200           NO VALUE, CONTINUE                    64MODE
         SPACE 2
         CLI   0(R7),L'CHARTAPE-1 PARAMETER LENGTH OK?
         JNE   QXTG7110           NO, SEND OUT ERROR MESSAGE            64MODE
         SPACE 2
         MVC   CHARTAPE(L'CHARTAPE-1),1(R7) SET CHARACTER TAPE NAME
         OI    TAPIND,VALSPEC     INDICATE OPTION WAS SPECIFIED
         J     QXTG7200           CONTINUE TO NEXT OPTION CHECK         64MODE
QXTG7110 DS    0H
         LA    R2,QXTGM11         ERROR MESSAGE 11
         BRAS  R4,QXTGWTPC        SEND ERROR MESSAGE                    64MODE
         EXITC                    AND EXIT
         EJECT 1
QXTG7200 DS    0H                 PROCESS 'OPEN' OPTION
         ICM   R7,X'F',OPENACT    GET ADDR OF PARAMETER
         JZ    QXTG7300           NO VALUE, CONTINUE                    64MODE
         SPACE 2
         OI    OPENIND,VALSPEC    INDICATE OPTION WAS SPECIFIED
         LA    R3,OPN$TBL         SEARCH PARAMETER TABLE FOR VALUE
         USING OPN$LIST,R3        ADDRESS DSECT FOR PARAMETER LIST
         SGR   R15,R15                                                  64MODE
         IC    R15,0(R7)          GET LENGTH OF SPECIFIED VALUE
         BCTGR R15,0              SUBTRACT FOR EXECUTE COMPARE          64MODE
QXTG7210 DS    0H
         CLC   OPN$PARM,ENDLIST   END OF LIST YET?
         JE    QXTG7230           YES, PARAMETER VALUE NOT FOUND        64MODE
         EX    R15,CLC$OPN        IS ENTERED NAME SAME AS THIS ENTRY
         JE    QXTG7220           YES, SET INDICATOR FROM TABLE         64MODE
         LA    R3,LOPN$TBL(R3)    INCREMENT TO NEXT ENTRY IN TABLE
         J     QXTG7210           AND ITERATE                           64MODE
QXTG7220 DS    0H
         MVC   OPENDEF,OPN$VAL    SET INDICATOR FOR VALUE
         J     QXTG7300           CONTINUE TO NEXT OPTION CHECK         64MODE
QXTG7230 DS    0H
         LA    R2,QXTGM12         ERROR MESSAGE 12
         BRAS  R4,QXTGWTPC        SEND ERROR MESSAGE                    64MODE
         EXITC                    TAKE ERROR EXIT
CLC$OPN  CLC   OPN$PARM(0),1(R7)  IS ENTERED NAME SAME AS THIS ENTRY
         DROP  R3                 NO LONGER NEED R3 FOR DSECT
         EJECT 1
QXTG7300 DS    0H                 PROCESS 'CLOSE' OPTION
         ICM   R7,X'F',CLOSEACT   GET ADDR OF PARAMETER
         JZ    QXTG7400           NO VALUE, CONTINUE                    64MODE
         SPACE 2
         OI    CLOSEIND,VALSPEC   INDICATE OPTION WAS SPECIFIED
         LA    R3,CLS$TBL         SEARCH PARAMETER TABLE FOR VALUE
         USING CLS$LIST,R3        ADDRESS DSECT FOR PARAMETER LIST
         SGR   R15,R15                                                  64MODE
         IC    R15,0(R7)          GET LENGTH OF SPECIFIED VALUE
         BCTGR R15,0              SUBTRACT FOR EXECUTE COMPARE          64MODE
QXTG7310 DS    0H
         CLC   CLS$PARM,ENDLIST   END OF LIST YET?
         JE    QXTG7330           YES, PARAMETER VALUE NOT FOUND        64MODE
         EX    R15,CLC$CLS        IS ENTERED NAME SAME AS THIS ENTRY
         JE    QXTG7320           YES, SET INDICATOR FROM TABLE         64MODE
         LA    R3,LCLS$TBL(R3)    INCREMENT TO NEXT ENTRY IN TABLE
         J     QXTG7310           AND ITERATE                           64MODE
QXTG7320 DS    0H
         MVC   CLOSEDEF,CLS$VAL   SET INDICATOR FOR VALUE
         J     QXTG7400           CONTINUE TO NEXT OPTION CHECK         64MODE
QXTG7330 DS    0H
         LA    R2,QXTGM13         ERROR MESSAGE 13
         BRAS  R4,QXTGWTPC        SEND ERROR MESSAGE                    64MODE
         EXITC                    TAKE ERROR EXIT
CLC$CLS  CLC   CLS$PARM(0),1(R7)  IS ENTERED NAME SAME AS THIS ENTRY
         DROP  R3                 NO LONGER NEED R3 FOR DSECT
         EJECT 1
QXTG7400 DS    0H                 'BUFFERED' OPTION
         ICM   R7,X'F',BUFFERED   GET ADDR OF PARAMETER
         JZ    QXTG7500           NO VALUE, CONTINUE                    64MODE
         SPACE 2
         OI    BUFFIND,VALSPEC    INDICATE OPTION WAS SPECIFIED
         CLC   1(1,R7),=C'Y'      BUFFERED=YES?
         JE    QXTG7410           YES, SET VALUE OF YES                 64MODE
         MVI   BUFFDEF,BUFFNO     OTHERWISE, SET VALUE OF NO
         J     QXTG7500           CONTINUE TO NEXT OPTION CHECK         64MODE
QXTG7410 DS    0H
         MVI   BUFFDEF,BUFFYES
         EJECT 1
QXTG7500 DS    0H                 'MSG' OPTION
         ICM   R7,X'F',MSGMODE    GET ADDR OF PARAMETER
         JZ    QXTG7600           NO VALUE, CONTINUE                    64MODE
         SPACE 2
         OI    MSGIND,VALSPEC     INDICATE OPTION WAS SPECIFIED
         LA    R3,MSG$TBL         SEARCH PARAMETER TABLE FOR VALUE
         USING MSG$LIST,R3        ADDRESS DSECT FOR PARAMETER LIST
         SGR   R15,R15                                                  64MODE
         IC    R15,0(R7)          GET LENGTH OF SPECIFIED VALUE
         BCTGR R15,0              SUBTRACT FOR EXECUTE COMPARE          64MODE
QXTG7510 DS    0H
         CLC   MSG$PARM,ENDLIST   END OF LIST YET?
         JE    QXTG7530           YES, PARAMETER VALUE NOT FOUND        64MODE
         EX    R15,CLC$MSG        IS ENTERED NAME SAME AS THIS ENTRY
         JE    QXTG7520           YES, SET INDICATOR FROM TABLE         64MODE
         LA    R3,LMSG$TBL(R3)    INCREMENT TO NEXT ENTRY IN TABLE
         J     QXTG7510           AND ITERATE                           64MODE
QXTG7520 DS    0H
         MVC   MSGDEF,MSG$VAL     SET INDICATOR FOR VALUE
         J     QXTG7600           CONTINUE TO NEXT OPTION CHECK         64MODE
QXTG7530 DS    0H
         LA    R2,QXTGM16         ERROR MESSAGE 16
         BRAS  R4,QXTGWTPC        SEND ERROR MESSAGE                    64MODE
         EXITC                    TAKE ERROR EXIT
CLC$MSG  CLC   MSG$PARM(0),1(R7)  IS ENTERED NAME SAME AS THIS ENTRY
         DROP  R3                 NO LONGER NEED R3 FOR DSECT
         EJECT 1
QXTG7600 DS    0H                 'COUNT' OPTION
         ICM   R7,X'F',COUNT      GET ADDR OF PARAMETER
         JZ    QXTG7700           NO VALUE, CONTINUE                    64MODE
QXTG7610 DS    0H
         OI    CNTIND,VALSPEC     INDICATE OPTION WAS SPECIFIED
         LLGF  R3,0(,R7)          GET COUNT VALUE INTO R3               64MODE
         ST    R3,CNTDEF          SAVE FOR LATER
         LTGR  R3,R3              DID WE GET COUNT=0?                   64MODE
         JNZ   QXTG7700           NO, CONTINUE TO NEXT OPTION CHK       64MODE
         LA    R2,QXTGM14         ERROR MESSAGE 14
         BRAS  R4,QXTGWTPC        SEND ERROR MESSAGE                    64MODE
         EXITC                    TAKE ERROR EXIT
         EJECT 1
QXTG7700 DS    0H                 'FILLCHAR' OPTION
         ICM   R7,X'F',FILLCHAR   GET ADDR OF PARAMETER
         JZ    QXTG7800           NO VALUE, CONTINUE                    64MODE
QXTG7710 DS    0H
         OI    FILLIND,VALSPEC    INDICATE OPTION WAS SPECIFIED
         SGR   R3,R3              CLEAR REGISTER FOR TESTS              64MODE
         IC    R3,1(,R7)          GET CHARACTER INTO R3
         STC   R3,FILLDEF         SAVE FOR LATER
         EJECT 1
QXTG7800 DS    0H                 'OPCODE' OPTION
         ICM   R7,X'F',OPCODE     GET ADDR OF PARAMETER
         JZ    QXTG7900           NO VALUE, CONTINUE                    64MODE
         SPACE 2
         OI    OPCIND,VALSPEC     INDICATE OPTION WAS SPECIFIED
         LA    R3,OPC$TBL
         USING OPC$LIST,R3        ADDRESS DSECT FOR OPCODE LIST
QXTG7810 DS    0H
         CLC   OPC$PARM,ENDLIST   END OF LIST YET?
         JE    QXTG7830           YES, OPCODE NOT FOUND                 64MODE
         CLC   OPC$PARM,1(R7)     IS ENTERED PARM SAME AS THIS ENTRY
         JE    QXTG7820           YES, SET INDICATOR FROM TABLE         64MODE
         LA    R3,LOPC$TBL(R3)    INCREMENT TO NEXT ENTRY IN TABLE
         J     QXTG7810           AND ITERATE                           64MODE
QXTG7820 DS    0H
         MVC   OPCDEF,OPC$VAL     SET INDICATOR FOR OPCODE
         J     QXTG7900           CONTINUE TO NEXT OPTION CHECK         64MODE
         DROP  R3                 NO LONGER NEED R3 FOR DSECT
         SPACE 2
QXTG7830 DS    0H                 NO OP CODES LEFT
         LA    R2,QXTGM15         ERROR MESSAGE 15
         BRAS  R4,QXTGWTPC        SEND ERROR MESSAGE                    64MODE
         EXITC                    TAKE ERROR EXIT
         EJECT 1
QXTG7900 DS    0H                 'AUTO' OPTION
         ICM   R7,X'F',AUTOOPT    GET ADDR OF PARAMETER
         JZ    QXTG8000           NO VALUE, CONTINUE                    64MODE
         OI    AUTOIND,VALSPEC    INDICATE OPTION WAS SPECIFIED
         CLC   1(1,R7),=C'Y'      AUTO=YES?
         JE    QXTG7910           YES, SET VALUE OF YES                 64MODE
         MVI   AUTODEF,AUTONO     OTHERWISE, SET VALUE OF NO
         J     QXTG8000           CONTINUE TO NEXT OPTION CHECK         64MODE
QXTG7910 DS    0H
         MVI   AUTODEF,AUTOYES
         EJECT 1
QXTG8000 DS    0H                 NO MORE OPTIONS
         ICM   R7,X'F',CYCLEOPT   GET ADDR OF PARAMETER
         JZ    QXTG8100           NO VALUE, CONTINUE                    64MODE
         OI    CYCLEIND,VALSPEC   INDICATE OPTION WAS SPECIFIED
         CLC   1(1,R7),=C'Y'      AUTO=YES?
         JE    QXTG8010           YES, SET VALUE OF YES                 64MODE
         MVI   CYCLEDEF,CYCLENO   OTHERWISE, SET VALUE OF NO
         J     QXTG8100           CONTINUE TO NEXT OPTION CHECK         64MODE
QXTG8010 DS    0H
         MVI   CYCLEDEF,CYCLEYES
QXTG8100 DS    0H                 NO MORE OPTIONS
QXTG8990 DS    0H                 RETURN TO CALLER
         BR    R1
*
         EJECT 1
QXTG9985 DS    0H                 HELP TEXT                   @TEDMLIN
         MVI   TAPEFUNC,FUNCHLPO                              @TEDMLIN
         J     QXTG9999           GENERATE MESSAGE            @TEDMLIN  64MODE
QXTG9990 DS    0H                 HELP TEXT
         MVI   TAPEFUNC,FUNCHLPF
         J     QXTG9999           GENERATE MESSAGE                      64MODE
QXTG9995 DS    0H                 HELP TEXT
         MVI   TAPEFUNC,FUNCHLPS
QXTG9999 DS    0H                 HELP TEXT
         ENTRC QXTK
         EXITC ,
         EJECT 1
*
*  Messages
*
QXTGM10  GENMSG LET=E,NUM=10,TEXT='BLOCK size must be 127, 381, 1055 orX
                4K'
QXTGM11  GENMSG LET=E,NUM=11,TEXT='TAPE parameter must be 3 characters'
QXTGM12  GENMSG LET=E,NUM=12,TEXT='OPEN parameter must be ASSIGN, INPUTX
                OUTPUT or NO'
QXTGM13  GENMSG LET=E,NUM=13,TEXT='CLOSE parameter must be RESERVE, YESX
                or EXIT'
QXTGM14  GENMSG LET=E,NUM=14,TEXT='COUNT parameter cannot be 0'
QXTGM15  GENMSG LET=E,NUM=15,TEXT='OPCODE value not valid - not found'
QXTGM16  GENMSG LET=E,NUM=16,TEXT='MSG parameter must be PARMLIST, WTOPX
               C, IMSG, EMSG, or ALL'
QXTGM71  GENMSG LET=W,NUM=71,TEXT='AUTO     option ignored for this comX
               mand'
QXTGM72  GENMSG LET=W,NUM=72,TEXT='BLOCK    option ignored for this comX
               mand'
QXTGM73  GENMSG LET=W,NUM=73,TEXT='TAPE     option ignored for this comX
               mand'
QXTGM74  GENMSG LET=W,NUM=74,TEXT='OPEN     option ignored for this comX
               mand'
QXTGM75  GENMSG LET=W,NUM=75,TEXT='CLOSE    option ignored for this comX
               mand'
QXTGM76  GENMSG LET=W,NUM=76,TEXT='MSG      option ignored for this comX
               mand'
QXTGM77  GENMSG LET=W,NUM=77,TEXT='BUFFERED option ignored for this comX
               mand'
QXTGM78  GENMSG LET=W,NUM=78,TEXT='COUNT    option ignored for this comX
               mand'
QXTGM79  GENMSG LET=W,NUM=79,TEXT='FILLCHAR option ignored for this comX
               mand'
QXTGM80  GENMSG LET=W,NUM=80,TEXT='OPCODE   option ignored for this comX
               mand'
QXTGM81  GENMSG LET=W,NUM=80,TEXT='CYCLE    option ignored for this comX
               mand'
*
*  Skeleton WTOPC macros
*
         DCTMSG
         USING DCTMSG,R2
QXTGWTPC WTOPC PREFIX=TAPE,HEADER=YES,TIME=NO,CHAIN=YES,ENDOFM=YES,    X
               DOT=YES,                                                X
               NUMA=MSGNUM,                                            X
               LETA=MSGLET,                                            X
               TEXTA=MSGLEN,                                           X
               COMP=YES,                                               X
               EBROUT=YES,                                             X
               ROUT=(TAPE) ROUTE TO PRC IF NOT ALREADY
         DROP  R2
         BR    R4
*
         EJECT 1
MAINBPK  BPKDC (PSR,ACTION,ACTBPP)
         SPACE 2
ACTBPP   BPPSC (HELPSYS,4),                                            X
               (HELPFUNC,5),                                           X
               (HELPOTH,5),                                            X
               (BACKSPACE,1,OPTBPK),                                   X
               (INPUT,1,OPTBPK),                                       X
               (OUTPUT,1,OPTBPK),                                      X
               (REWIND,1,OPTBPK),                                      X
               (SYNC,2,OPTBPK),                                        X
               (QUERY,1,OPTBPK),                                       X
               (WRITE,1,OPTBPK),                                       X
               (DUMP,1,OPTBPK),                                        X
               (CCW,,OPTBPK),                                          X
               (SWITCH,3,OPTBPK),                                      X
               (VERIFY,1,OPTBPK),                                      X
               (SETUP,2),                                              X
               (START,4,SYSBPK1),                                      X
               (STOP,3,SYSBPK2),                                       X
               (STATUS,4),                                             X
               (PAUSE,1),                                              X
               (RESTART,4),                                            X
               (ABORT,1,OPTBPK),                                       X
               (CONTROL,3,SYSBPK3),                                    X
               (OTEST1,6)                                     @TEDMLIN
         SPACE 2
OPTBPK   BPKDC (PSR,ACTION,ACTBPP),                                    X
               (K,MSG,,8),                                             X
               (K,BLOCK,2,4),                                          X
               (KD,COUNT,2,9),                                         X
               (K,TAPE,2,3),                                           X
               (K,OPEN,3,6),                                           X
               (K,CLOSE,2,7),                                          X
               (KS,BUFFERED,2),                                        X
               (KX,FILLCHAR,1,2),                                      X
               (K,OPCODE,3,3),                                         X
               (KS,AUTO,1),                                            X
               (KS,CYCLE,2)
         SPACE 2
SYSBPK1  BPKDC (PSR,ACTION,ACTBPP),                                    X
               (PR,TAPENAME,3)
         SPACE 2
SYSBPK2  BPKDC (PSR,ACTION,ACTBPP),                                    X
               (PR,TAPENAME,3),                                        X
               (KS,BP)
         SPACE 2
SYSBPK3  BPKDC (PSR,ACTION,ACTBPP),                                    X
               (KS,RTA),                                               X
               (K,MSGMODE,3,8),                                        X
               (KD,CRETC,2,2),                                         X
               (KD,MINCOUNT,3,5),                                      X
               (KD,MAXCOUNT,3,5)
*
*  ASSIGN MEANINGFUL NAMES TO THE LABELS GENERATED BY BPKDC
*
ACTION   EQU   BPKOPM1        ACTION PARM AT LABEL BPKOPM1
MSGMODE  EQU   BPKOPM2        KEYWORD   POINTER AT LABEL BPKOPM2
BLKTYPE  EQU   BPKOPM3        KEYWORD   POINTER AT LABEL BPKOPM3
COUNT    EQU   BPKOPM4        KEYWORD   POINTER AT LABEL BPKOPM4
TAPNAME  EQU   BPKOPM5        KEYWORD   POINTER AT LABEL BPKOPM5
OPENACT  EQU   BPKOPM6        KEYWORD   POINTER AT LABEL BPKOPM6
CLOSEACT EQU   BPKOPM7        KEYWORD   POINTER AT LABEL BPKOPM7
BUFFERED EQU   BPKOPM8        KEYWORD   POINTER AT LABEL BPKOPM8
FILLCHAR EQU   BPKOPM9        KEYWORD   POINTER AT LABEL BPKOPM9
OPCODE   EQU   BPKOPM10       KEYWORD   POINTER AT LABEL BPKOPM10
AUTOOPT  EQU   BPKOPM11       KEYWORD   POINTER AT LABEL BPKOPM11
CYCLEOPT EQU   BPKOPM12       KEYWORD   POINTER AT LABEL BPKOPM12
         EJECT 1
**
** TABLES USED TO PROCESS USER OPTIONS
**
         SPACE 5
BLK$TBL  DS    0F
         DC    CL(L'BLK$PARM)'127',AL1(L0)
         DC    CL(L'BLK$PARM)'381',AL1(L1)
         DC    CL(L'BLK$PARM)'1055',AL1(L2)
         DC    CL(L'BLK$PARM)'4K',AL1(L4)
         DC    (L'BLK$PARM)AL1(EX),AL1(0)
         SPACE 5
OPN$TBL  DS    0F
         DC    CL(L'OPN$PARM)'INPUT',AL1(OPENI)
         DC    CL(L'OPN$PARM)'OUTPUT',AL1(OPENO)
         DC    CL(L'OPN$PARM)'ASSIGN',AL1(OPENA)
         DC    CL(L'OPN$PARM)'NO',AL1(OPENN)
         DC    (L'OPN$PARM)AL1(EX),AL1(0)
         SPACE 5
CLS$TBL  DS    0F
         DC    CL(L'CLS$PARM)'YES',AL1(CLOSEY)
         DC    CL(L'CLS$PARM)'RESERVE',AL1(CLOSER)
         DC    CL(L'CLS$PARM)'EXIT',AL1(CLOSEE)
         DC    (L'CLS$PARM)AL1(EX),AL1(0)   END OF LIST INDICATOR
         SPACE 5
         DS    0F
MSG$TBL  DC    CL(L'MSG$PARM)'EMSG',AL1(MSGEMSG)
         DC    CL(L'MSG$PARM)'ALL',AL1(MSGALL)
         DC    CL(L'MSG$PARM)'PARMLIST',AL1(MSGPARM)
         DC    CL(L'MSG$PARM)'WTOPC',AL1(MSGWTP)
         DC    CL(L'MSG$PARM)'IMSG',AL1(MSGIMSG)
         DC    (L'MSG$PARM)AL1(EX),AL1(0)   END OF LIST INDICATOR
         SPACE 5
OPC$TBL  DS    0F
         DC    C'WRT',AL1(OPCWRT)           HANDLED BY TDTAC
         DC    C'RDF',AL1(OPCRDF)
         DC    C'SNS',AL1(OPCSNS)
         DC    C'RDB',AL1(OPCRDB)
         DC    C'BID',AL1(OPCBID)
         DC    C'RBL',AL1(OPCRBL)
         DC    C'SYN',AL1(OPCSYN)
         DC    C'LBL',AL1(OPCLBL)
         DC    C'SID',AL1(OPCSID)
         SPACE 1
         DC    C'NOP',AL1(OPCNOP)           HANDLED BY TPCNC
         DC    C'REW',AL1(OPCREW)
         DC    C'RUN',AL1(OPCRUN)
         DC    C'ERG',AL1(OPCERG)
         DC    C'WTM',AL1(OPCWTM)
         DC    C'BSB',AL1(OPCBSB)
         DC    C'BSF',AL1(OPCBSF)
         DC    C'FSB',AL1(OPCFSB)
         DC    C'FSF',AL1(OPCFSF)
         DC    C'DD0',AL1(OPCDD0)
         DC    C'DD2',AL1(OPCDD2)
         DC    C'DD1',AL1(OPCDD1)
         DC    (L'OPC$PARM)AL1(EX),AL1(0)   END OF LIST INDICATOR
         SPACE 2
* CONSTANTS FOR COMPARISONS
ZERO     DC    F'0'
EX       EQU   C'X'
ENDLIST  DC    16AL1(EX)          LOTS OF X'S TO COMPARE AGAINST
         EJECT 1
         PRINT GEN
         TA0EQ GLOB=NO
         PRINT NOGEN
         LTORG
         FINIS QXTG
         END   ,
