         PRINT NOGEN
         BEGIN NAME=QXTH,AMODE=64
***********************************************************************
*       Licensed Materials - Property of IBM                          *
*       "Restricted Materials of IBM"                                 *
*       5748-T15                                                      * 
*       Copyright IBM Corp. 2003, 2006                                *
*       All Rights Reserved                                           *
*       US Government Users Restricted Rights - Use, duplication      *
*       or dislosure restricted by GSA ADP Schedule Contract          *
*       with IBM Corp.                                                *
***********************************************************************
*  This file is provided on an "as is" basis.  There are no           *
*  warranties, express or implied, including the implied warranties   *
*  of merchantability and fitness for a particular purpose.           *
*  IBM has no obligation to provide service, defect correction, or    *
*  any maintenance for the file.  IBM has no obligation to supply     *
*  any updates or enhancements for the file to you even if such are   *
*  or later become available.                                         *
***********************************************************************
 TITLE 'QXTH - GENERAL TAPE DRIVER'
***********************************************************************
*
* MODULE NAME..... QXTH
* DESCRIPTION..... GENERAL TAPE DRIVER
* RELATED MODULE.. QXTG (ENTNC FROM CVZZ)
*                  QTP2
*                  QXTI
*                  QXTK
*                  QTP5
*                  QTP6
*                  QTP7
*                  QTP8
*                  QTP9
* DOCUMENT NAME... GENERAL TAPE DRIVER USERS GUIDE
* LEVEL........... N/A
*
* FUNCTION...      THIS SEGMENT ACTS AS A DISTRIBUTION POINT TO
*                  WHICH OTHER SEGMENTS ARE ENTERED BASED ON THE
*                  REQUESTED FUNCTION.  THE COMMAND IS EITHER
*                  HANDLED HERE DIRECTLY, OR ANOTHER SEGMENT IS
*                  ENTERED TO HANDLE THE REQUEST.
*
*                  THE FOLLOWING FUNCTION TEST COMMANDS ARE DIRECTLY
*                  HANDLED IN THIS SEGMENT.
*                   - ABORT
*                   - CCW
*                   - SYNC
*                   - DUMP
*
* MODULE ATTRIBUTES..
*   TYPE.......... 'E' (ECB CONTROLLED)
*   DEPENDENCY.... NONE (OBJECT SHIPPABLE)
*   ALLOCATION.... 4K FILE RESIDENT
*   ENVIRONMENT... BASE, BSS
*   ENTRY POINT... QXTH  (ENTNC FROM QXTG)
*
* ECB-CONVENTIONS.......
*
*   WORK AREAS
*         EBX000-103    SEE EQUATE MACRO TA0EQ
*
* DATA LEVELS
*         D1 - WORK BLOCK (INPUT OR OUTPUT)
*         D2 - SCRATCH - USED TO READ CORE LEVELS
*
* REGISTER-CONVENTIONS..
*         R0 -
*         R1 - SCRATCH - DECIMAL 'COUNT' VALUE
*              ALSO BASE OF TAPE LABEL LIST
*         R2 - POINTER TO WTOPC MESSAGE
*         R3 - GLOBALS BASE
*         R4 - POINTER TO OUTPUT BLOCK
*            - POINTER TO INPUT BLOCK
*         R5 - WORK REG
*         R6 -
*         R7 - BAS REG
*         R14 - WORK REG
*             - USED BY WTOPC
*         R15 - WORK REG
*             - USED BY WTOPC
*
**********************************************************************
** CHANGE ACTIVITY...
**
**        UPDATE BY      PLOG #   DATE   DESCRIPTION
**        ---------      -------  ------ ------------
**        KENT           TD00301  071390 MSG SETTING ENHANCEMENTS
**        KENT           TD00346  022891 TEP PROJECT IMPROVEMENTS
**        KENT           TD00368  070391 ADD TO TDTAC DISPLAY
**        KENT           TD00368  070791 FIX RBL OPCODE
**        MKENT          TD00578  062194 ADD TDCTC SUPPORT
**        LYNNEK         TD00624  032495 FIX TDCTC SUPT: 31-BIT ADR
**        YMLI           D14596   080106 Fix the dump problem
**        YMLI           F14612   081506 The output routing
**        YMLI           F14827   092106 Fix the COUNT < 15 of
**                                       'write' CCW dump problem
**        YMLI           D14945   112706 'ZTEST TA64 START tape_label'
**                                     should exercise TA64 test cases
***********************************************************************
         EJECT
         CW0CW ,
*---------------------------------------------------------------------*
* INSERT DATE AND TIME OF MOST CURRENT ASSEMBLY FOR DEBUGGING PURPOSES
*---------------------------------------------------------------------*
         J     QXTH0000                                                 64MODE
         DC    CL8'&SYSDATE'
         DC    CL1' '
         DC    CL8'&SYSTIME'
*
         SPACE 1
QXTH0000 DS    0H
*---------------------------------------------------------------------*
* ESTABLISH INDEX TO GLOBAL AREA FOR INDICATED TAPE NAME
*---------------------------------------------------------------------*
         SPACE 2
         CLC   CHARTAPE,ZERO           TAPE NAME SPECIFIED?
         JE    QXTH0100                NO, SKIP GLOBALS STUFF           64MODE
         CLI   ACTCMD,QUERYCMD         NON-TRACKED COMMAND?
         JE    QXTH0100                YES, SKIP GLOBALS STUFF          64MODE
         CLI   ACTCMD,CPSWCMD          NON-TRACKED COMMAND?
         JE    QXTH0100                YES, SKIP GLOBALS STUFF          64MODE
         SPACE 2
         MVI   TAPEFUNC,FUNC@EGE       ESTABLISH GLOBAL TAPE ENTRY
         ENTRC QXTK                    CALL SEGMENT TO DO FUNCTION
         LTGR  R5,R5                   WAS THE RETURN CODE 0?           64MODE
         JZ    QXTH0100                YES, CONTINUE NORMALLY           64MODE
         SPACE 1
         CGHI  R5,GLOBDUPL             DUPLICATE TAPE NAME?             64MODE
         JE    QXTH0100                YES, OK IN THIS INSTANCE         64MODE
         SPACE 1
         XC    TAPEGLOB,TAPEGLOB       ENSURE INDEX CLEARED
         SPACE 1
         CGHI  R5,GLOBFULL             TOO MANY ENTRIES?                64MODE
         JE    QXTH0100                YES, OK IN THIS INSTANCE         64MODE
         CGHI  R5,GLOBNULL             GLOBALS NOT AVAILABLE?           64MODE
         JE    QXTH0100                YES, OK IN THIS INSTANCE         64MODE
         SPACE 1
         J     QXTHX000                OTHERWISE, ANALYZE RETURN CODE   64MODE
         SPACE 5
*---------------------------------------------------------------------*
* FIRST CHECK FOR COMMANDS HANDLED OUTSIDE OF THIS ROUTINE
*---------------------------------------------------------------------*
QXTH0100 DS    0H
         CLI   ACTCMD,VERFYCMD         IS VERIFY REQUESTED?
         JE    QXTHVFY                 YES - GO THERE                   64MODE
         SPACE 2
         CLI   ACTCMD,INPUTCMD         IS INPUT REQUESTED?
         JE    QXTHINP                 YES - GO THERE                   64MODE
         CLI   ACTCMD,BKSPCMD          IS BACKSPACE REQUESTED?
         JE    QXTHBCK                 YES - GO THERE                   64MODE
         CLI   ACTCMD,REWCMD           IS REWIND REQUESTED?
         JE    QXTHREW                 YES - GO THERE                   64MODE
         SPACE 2
         CLI   ACTCMD,OUTPTCMD         IS OUTPUT REQUESTED?
         JE    QXTHOUT                 YES - GO THERE                   64MODE
         CLI   ACTCMD,WRITECMD         IS WRITE REQUESTED?
         JE    QXTHWRT                 YES - GO THERE                   64MODE
         SPACE 2
         CLI   ACTCMD,QUERYCMD         IS QUERY REQUESTED?
         JE    QXTHQRY                 YES - GO THERE                   64MODE
         CLI   ACTCMD,CPSWCMD          IS TAPE SWITCH REQUESTED?
         JE    QXTHSWT                 YES - GO THERE                   64MODE
         SPACE 5
*---------------------------------------------------------------------*
* CONTINUE WITH COMMANDS HANDLED IN THIS SEGMENT
*---------------------------------------------------------------------*
         SPACE 2
         USING TAPRCD,R4
         SPACE 2
         CLC   TAPEGLOB,ZERO           GLOBAL INDEX = 0?
         JE    QXTHCMDS                YES, DON'T USE GLOBALS           64MODE
         SPACE 2
         CLI   ACTCMD,ABORTCMD         COMMAND = ABORT?
         JE    QXTHCMDS                YES, DON'T TRACK USAGE           64MODE
         SPACE 2
         MVI   TAPEFUNC,FUNC@SET       MARK COMMAND START
         ENTRC QXTK                    CALL SEGMENT TO DO FUNCTION
         LTGR  R5,R5                   WAS THE RETURN CODE NON-0?       64MODE
         JNZ   QXTHX000                YES, ANALYZE THE RETURN CODE     64MODE
         EJECT 1
QXTHCMDS DS    0H
         SPACE 1
*---------------------------------------------------------------------*
* DETERMINE ACTION TO BE TAKEN
*---------------------------------------------------------------------*
         CLI   ACTCMD,SYNCCMD          IS SYNC TAPE REQUESTED?
         JE    QXTHSYN                 YES - GO THERE                   64MODE
         CLI   ACTCMD,DUMPCMD          IS DUMP REQUESTED?
         JE    QXTHDMP                 YES - GO THERE                   64MODE
         CLI   ACTCMD,CCWCMD           IS CCW REQUESTED?
         JE    QXTHCCW                 YES - GO THERE                   64MODE
         CLI   ACTCMD,ABORTCMD         IS ABORT REQUESTED?
         JE    QXTHABT                 YES - GO THERE                   64MODE
         J     QXTHERR2                NO - INTERNAL ERROR              64MODE
         EJECT 1
***********************************************************************
* ROUTINE: QXTHVFY
*
*          VERIFY DATA ON A PREVIOUSLY WRITTEN GENERAL TAPE
***********************************************************************
         SPACE
QXTHVFY  DS    0H
         ENTNC QTP5
         EJECT 1
***********************************************************************
* ROUTINE: QXTHINP
*
*          READ FROM A GENERAL TAPE THE REQUESTED NUMBER OF RECORDS.
***********************************************************************
         SPACE
QXTHINP  DS    0H
         ENTNC QTP6
         SPACE 5
***********************************************************************
* ROUTINE: QXTHBCK
*
*          BACKSPACE GEN TAPE REQUESTED RECORDS
***********************************************************************
         SPACE
QXTHBCK  DS    0H
         ENTNC QTP6
         SPACE 5
***********************************************************************
* ROUTINE: QXTHREW
*
*          REWIND THE GENERAL TAPE
***********************************************************************
         SPACE
QXTHREW  DS    0H
         ENTNC QTP6
         EJECT 1
         SPACE 5
***********************************************************************
* ROUTINE: OUTPUT
*
*          OUTPUT TO EITHER THE RTA/RTL OR A GENERAL TAPE
***********************************************************************
         SPACE
QXTHOUT  DS    0H
         ENTNC QTP7
         SPACE 5
***********************************************************************
* ROUTINE: QXTHWRT
*
*          WRITE TO THE RTA/RTL TAPE USING TOUTC MACRO
***********************************************************************
         SPACE
QXTHWRT  DS    0H
         ENTNC QTP7
         EJECT 1
***********************************************************************
* ROUTINE: QXTHQRY
*
*          QUERY THE CURRENT STATUS OF THE GIVEN TAPE.
***********************************************************************
         SPACE
QXTHQRY  DS    0H
         ENTNC QTP8
         EJECT 1
***********************************************************************
* ROUTINE: QXTHSWT
*
*          FORCE A CP TAPE SWITCH, EITHER REALTIME OR GENERAL TAPE.
***********************************************************************
         SPACE
QXTHSWT  DS    0H
         ENTNC QTP9
         EJECT 1
***********************************************************************
* ROUTINE: QXTHSYN
*
*          SYNCHRONIZE 3480 TAPE (NOP  3420 TAPE)
***********************************************************************
         SPACE
QXTHSYN  DS    0H
         BRAS  R7,QXTHOPN              OPEN TAPE AS REQUESTED           64MODE
         LA    R15,CHARTAPE            POINT TO TAPE NAME
         CLI   AUTODEF,AUTOYES         AUTO=YES?
         JNE   QXTHSYN2                NO                               64MODE
         SPACE 5
QXTHSYN1 DS    0H
         TSYNC NAME=(R15),AUTO=YES     PERFORM OPERATION
         WAITC QXTHERR1                WAIT FOR COMPLETION
         J     QXTHCLS                 CLOSE TAPE                       64MODE
         SPACE 2
QXTHSYN2 DS    0H
         TSYNC NAME=(R15),AUTO=NO      PERFORM OPERATION
         WAITC QXTHERR1                WAIT FOR COMPLETION
         J     QXTHCLS                 CLOSE TAPE                       64MODE
         EJECT
***********************************************************************
* ROUTINE: QXTHDMP
*
*          TAKE THE REQUESTED NUMBER OF DUMPS
***********************************************************************
         SPACE
QXTHDMP  DS    0H
         LLGF  R1,CNTDEF               GET RECORD COUNT INTO R1         64MODE
QXTHDMP1 DS    0H
         SERRC R,C0DE00,,,ECB          TAKE DUMP
         DLAYC ,                       DELAY A LITTLE
         BRCTG R1,QXTHDMP1             LOOP UNTIL DONE                  64MODE
         J     QXTHEXIT                EXIT THIS SEGMENT                64MODE
         EJECT
***********************************************************************
* ROUTINE: QXTHCCW
*
*          BUILD CCW FOR OPERATION ON GENERAL TAPE
***********************************************************************
         SPACE
QXTHCCW  DS    0H
         BRAS  R7,QXTHOPN                   MAKE TAPE AVAILABLE         64MODE
         GETCC D1,L4                        GET 4K BLOCK
*  ZERO OUT BLOCK BEFORE USE
         LLGF  R4,CE1CR1                    POINTER TO BLOCK            64MODE
         LGH   R5,CE1CC1                    SET LEN TO CLEAR            64MODE
         SGR   R15,R15                      CLEAR R15                   64MODE
         MVCL  R4,R14                       INITIALIZE BLK
         LLGF  R4,CE1CR1                    RESTORE POINTER TO BLOCK    64MODE
*
         CLI   OPCDEF,OPCWRT                IS THIS A WRITE?
         JNE   QXTHCCW1                     NO - DONT FILL BLOCK        64MODE
         SPACE 2
         LLGF  R1,CNTDEF                    GET RECORD COUNT INTO R1    64MODE
         C     R1,=F'4096'                  IS RECORD COUNT > 4096 ?    64MODE
         BH    QXTHERR7                     Yes, error                  64MODE
         L     R6,OVERHEAD                  GET OVERHEAD INTO R6        64MODE
         CR    R1,R6                        IS R1 > R6?                 64MODE
         BH    REGOK                        JUMP TO REGOK
         LR    R1,R6                        SET R1 TO R6
REGOK    CVD   R1,TA0WORK3                  CREATE A DECIMAL REC NUM
         SGF   R1,OVERHEAD                  LENGTH OF DATA SO FAR       64MODE
         LA    R5,TAPFILL(R1)               CALCULATE BLOCK OFFSET
         MVC   0(L'TRAILER,R5),TRAILER      MOVE IN RECORD TRAILER
*
         UNPK  TAPRECN,TA0WORK3             MAKE IT EBCDIC
         OI    TAPRECN+L'TAPRECN-1,X'F0'    FIX SIGN BIT
*
         MVC   TAPRECID,=AL2(TP$RECID)      MOVE IN RECORD ID
         MVC   TAPHDR,TEST5                 COPY HEADER
*
         LGR   R5,R1                        TOTAL DATA LENGTH           64MODE
         LA    R14,TAPFILL                  ADDRESS TO START FILLING
         SGR   R15,R15                                                  64MODE
         IC    R15,FILLDEF                  FILL CHARACTER TO USE
         BRAS  R7,QXTHFIL                   FILL BLOCK                  64MODE
QXTHCCW1 DS    0H
         SPACE
         MVC   QXT3XCCW,CCW_SKEL            COPY CCW SKELETON @TD00578
         MVC   CW0CMD1+QXT3XCCW,OPCDEF      ... AND OPCODE    @TD00578
         LLGF  R5,CE1CR1                    GET BLK ADDRESS             64MODE
         CLI   OPCDEF,OPCRDB                READ BACKWARDS?
         JNE   QXTHCCW2                     NO, HANDLE NORMALLY         64MODE
         AGF   R5,CNTDEF                    ADD LENGTH                  64MODE
         BCTGR R5,R0                        SUBTRACT ONE                64MODE
QXTHCCW2 DS    0H
         ST    R5,CW0ADR1+QXT3XCCW          SET UP ADDR FIELD @TD00624
         LLGF  R5,CNTDEF                                                64MODE
         STH   R5,CW0BCT1+QXT3XCCW          SET UP COUNT FLD  @TD00578
         CLC   TAPE#RTA,CHARTAPE            USING REALTIME TAPE?
         JE    QXTHERR3                     YES, SEND OUT ERROR MSG     64MODE
         SPACE 2
         LA    R15,CHARTAPE                 POINT TO TAPE NAME
         SPACE 1
         CLI   OPCDEF,OPCNOP                NO-OPERATION COMMAND?
         JE    CCWTPCNC                     ISSUE MACRO TO EXECUTE      64MODE
         CLI   OPCDEF,OPCREW                REWIND?
         JE    CCWTPCNC                     ISSUE MACRO TO EXECUTE      64MODE
         CLI   OPCDEF,OPCRUN                REWIND/UNLOAD?
         JE    CCWTPCNC                     ISSUE MACRO TO EXECUTE      64MODE
         CLI   OPCDEF,OPCERG                ERASE GAPS?
         JE    CCWTPCNC                     ISSUE MACRO TO EXECUTE      64MODE
         CLI   OPCDEF,OPCWTM                WRITE TAPE MARKS?
         JE    CCWTPCNC                     ISSUE MACRO TO EXECUTE      64MODE
         CLI   OPCDEF,OPCBSB                BACKWARD SPACE BLOCK?
         JE    CCWTPCNC                     ISSUE MACRO TO EXECUTE      64MODE
         CLI   OPCDEF,OPCBSF                BACKWARD SPACE FILE?
         JE    CCWTPCNC                     ISSUE MACRO TO EXECUTE      64MODE
         CLI   OPCDEF,OPCFSB                FORWARD SPACE BLOCK?
         JE    CCWTPCNC                     ISSUE MACRO TO EXECUTE      64MODE
         CLI   OPCDEF,OPCFSF                FORWARD SPACE FILE?
         JE    CCWTPCNC                     ISSUE MACRO TO EXECUTE      64MODE
         CLI   OPCDEF,OPCDD0                SET SYSTEM DENSITY 0?
         JE    CCWTPCNC                     ISSUE MACRO TO EXECUTE      64MODE
         CLI   OPCDEF,OPCDD1                SET SYSTEM DENSITY 1?
         JE    CCWTPCNC                     ISSUE MACRO TO EXECUTE      64MODE
         CLI   OPCDEF,OPCDD2                SET SYSTEM DENSITY 2?
         JE    CCWTPCNC                     ISSUE MACRO TO EXECUTE      64MODE
         SPACE
         CLI   OPCDEF,OPCWRT                WRITE COMMAND?
         JE    CCWTDCTC                     ISSUE MACRO       @TD00578  64MODE
         CLI   OPCDEF,OPCRDF                READ COMMAND?
         JE    CCWTDCTC                     ISSUE MACRO       @TD00578  64MODE
         SPACE
         CLI   OPCDEF,OPCSNS                SENSE COMMAND?
         JE    CCWTDTAC                     ISSUE MACRO TO EXECUTE      64MODE
         CLI   OPCDEF,OPCRDB                READ BACKWARD?
         JE    CCWTDTAC                     ISSUE MACRO TO EXECUTE      64MODE
         CLI   OPCDEF,OPCBID                READ BLOCK ID?
         JE    CCWTDTAC                     ISSUE MACRO TO EXECUTE      64MODE
         CLI   OPCDEF,OPCRBL                READ BUFFERED LOG?
         JE    CCWTDTAC                     ISSUE MACRO TO EXECUTE      64MODE
         CLI   OPCDEF,OPCSYN                SYNCRONIZE?
         JE    CCWTDTAC                     ISSUE MACRO TO EXECUTE      64MODE
         CLI   OPCDEF,OPCLBL                LOCATE BLOCK?
         JE    CCWTDTAC                     ISSUE MACRO TO EXECUTE      64MODE
         CLI   OPCDEF,OPCSID                SENSE ID?
         JE    CCWTDTAC                     ISSUE MACRO TO EXECUTE      64MODE
         SPACE
         J     QXTHERR4                     UNKNOWN CCW OPCODE          64MODE
         SPACE 2
CCWTDTAC DS    0H
         MVC   QXT3MACN,=C'TDTAC'           Set macro name    @TD00578
         MVC   CE1FA1(L'QXT3XCCW),QXT3XCCW  Setup CCW string  @TD00578
         TDTAC NAME=(R15),LEVEL=D1,CCW1=YES                   @TD00578
         J     CCWWAITC                     Continue processing         64MODE
         SPACE 2                                              @TD00578
CCWTDCTC DS    0H                                             @TD00578
         MVC   QXT3MACN,=C'TDCTC'           Set macro name    @TD00578
         LA    R14,QXT3XCCW                 Setup CCW pointer @TD00578
         ST    R14,CE1FA1                   Store for TDCTC execution
         TDCTC NAME=(R15),LEVEL=D1,CCW1=YES                   @TD00578
         SPACE 1                                              @TD00578
CCWWAITC DS    0H                                             @TD00578
         WAITC QXTHERR1                     WAIT TO COMPLETE
         SPACE
         TM    MSGDEF,MSGWTP                WTOPC TO OPERATOR?
         JNO   QXTHCCW3                     NO - JUMP AROUND            64MODE
         LLGF  R4,CE1CR1                    SET UP PTR TO DATA          64MODE
         LA    R2,QXTHM64A
         BRAS  R7,QXTHWTP1                  SEND OUT MESSAGE            64MODE
         SPACE 1
         AGHI  R4,8                         SET UP PTR TO DATA+8        64MODE
         LA    R2,QXTHM64B
         BRAS  R7,QXTHWTP1                  SEND OUT NEXT MESSAGE       64MODE
         SPACE 1
         AGHI  R4,8                         SET UP PTR TO DATA+8        64MODE
         LA    R2,QXTHM64C
         BRAS  R7,QXTHWTP1                  SEND OUT NEXT MESSAGE       64MODE
         SPACE 1
         AGHI  R4,8                         SET UP PTR TO DATA+8        64MODE
         LA    R2,QXTHM64D
         BRAS  R7,QXTHWTP1                  SEND OUT NEXT MESSAGE       64MODE
         SPACE 1
         AGHI  R4,8                         SET UP PTR TO DATA+8        64MODE
         LA    R2,QXTHM64E
         BRAS  R7,QXTHWTP1                  SEND OUT NEXT MESSAGE       64MODE
         SPACE 1
         AGHI  R4,8                         SET UP PTR TO DATA+8        64MODE
         LA    R2,QXTHM64F
         BRAS  R7,QXTHWTP1                  SEND OUT NEXT MESSAGE       64MODE
         SPACE 1
         AGHI  R4,8                         SET UP PTR TO DATA+8        64MODE
         LA    R2,QXTHM64G
         BRAS  R7,QXTHWTP1                  SEND OUT NEXT MESSAGE       64MODE
         SPACE 1
         AGHI  R4,8                         SET UP PTR TO DATA+8        64MODE
         LA    R2,QXTHM64H
         BRAS  R7,QXTHWTP1                  SEND OUT NEXT MESSAGE       64MODE
         SPACE 1
         J     QXTHCCW3                     CONTINUE                    64MODE
         SPACE 2
CCWTPCNC DS    0H
         MVC   CE1FA1(L'QXT3XCCW),QXT3XCCW  Setup CCW string  @TD00578
         TPCNC NAME=(R15),LEVEL=D1,CCW1=YES                   @TD00578
         WAITC QXTHERR1                     WAIT TO COMPLETE
QXTHCCW3 DS    0H
         RELCC D1                           RELEASE CORE BLOCK
         J     QXTHCLS                      CLOSE TAPE IF NECESSARY     64MODE
         EJECT
         SPACE 5
***********************************************************************
* ROUTINE: QXTHABT
*
*          MARK THE GLOBAL AREA FOR THIS TAPE AS 'USER ABORTED'
***********************************************************************
         SPACE
QXTHABT  DS    0H
         CLC   TAPEGLOB,ZERO                TAPE FOUND IN TABLE?
         JE    QXTHERR5                     NO, INDICATE ERROR          64MODE
         PUSH  USING
         GLOBZ REGR=R3
         LLGF  R3,@ISSDRV                                               64MODE
         SD0RV REG=R3
         SPACE 2
         SGR   R15,R15                                                  64MODE
         IC    R15,TAPEGLOB                 GET INDEX INTO GLOBALS
         MGHI  R15,L'@TD2FLD                MULTIPLY TO GET TRUE INDEX  64MODE
         LA    R3,0(R15,R3)                 CALCULATE ENTRY ADDRESS
         SPACE
         CLC   @TD2FTCC,@TD2FTCS            COMMAND COMPLETED?
         JE    QXTHERR6                     YES, INDICATE ERROR         64MODE
         SPACE
         OI    @TD2STAT,@TD2UABT            MARK ENTRY ACCORDINGLY
         POP   USING
         SPACE 2
         J     QXTHEXIT                     AND EXIT IMMEDIATELY        64MODE
         EJECT
***********************************************************************
* SUBROUTINE: QXTHOPN
*
*    OPEN OR ASSIGN GENERAL TAPE WITH THE REQUESTED PARAMETERS
***********************************************************************
         SPACE
QXTHOPN  DS    0H
         MVI   TAPEFUNC,FUNCOPEN            OPEN THE TAPE
         ENTRC QXTK
         BR    R7                           RETURN
         EJECT
***********************************************************************
* SUBROUTINE: QXTHFIL
*
*          FILL STORAGE BLOCK WITH FILL CHARACTER
*
* INPUT: R5  - LENGTH OF BLOCK TO FILL
*        R14 - ADDRESS OF BLOCK TO FILL
*        R15 - FILL CHARACTER TO USE
*
* OUTPUT:BLOCK WILL BE FILLED BY FILL CHARACTER
*        R5  - UNKNOWN
*        R14 - UNKNOWN
*        R15 - UNKNOWN
*
*        ALL OTHER REGISTERS UNCHANGED
***********************************************************************
         SPACE
QXTHFIL  DS    0H
         CGF   R5,OVERHEAD                  IS RECORD OVERHEAD > COUNT? 64MODE
         BNHR  R7                           NO, JUST RETURN
QXTHFIL1 DS    0H
         STC   R15,0(R14)                   STORE FILL CHARACTER
         CLI   CYCLEDEF,CYCLEYES            CYCLE THE FILL CHARACTER?
         JNE   QXTHFIL2                     NO, SKIP INCREMENT          64MODE
         AGHI  R15,1                        INCREMENT THE FILL BYTE     64MODE
QXTHFIL2 DS    0H
         AGHI  R14,1                        BUMP ADDRESS BY 1           64MODE
         BRCTG R5,QXTHFIL1                  LOOP UNTIL DONE             64MODE
         BR    R7                           RETURN
         EJECT
**********************************************************************
* ROUTINE: QXTHERR
*
* ERROR PROCESSING
**********************************************************************
         SPACE
**********************************************************************
* WAITC ERROR OCCURED
**********************************************************************
QXTHERR1 DS    0H
         CLC   TAPEGLOB,ZERO                GLOBAL INDEX VALID?
         JE    ERR1$100                     NO, SKIP AROUND GLOBALS     64MODE
         SPACE 2
         PUSH  USING
         GLOBZ REGR=R3
         LLGF  R3,@ISSDRV                                               64MODE
         SD0RV REG=R3
         SPACE 2
         SGR   R15,R15                                                  64MODE
         IC    R15,TAPEGLOB                 GET INDEX INTO GLOBALS
         MGHI  R15,L'@TD2FLD                MULTIPLY TO GET TRUE INDEX  64MODE
         LA    R3,0(R15,R3)                 CALCULATE ENTRY ADDRESS
         SPACE 1
         LGH   R15,@TD2ECNT                 GET OLD COMMAND COUNT       64MODE
         AGHI  R15,1                        INCREMENT COUNT             64MODE
         STH   R15,@TD2ECNT                 STORE NEW COUNT
         POP   USING
         SPACE 2
*
ERR1$100 DS    0H
         LA    R2,QXTHM03
         BRAS  R7,QXTHWTP2                  WTOPC                       64MODE
         J     QXTHCLS                      CLOSE TAPE                  64MODE
         SPACE
**********************************************************************
* COUNT ERROR OCCURED
**********************************************************************
**********************************************************************
*  OPCODE VALUE too big
**********************************************************************
QXTHERR7 DS    0H
         CLC   TAPEGLOB,ZERO                GLOBAL INDEX VALID?
         JE    ERR7$100                     NO, SKIP AROUND GLOBALS     64MODE
         SPACE 2
         PUSH  USING
         GLOBZ REGR=R3
         LLGF  R3,@ISSDRV                                               64MODE
         SD0RV REG=R3
         SPACE 2
         SGR   R15,R15                                                  64MODE
         IC    R15,TAPEGLOB                 GET INDEX INTO GLOBALS
         MGHI  R15,L'@TD2FLD                MULTIPLY TO GET TRUE INDEX  64MODE
         LA    R3,0(R15,R3)                 CALCULATE ENTRY ADDRESS
         SPACE 1
         LGH   R15,@TD2ECNT                 GET OLD COMMAND COUNT       64MODE
         AGHI  R15,1                        INCREMENT COUNT             64MODE
         STH   R15,@TD2ECNT                 STORE NEW COUNT
         POP   USING
         SPACE 2

ERR7$100 DS    0H
         LA    R2,QXTHM13
         BRAS  R7,QXTHWTP2                  WTOPC                       64MODE
         J     QXTHCLS                      CLOSE TAPE                  64MODE
         SPACE
**********************************************************************
* INVALID ACTION COMMAND CODE
**********************************************************************
QXTHERR2 DS    0H
         SGR   R3,R3                                                    64MODE
         IC    R3,ACTCMD                    INCLUDE VALUE IN MSG
         LA    R2,QXTHM05A
         BRAS  R7,QXTHWTP3                  SEND OUT MESSAGE            64MODE
         J     QXTHEXIT                     AND EXIT                    64MODE
         SPACE
**********************************************************************
* INAPPROPRIATE TAPE FOR INDICATED COMMAND
**********************************************************************
QXTHERR3 DS    0H
         LA    R2,QXTHM19
         BRAS  R7,QXTHWTP2                  SEND OUT MSG                64MODE
         J     QXTHEXIT                     AND EXIT                    64MODE
         SPACE
**********************************************************************
* INVALID OPCODE VALUE FOUND
**********************************************************************
QXTHERR4 DS    0H
         SGR   R3,R3                                                    64MODE
         IC    R3,OPCDEF                    INCLUDE VALUE IN MSG
         LA    R2,QXTHM05B
         BRAS  R7,QXTHWTP3                  SEND OUT MESSAGE            64MODE
         J     QXTHEXIT                     AND EXIT                    64MODE
         SPACE
**********************************************************************
* TAPE NOT FOUND IN GLOBAL FOR 'ABORT' PROCESSING
**********************************************************************
QXTHERR5 DS    0H
         LA    R2,QXTHM33A
         BRAS  R7,QXTHWTP2                  SEND OUT MSG                64MODE
         J     QXTHEXIT                     AND EXIT                    64MODE
         SPACE
**********************************************************************
* SPECIFIED TAPE FOR 'ABORT' COMMAND SHOWS NO ACTIVE COMMAND
**********************************************************************
QXTHERR6 DS    0H
         LA    R2,QXTHM34
         BRAS  R7,QXTHWTP2                  SEND OUT MSG                64MODE
         J     QXTHEXIT                     AND EXIT                    64MODE
         EJECT 1
**********************************************************************
* QXTK RETURN CODES
**********************************************************************
QXTHX000 DS    0H                           ANALYZE QXTK RETURN CODE
         CGHI  R5,GLOBFULL                  WAS THE GLOBAL AREA FULL?   64MODE
         JE    QXTHX010                     YES, PROCESS ERROR          64MODE
         CGHI  R5,GLOBNFND                  TAPE ENTRY NOT FOUND?       64MODE
         JE    QXTHX020                     YES, PROCESS ERROR          64MODE
         CGHI  R5,GLOBDUPL                  DUPLICATE TAPE NAME?        64MODE
         JE    QXTHX030                     YES, PROCESS ERROR          64MODE
         CGHI  R5,GLOBNULL                  GLOBALS NOT AVAILABLE?      64MODE
         JE    QXTHX040                     YES, PROCESS ERROR          64MODE
         J     QXTHX999                     OTHERWISE, UNKNOWN ERROR    64MODE
         SPACE 2
QXTHX010 DS    0H                           GLOBAL TABLE FULL
         LA    R2,QXTHM33B
         BRAS  R7,QXTHWTP2                  SEND OUT MSG                64MODE
         J     QXTHEXIT                     AND EXIT                    64MODE
         SPACE 2
QXTHX020 DS    0H                           GLOBAL TABLE FULL
         LA    R2,QXTHM33B
         BRAS  R7,QXTHWTP2                  SEND OUT MSG                64MODE
         J     QXTHEXIT                     AND EXIT                    64MODE
         SPACE 2
QXTHX030 DS    0H                           ATTEMPT TO ADD DUPL TAPE
         LA    R2,QXTHM33C
         BRAS  R7,QXTHWTP2                  SEND OUT MSG                64MODE
         J     QXTHEXIT                     AND EXIT                    64MODE
         SPACE 2
QXTHX040 DS    0H                           INVALID GLOBAL ADDRESS
         LA    R2,QXTHM33D
         BRAS  R7,QXTHWTP2                  SEND OUT MSG                64MODE
         J     QXTHEXIT                     AND EXIT                    64MODE
         SPACE 5
QXTHX999 DS    0H                           UNKNOWN RETURN CODE
         LGR   R3,R5                        INCLUDE BAD RETURN CODE     64MODE
         LA    R2,QXTHM33Z                  ... IN THE ERROR MSG
         BRAS  R7,QXTHWTP3                  SEND OUT MSG                64MODE
         J     QXTHEXIT                     AND EXIT                    64MODE
         EJECT 1
**********************************************************************
* ROUTINE: QXTHCLS
*
* CALL QXTK TO CLOSE THE TAPE AND MARK COMMAND COMPETION
**********************************************************************
         SPACE
QXTHCLS  DS    0H
         MVI   TAPEFUNC,FUNCCLOS            CLOSE THE TAPE
         ENTRC QXTK
         SPACE
QXTHFINI DS    0H                           START OF EXIT PROCESSING
**********************************************************************
* SEND USER A MESSAGE TELLING THEM COMMAND FINISHED
**********************************************************************
         LA    R2,QXTHM67
         BRAS  R7,QXTHWTP2                  SEND COMPLETION MSG         64MODE
**********************************************************************
* IF APPROPRIATE, MARK GLOBAL ENTRY FOR THIS TAPE AS 'CMD COMPLETE'
**********************************************************************
         SPACE 2
         CLC   TAPEGLOB,ZERO                GLOBALS VALID?
         JE    QXTHEXIT                     NO, CONTINUE ON             64MODE
         SPACE 2
         MVI   TAPEFUNC,FUNC@END            MARK COMMAND FINISH
         ENTRC QXTK                         CALL SEGMENT TO DO IT
         LTGR  R5,R5                        WAS THE RETURN CODE NON-0?  64MODE
         JNZ   QXTHX000                     YES, ANALYZE RETURN CODE    64MODE
         SPACE 1
**********************************************************************
* EXIT THE SEGMENT
**********************************************************************
QXTHEXIT DS    0H
         EXITC
         EJECT
**********************************************************************
* SUBROUTINE: QXTHWTP
*
*           SKELETON WTOPC MACROS
**********************************************************************
         SPACE
         DCTMSG
         USING DCTMSG,R2
QXTHWTP1 DS    0H
         CLI   MSGLET,C'E'        E TYPE MESSAGE?
         JNE   WTP1$010           NO, CHECK FOR ANOTHER                 64MODE
         TM    MSGDEF,MSGEMSG     PROCESS THESE MESSAGES?
         BNOR  R7                 NO, JUST RETURN
         J     WTP1$100           PRINT MESSAGE                         64MODE
WTP1$010 DS    0H
         CLI   MSGLET,C'I'        'I' TYPE MESSAGE?
         BNER  R7                 NO, I DON'T KNOW WHAT IT IS - RETURN
         TM    MSGDEF,MSGIMSG     PROCESS THESE MESSAGES?
         BNOR  R7                 NO, JUST RETURN
         SPACE
WTP1$100 DS    0H
         WTOPC PREFIX=TA64,HEADER=YES,TIME=NO,CHAIN=YES,ENDOFM=NO,     X
               DOT=NO,                                                 X
               NUMA=MSGNUM,                                            X
               SUB=(CHARA,QXT3MACN,CHARA,CHARTAPE,HEX4A,(R4)),         X
               LETA=MSGLET,                                            X
               TEXTA=MSGLEN,                                           X
               COMP=YES,                                               X
               EBROUT=YES,                                             X
               ROUT=(TAPE)
         BR    R7                 RETURN
         SPACE
QXTHWTP2 DS    0H
         CLI   MSGLET,C'E'        E TYPE MESSAGE?
         JNE   WTP2$010           NO, CHECK FOR ANOTHER                 64MODE
         TM    MSGDEF,MSGEMSG     PROCESS THESE MESSAGES?
         BNOR  R7                 NO, JUST RETURN
         J     WTP2$100           PRINT MESSAGE                         64MODE
WTP2$010 DS    0H
         CLI   MSGLET,C'I'        I TYPE MESSAGE?
         BNER  R7                 NO, I DON'T KNOW WHAT IT IS - RETURN
         TM    MSGDEF,MSGIMSG     PROCESS THESE MESSAGES?
         BNOR  R7                 NO, JUST RETURN
         SPACE
WTP2$100 DS    0H
         WTOPC PREFIX=TA64,HEADER=YES,TIME=NO,CHAIN=YES,ENDOFM=YES,    X
               DOT=NO,                                                 X
               NUMA=MSGNUM,                                            X
               SUB=(CHARA,CHARTAPE,CHARA,(R4)),                        X
               LETA=MSGLET,                                            X
               TEXTA=MSGLEN,                                           X
               COMP=YES,                                               X
               EBROUT=YES,                                             X
               ROUT=(TAPE)
         BR    R7                 RETURN
         SPACE
QXTHWTP3 DS    0H
         CLI   MSGLET,C'E'        E TYPE MESSAGE?
         JNE   WTP3$010           NO, CHECK FOR ANOTHER                 64MODE
         TM    MSGDEF,MSGEMSG     PROCESS THESE MESSAGES?
         BNOR  R7                 NO, JUST RETURN
         J     WTP3$100           PRINT MESSAGE                         64MODE
WTP3$010 DS    0H
         CLI   MSGLET,C'I'        I TYPE MESSAGE?
         BNER  R7                 NO, I DON'T KNOW WHAT IT IS - RETURN
         TM    MSGDEF,MSGIMSG     PROCESS THESE MESSAGES?
         BNOR  R7                 NO, JUST RETURN
         SPACE
WTP3$100 DS    0H
         WTOPC PREFIX=TA64,HEADER=YES,TIME=NO,CHAIN=YES,ENDOFM=YES,    X
               DOT=NO,                                                 X
               NUMA=MSGNUM,                                            X
               LETA=MSGLET,                                            X
               TEXTA=MSGLEN,                                           X
               COMP=YES,                                               X
               EBROUT=YES,                                             X
               ROUT=(TAPE),                                            X
               SUB=(HEX,(R3))
         BR    R7
         DROP  R2
         EJECT
**********************************************************************
* MESSAGE TEXT
**********************************************************************
         SPACE
QXTHM03  GENMSG LET=E,NUM=03,                                          X
               TEXT='WAITC Error for ... tape - check BLOCK and COUNT'
QXTHM13  GENMSG LET=E,NUM=03,                                          X
               TEXT='Internal error - COUNT should less than 4097'
QXTHM05A GENMSG NUM=05,LET=E,                                          X
               TEXT='Internal error - invalid command indicator ..'
QXTHM05B GENMSG NUM=05,LET=E,                                          X
               TEXT='Internal error - invalid OPCODE indicator ..'
QXTHM19  GENMSG NUM=19,LET=E,                                          X
               TEXT='Tape specified is not valid for this command'
QXTHM33A GENMSG NUM=33,LET=E,                                          X
               TEXT='Tape ... not found - tape is not active'
QXTHM33B GENMSG NUM=33,LET=E,                                          X
               TEXT='No tape entries available - too many active tapes'
QXTHM33C GENMSG NUM=33,LET=E,                                          X
               TEXT='Tape ... is already in the table'
QXTHM33D GENMSG NUM=33,LET=E,                                          X
               TEXT='GLOBALS NOT AVAILABLE'
QXTHM33Z GENMSG NUM=33,LET=E,                                          X
               TEXT='Invalid return code .... from QXTK'
QXTHM34  GENMSG NUM=34,LET=E,                                          X
               TEXT='No command to ABORT for ... tape'
QXTHM64A GENMSG LET=I,NUM=64,                                          X
               TEXT='Macro:..... Tape:... HEX Data (00-07): ...........X
               ......'
QXTHM64B GENMSG LET=I,NUM=64,                                          X
               TEXT='Macro:..... Tape:... HEX Data (08-15): ...........X
               ......'
QXTHM64C GENMSG LET=I,NUM=64,                                          X
               TEXT='Macro:..... Tape:... HEX Data (16-23): ...........X
               ......'
QXTHM64D GENMSG LET=I,NUM=64,                                          X
               TEXT='Macro:..... Tape:... HEX Data (24-31): ...........X
               ......'
QXTHM64E GENMSG LET=I,NUM=64,                                          X
               TEXT='Macro:..... Tape:... HEX Data (32-39): ...........X
               ......'
QXTHM64F GENMSG LET=I,NUM=64,                                          X
               TEXT='Macro:..... Tape:... HEX Data (40-47): ...........X
               ......'
QXTHM64G GENMSG LET=I,NUM=64,                                          X
               TEXT='Macro:..... Tape:... HEX Data (48-55): ...........X
               ......'
QXTHM64H GENMSG LET=I,NUM=64,                                          X
               TEXT='Macro:..... Tape:... HEX Data (56-63): ...........X
               ......'
QXTHM67  GENMSG LET=I,NUM=67,                                          X
               TEXT='... tape operation complete'
         EJECT 1
**********************************************************************
* CONSTANTS
**********************************************************************
TAPE#RTA DC    C'RT'                   REALTIME TAPE PREFIX
TEST5    DC    CL(L'TAPHDR)'VAR  LEN'
TRAILER  DC    C'*END OF RECORD*'      RECORD TRAILER TAG
OVERHEAD DC    A(TAPOVRHD+L'TRAILER)   TOTAL RECORD OVERHEAD LENGTH
         SPACE 2
CCW_SKEL CCW1  CTONOP,0,CTFSLI,0       CCW SKELETON
         SPACE 2
ZERO     DC    F'0'                    ZERO FOR COMPARISONS
*
         LTORG
         PRINT GEN
         TA0EQ GLOB=YES
         PRINT NOGEN
         FINIS QXTH
         END
