         PRINT NOGEN
         BEGIN NAME=QXTK,AMODE=64
***********************************************************************
*       Licensed Materials - Property of IBM                          *
*       "Restricted Materials of IBM"                                 *
*       5748-T15                                                      * 
*       Copyright IBM Corp. 2003, 2008                                *
*       All Rights Reserved                                           *
*       US Government Users Restricted Rights - Use, duplication      *
*       or dislosure restricted by GSA ADP Schedule Contract          *
*       with IBM Corp.                                                *
***********************************************************************
*  This file is provided on an "as is" basis.  There are no           *
*  warranties, express or implied, including the implied warranties   *
*  of merchantability and fitness for a particular purpose.           *
*  IBM has no obligation to provide service, defect correction, or    *
*  any maintenance for the file.  IBM has no obligation to supply     *
*  any updates or enhancements for the file to you even if such are   *
*  or later become available.                                         *
***********************************************************************
***********************************************************************
*
* MODULE NAME..... QXTK
* DESCRIPTION..... GENERAL TAPE DRIVER
* RELATED MODULE.. QXTG
*                  QTP2
*                  QXTH
*                  QXTJ
*                  QTP5
*                  QTP6
*                  QTP7
*
* DOCUMENT NAME... GENERAL TAPE DRIVER USERS GUIDE
* LEVEL........... N/A
*
* FUNCTION...      PERFORM COMMON FUNCTIONS TO ALL TAPE SEGMENTS
*
*                  INCLUDED ARE THE FOLLOWING FUNCTIONS:
*                   -- CHECK_CORE_LEVELS
*                   -- OPEN/CLOSE TAPE
*                   -- DISPLAY HELP MESSAGES
*                   GLOBAL AREA MAINTENANCE FUNCTIONS
*                   -- LOCATE TAPE NAME
*                   -- INDICATE COMMAND START
*                   -- INDICATE COMMAND COMPLETION
*                   -- ADD TAPE NAME TO TABLE
*
* MODULE ATTRIBUTES..
*   TYPE.......... 'E' (ECB CONTROLLED)
*   DEPENDENCY.... NONE (OBJECT SHIPPABLE)
*   ALLOCATION.... 4K FILE RESIDENT
*   ENVIRONMENT... BASE, BSS
*   ENTRY POINT... QXTK
*
* ECB-CONVENTIONS.......
*
*   WORK AREA
*    EBX000-103      SEE EQUATE MACRO TA0EQ
*    EBW000-103      SEE EQUATE MACRO TA0EQ
*
* DATA LEVELS
*         NONE USED
*
* REGISTER-CONVENTIONS..
*
*         R0 -  USED TO HOLD RESULT OF QXTKRDCL ROUTINE
*         R1 -  END-OF-MESSAGE INDICATOR FOR HELP MESSAGES
*         R2 -  SCRATCHED BY CORE LEVEL TEST ROUTINES
*         R3 -  SCRATCHED BY CORE LEVEL TEST ROUTINES
*         R4 -
*         R5 -  RETURN CODE VALUE FROM ROUTINES
*         R6 -  ADDRESS OF MSG TEXT -- HELP MESSAGES
*         R7 -  BAS REGISTER
*         R14 - SCRATCH
*         R15 - SCRATCH
*
***********************************************************************
** CHANGE ACTIVITY...
**
**        UPDATE BY          PLOG #   DATE   DESCRIPTION
**        ---------          -------  ------ ------------
**        KENT               TD00301  071390 MSG SETTING ENHANCEMENTS
**        KENT               TD00346  022891 TEP PROJECT IMPROVEMENTS
**        KENT               TD00368  070791 FIX RBL OPCODE
**        MKENT              TD00508  021494 CHECK SWBS FOR SHUTDOWN
**        YMLI               F14612   081506 The output routing
**        YMLI               F14943   020807 ZTEST TA64 START
**                                           tape_label should exercise
**                                           TA64 test cases
***********************************************************************
*
         EJECT ,                                              @TD00508
         MACRO                                                @TD00508
&LABEL   READSLVL &BLKEQU,&BLKTAG                             @TD00508
.*    EXIT CONDITIONS:  R0 HOLDS SHUTDOWN LVL FOR IND BLOCK   @TD00508
.*                      R1 HOLDS CURRENT BLOCK AVAIL PCTAGE   @TD00508
.*                      R2 HOLDS ALLOCATION OF IND BLOCK      @TD00508
.*                      R3,R15 SCRATCHED                      @TD00508
         SPACE ,                                              @TD00508
         LA    R1,&BLKEQU              COPY BLK TYPE INTO R1  @TD00508
         LGR   R2,R1                   COPY REG CONTENTS      @TD00508  64MODE
         NUMBC TYPE=R1                 GET AVAILABILITY COUNT @TD00508
         MAXBC TYPE=R2                 GET ALLOCATION COUNT   @TD00508
         SPACE ,                                              @TD00508
         LGH   R0,&BLKTAG              ACTUAL SHUTDOWN LEVEL  @TD00508  64MODE
         SPACE ,                                              @TD00508
         SGR   R1,R0                   ADJ FOR SHUTDOWN LVL   @TD00508  64MODE
         SGR   R2,R0                   ADJ FOR SHUTDOWN LVL   @TD00508  64MODE
         XGR   R0,R0                   CLEAR FOR CALCULATION  @TD00508  64MODE
         MGHI  R1,100                  ADJ TO GET PERCENTAGE  @TD00508  64MODE
         DR    R0,R2                   DIVIDE TO GET RATIO    @TD00508
.*                                                            @TD00508
.*   R1 = Percentage of blocks available, exclusive of the    @TD00508
.*        shutdown level.                                     @TD00508
         MEND                                                 @TD00508
         EJECT ,                                              @TD00508
QXTK     DS    0H
         J     QXTK0000                BRANCH AROUND EYE-CATCHER        64MODE
         DC    CL8'&SYSDATE',CL1' ',CL8'&SYSTIME'
         SPACE 2
QXTK0000 DS    0H
         STM   R0,R7,CE1UR0            SAVE REGS BEFORE USE
         XC    CE1UR5,CE1UR5           SET RETURN CODE = 0 (R5)
         SPACE 1
         SGR   R5,R5                                                    64MODE
         IC    R5,TAPEFUNC             GET FUNCTION INDICATOR
         MGHI  R5,L'QXTK0100           CALCULATE TABLE OFFSET           64MODE
         B     QXTK0100(R5)            BRANCH TO CORRECT ROUTINE
         SPACE 5
QXTK0100 J     QXTKERR1                UNKNOWN COMMAND INDICATOR        64MODE
         J     QXTKOPN                 OPEN A TAPE                      64MODE
         J     QXTKCLS                 CLOSE A TAPE                     64MODE
         SPACE 1
         J     QXTKCCL                 CHECK CORE LEVELS                64MODE
         SPACE 1
         J     QXTKHLPF                DISPLAY FUNCTION TEST HELP       64MODE
         J     QXTKHLPS                DISPLAY SYSTEM TEST HELP         64MODE
         SPACE 1
         J     QXTK$LOC                LOCATE TAPE NAME IN GLOBAL       64MODE
         J     QXTK$SET                SET COMMAND START DATA           64MODE
         J     QXTK$END                SET COMMAND COMPLETION DATA      64MODE
         J     QXTK$EGE                ESTABLISH GLOBAL ENTRY           64MODE
         SPACE 1                                              @TEDMLIN
         J     QXTKHLPO                DISPLAY OTHER HELP     @TEDMLIN  64MODE
         EJECT 1
*----------------------------------------------------------------------
*
* SUBROUTINE: QXTKOPN
*
*  ISSUE THE APPROPRIATE MACRO TO GAIN USE OF THE INDICATED TAPE
*----------------------------------------------------------------------
QXTKOPN  DS    0H
         CLC   REALPRFX,CHARTAPE       USING REALTIME TAPE?
         JE    QXTKBACK                YES, DON'T BOTHER IT             64MODE
         SPACE 2
         LA    R5,CHARTAPE             POINT TO TAPE NAME
         CLI   OPENDEF,OPENA           OPEN=ASSIGN?
         JE    QXTKOPN1                YES, GO DO IT                    64MODE
         SPACE
         CLI   OPENDEF,OPENI           OPEN=INPUT?
         JE    QXTKOPN2                YES, GO DO IT                    64MODE
         SPACE
         CLI   OPENDEF,OPENO           OPEN=OUTPUT?
         JE    QXTKOPN3                YES, GO DO IT                    64MODE
         SPACE
         CLI   OPENDEF,OPENN           OPEN=NO?
         JE    QXTKBACK                YES, JUST RETURN                 64MODE
         SPACE
         J     QXTKERR2                ERROR CONDITION                  64MODE
         SPACE 5
QXTKOPN1 DS    0H
         TASNC NAME=(R5)               ASSIGN THE TAPE TO US
         J     QXTKBACK                RETURN TO CALLER                 64MODE
         SPACE 5
QXTKOPN2 DS    0H
         TOPNC NAME=(R5),STATUS=I      OPEN THE TAPE FOR INPUT
         J     QXTKBACK                RETURN TO CALLER                 64MODE
         SPACE 5
QXTKOPN3 DS    0H
         CLI   BUFFDEF,BUFFNO          NONBUFFERED TAPE?
         JE    QXTKOPN4                YES, ISSUE CORRECT MACRO         64MODE
         TOPNC NAME=(R5),STATUS=O,BUF=YES                              X
                                       OPEN FOR OUTPUT, BUFFERED
         J     QXTKBACK                RETURN TO CALLER                 64MODE
         SPACE 5
QXTKOPN4 DS    0H
         TOPNC NAME=(R5),STATUS=O,BUF=NO                               X
                                       OPEN FOR OUTPUT, NONBUFFERED
         J     QXTKBACK                RETURN TO CALLER                 64MODE
         EJECT 1
*----------------------------------------------------------------------
*
* SUBROUTINE: QXTKCLS
*
*  ISSUE THE APPROPRIATE MACRO TO DROP USE OF THE INDICATED TAPE
*----------------------------------------------------------------------
QXTKCLS  DS    0H
         CLC   REALPRFX,CHARTAPE       USING REALTIME TAPE?
         JE    QXTKBACK                YES, DON'T BOTHER IT             64MODE
         SPACE 2
         LA    R5,CHARTAPE             POINT TO TAPE NAME
         CLI   CLOSEDEF,CLOSEY         CLOSE=YES?
         JE    QXTKCLS1                YES, GO DO IT                    64MODE
         SPACE 2
         CLI   CLOSEDEF,CLOSER         CLOSE=RESERVE?
         JE    QXTKCLS2                YES, GO DO IT                    64MODE
         SPACE 2
         CLI   CLOSEDEF,CLOSEE         CLOSE=EXIT?
         JE    QXTKBACK                YES, JUST RETURN                 64MODE
         SPACE 2
         J     QXTKERR2                ERROR DURING PROCESSING          64MODE
         SPACE 5
QXTKCLS1 DS    0H
         TCLSC NAME=(R5),D=NO,R=YES,U=NO    CLOSE THE TAPE
         J     QXTKBACK                RETURN TO CALLER                 64MODE
         SPACE 5
QXTKCLS2 DS    0H
         TRSVC NAME=(R5)               LEAVE TAPE AS RESERVED
         J     QXTKBACK                RETURN TO CALLER                 64MODE
         EJECT 1
*----------------------------------------------------------------------
*
* SUBROUTINE: QXTKCCL
*
*  ENSURE ENOUGH BLOCKS OF THE REQUESTED TYPE ARE AVAILABLE
*----------------------------------------------------------------------
QXTKCCL  DS    0H                      CHECK CORE LEVELS
         XC    TAPEFUNC,TAPEFUNC       CLEAR TO AVOID CONFUSION
         DLAYC                         ALLOW BLOCKS TO BE RETURNED
         CINFC R,CMMLVL,F,REG=R3       BASE OF SHUTDOWN TABLE @TD00508
         PUSH  USING                                          @TD00508
         IDSLVL REG=R3                 BLOCK SHUTDOWN INFO    @TD00508
QXTKCCL1 DS    0H                      CHECK BLOCK LEVELS
         SPACE 5                                              @TD00508
         READSLVL LFRAME,LVLMFRI                              @TD00508
         CGF   R1,MINAVAIL             ENOUGH AVAILABLE BLOCKS?         64MODE
         JNH   QXTKCCL2                NO, LOOP UNTIL AVAIL   @TD00508  64MODE
         READSLVL LSWB,LVLMSWI                                @TD00508
         CGF   R1,MINAVAIL             ENOUGH AVAIL BLOCKS?   @TD00508  64MODE
         JNH   QXTKCCL2                NO, LOOP UNTIL AVAIL   @TD00508  64MODE
         J     QXTKBACK                YES, RETURN TO CALLER  @TD00508  64MODE
         SPACE 2                                              @TD00508
QXTKCCL2 DS    0H                      CHECK BLOCK LEVELS     @TD00508
         POP   USING                                          @TD00508
         DLAYC                         ALLOW BLOCKS TO BE RETURNED
         J     QXTKCCL1                WAIT FOR BLOCK COUNT TO RISE     64MODE
         EJECT 1
*----------------------------------------------------------------------
*
* SUBROUTINE: QXTKGLBL
*
*  ESTABLISH ADDRESSABILITY TO GLOBAL AREA
*----------------------------------------------------------------------
QXTKGLBL DS    0H
         GLOBZ REGR=R1
         LLGF  R1,@ISSDRV              BASE OF DRIVER GLOBALS           64MODE
         SD0RV REG=R1                  DSECT DRIVER GLOBALS
         SGR   R5,R5                   CLEAR R5                         64MODE
         LTGR  R1,R1                   GLOBAL ADDRESS = 0?              64MODE
         BNZR  R7                      NO, OK TO RETURN TO CALLER
         SPACE 2
         LA    R5,GLOBNULL             INDICATE BAD RETURN CODE
         ST    R5,CE1UR5               PLACE IN REG SAVE AREA
         J     QXTKBACK                RETURN TO CALLER WITH RC         64MODE
         EJECT 1
*----------------------------------------------------------------------
*
* SUBROUTINE: QXTK$LOC
*
*  LOCATE TAPE NAME IN GLOBAL AREA 2
*
*----------------------------------------------------------------------
**  OBJECTIVE:  LOCATE THE TAPE NAME IN THE GLOBAL AREA
**  INPUT:      TAPE NAME (CHARTAPE)
**  RETURN:     INDEX OF THAT TAPE NAME (IN ECB AT TAPEGLOB)
**
*----------------------------------------------------------------------
QXTK$LOC DS    0H
         BRAS  R6,QXTK#LOC             CALL ROUTINE TO LOCATE ENTRY     64MODE
         LTGR  R5,R5                   SUCCESSFUL SEARCH?               64MODE
         JNZ   $LOC100                 NO, SET RETURN CODE              64MODE
         STC   R3,TAPEGLOB             STORE INDEX VALUE...
         J     QXTKBACK                AND RETURN TO CALLER             64MODE
         SPACE 2
$LOC100  DS    0H
         ST    R5,CE1UR5               STORE RETURN CODE...
         J     QXTKBACK                AND RETURN TO CALLER             64MODE
         SPACE 5
*----------------------------------------------------------------------
**  INTERNAL SUBROUTINE: QXTK#LOC
**
**  CALLED BY:  QXTK$LOC, QXTK$EGE, QXTK$SET, QXTK$END
**  RETURN TO:  ADDRESS IN R6
**  DESTROYS:   R1, R2, R3, R7
**  RETURN CODE = R5
**  RETURN VALUE = R3
**
*----------------------------------------------------------------------
         SPACE 2
QXTK#LOC DS    0H
         BRAS  R7,QXTKGLBL             ESTABLISH ADDRESS OF GLOBAL AREA 64MODE
         LA    R2,SD0MAXTAPE-1         MAXIMUM ENTRIES TO SEARCH
         SGR   R3,R3                   INDEX VALUE TO RETURN            64MODE
         SPACE 2
LOC#LOOP DS    0H
         LA    R1,L'@TD2FLD(R1)        INCREMENT TO NEXT GLOBAL ENTRY
         AGHI  R3,1                    INCREMENT INDEX BY 1             64MODE
         CLC   CHARTAPE,@TD2TAPE       ENTRIES MATCH?
         JE    LOC#FND                 YES, RETURN MATCH                64MODE
         BRCTG R2,LOC#LOOP             ITERATE AGAIN                    64MODE
         J     LOC#NFND                TAPE NAME NOT FOUND              64MODE
         SPACE 2
***
* ENTRY NOT FOUND
***
LOC#NFND DS    0H                      RETURN WITH INDEX VALUE IN R3
         LA    R5,GLOBNFND             INDICATE NOT FOUND
         BR    R6                      AND RETURN TO CALLER
         SPACE 2
***
* ENTRY FOUND
***
LOC#FND  DS    0H                      RETURN WITH INDEX VALUE IN R3
         SGR   R5,R5                   SET RC = 0                       64MODE
         BR    R6                      RETURN TO CALLER
         EJECT 1
*----------------------------------------------------------------------
*
* SUBROUTINE: QXTK$EGE
*
*  ESTABLISH GLOBAL ENTRY FOR TAPE NAME
*
*----------------------------------------------------------------------
**
**  OBJECTIVE:  ADD THE TAPE NAME TO THE GLOBAL AREA
**  INPUT:      TAPE NAME (CHARTAPE)
**  RETURN:     INDEX OF THAT TAPE NAME (IN ECB AT TAPEGLOB)
**
*----------------------------------------------------------------------
QXTK$EGE DS    0H
         BRAS  R6,QXTK#LOC             CALL ROUTINE TO LOCATE ENTRY     64MODE
         LTGR  R5,R5                   TAPE NAME FOUND?                 64MODE
         JNZ   EGE100                  NO, LOOK FOR NULL ENTRY          64MODE
         SPACE 2
*----------------------------------------------------------------------
**  TAPE NAME FOUND IN GLOBAL - CHECK FOR ATTEMPT TO ADD DUPLICATE
*----------------------------------------------------------------------
         STC   R3,TAPEGLOB             RETURN INDEX VALUE REGARDLESS
         LA    R5,GLOBDUPL             SET RC = DUPLICATE ENTRY
         TM    @TD2STAT,@TD2ACTV       IS ENTRY SYSTEM TEST ACTIVE?
         JO    EGE900                  YES, RETURN WITH NON-ZERO RC     64MODE
         J     EGE800                  ELSE, SET RC=0 / ENTRY REUSED    64MODE
         SPACE 5
*----------------------------------------------------------------------
**  NAME NOT FOUND - ATTEMPT TO LOCATE OPEN (NULL) ENTRY
*----------------------------------------------------------------------
EGE100   DS    0H
         LLGF  R0,CHARTAPE                                              64MODE
         LTGR   R0,R0                  SAVE TAPE NAME TEMPORARILY       64MODE
         XC    CHARTAPE,CHARTAPE       SET NULL TAPE NAME FOR SEARCH
         SPACE
         BRAS  R6,QXTK#LOC             CALL ROUTINE TO LOCATE ENTRY     64MODE
         STCM  R0,B'1111',CHARTAPE     RESTORE TAPE NAME VALUE
         LTGR  R5,R5                   NULL ENTRY FOUND?                64MODE
         JNZ   EGE200                  NO, LOOK FOR REUSABLE ENTRY      64MODE
         SPACE 2
*----------------------------------------------------------------------
**  NULL ENTRY FOUND IN GLOBAL - ESTABLISH OUR ENTRY THERE
*----------------------------------------------------------------------
         J     EGE800                  SET RC=0, ESTABLISH ENTRY        64MODE
         SPACE 5
*----------------------------------------------------------------------
**  ATTEMPT TO REUSE AN ENTRY
**
**  THE FOLLOWING CRITERION IS USED TO DETERMINE IF AN ENTRY IS
**  REUSABLE:  (ALL MUST HOLD TRUE)
**    - NOT ACTIVE FOR SYSTEM TEST ACTIVITY
**    - COMMAND COMPLETION FLAG NON-ZERO
**
*----------------------------------------------------------------------
EGE200   DS    0H
         BRAS  R7,QXTKGLBL             ESTABLISH GLOBAL AREA ADDRESS    64MODE
         LA    R2,SD0MAXTAPE-1         MAXIMUM ENTRIES TO SEARCH
         SGR   R3,R3                   INDEX VALUE TO RETURN            64MODE
         SPACE
EGE210   DS    0H
         LA    R1,L'@TD2FLD(R1)        INCREMENT TO NEXT GLOBAL ENTRY
         AGHI  R3,1                    INCREMENT INDEX BY 1             64MODE
         TM    @TD2STAT,@TD2ACTV       SYSTEM TEST ACTIVE?
         JO    EGE220                  YES, TRY AGAIN                   64MODE
         CLI   @TD2FTCC,0              FUNCTION TEST CMD COMPLETED?
         JE    EGE220                  NO, TRY AGAIN                    64MODE
         CLC   REALPRFX,@TD2TAPE       ENTRY FOR REALTIME TAPE?
         JE    EGE220                  YES, LEAVE IN TABLE - TRY AGAIN  64MODE
*----------------------------------------------------------------------
**  REUSABLE ENTRY FOUND IN GLOBAL - ESTABLISH OUR ENTRY THERE
*----------------------------------------------------------------------
         J     EGE800                  OTHERWISE FOUND REUSABLE ENTRY   64MODE
         SPACE
EGE220   DS    0H
         BRCTG R2,EGE210               ITERATE AGAIN                    64MODE
         SPACE 2
*----------------------------------------------------------------------
**  NO REUSABLE ENTRIES FOUND IN GLOBAL - GIVE UP AND RETURN TO CALLER
*----------------------------------------------------------------------
EGE250   DS    0H
         LA    R5,GLOBFULL             RC = GLOBAL AREA FULL
         J     EGE900                  GIVE UP AND RETURN TO CALLER     64MODE
         SPACE 5
*----------------------------------------------------------------------
**  INITIALIZE GLOBAL ENTRY AND RETURN TO USER
*----------------------------------------------------------------------
EGE800   DS    0H
         STC   R3,TAPEGLOB             RETURN INDEX VALUE
         MVC   @TD2TAPE,CHARTAPE       INDICATE TAPE NAME IN GLOBAL
         J     QXTKBACK                AND RETURN TO CALLER             64MODE
         SPACE 2
*----------------------------------------------------------------------
**  RETURN TO USER WITH RETURN CODE INDICATED - **ABNORMAL EXIT**
*----------------------------------------------------------------------
EGE900   DS    0H
         ST    R5,CE1UR5               STORE RETURN CODE
         J     QXTKBACK                AND RETURN TO CALLER             64MODE
         EJECT 1
*----------------------------------------------------------------------
*
* SUBROUTINE: QXTK$SET
*
*  SET COMMAND START VALUES IN GLOBAL AREA 2
*
**
**  INPUT:      TAPE INDEX (TAPEGLOB)
**  RETURN:     NOTHING
**
**  RETURN TO:  QXTKBACK
**  DESTROYS:   R1, R2, R7
**  RETURN CODE = NONE
**  RETURN VALUE = NONE
**
*----------------------------------------------------------------------
QXTK$SET DS    0H
         BRAS  R7,QXTKGLBL             ESTABLISH ADDRESS OF GLOBAL AREA 64MODE
         SGR   R2,R2                                                    64MODE
         IC    R2,TAPEGLOB             GET GLOBAL INDEX VALUE
         MGHI  R2,L'@TD2FLD            MULTIPLY TO GET TRUE INDEX       64MODE
         LA    R1,0(R2,R1)             CALCULATE GLOBAL ENTRY ADDRESS
         SPACE 2
         MVC   @TD2FTCS,ACTCMD         SET COMMAND STARTED
         XC    @TD2FTCC,@TD2FTCC       CLEAR COMMAND ENDED FIELD
         LLGF  R2,@TD2CMDS             GET CURRENT COMMAND COUNT        64MODE
         ALGF  R2,=F'1'                ADD 1                            64MODE
         ST    R2,@TD2CMDS             SAVE IN GLOBAL AREA
         SPACE 2
         J     QXTKBACK                RETURN TO CALLER                 64MODE
         EJECT 1
*----------------------------------------------------------------------
*
* SUBROUTINE: QXTK$END
*
*  SET COMMAND COMPLETION VALUES IN GLOBAL AREA 2
*
**
**  INPUT:      TAPE INDEX (TAPEGLOB)
**  RETURN:     NOTHING
**
**  RETURN TO:  QXTKBACK
**  DESTROYS:   R1, R2, R7
**  RETURN CODE = NONE
**  RETURN VALUE = NONE
**
*----------------------------------------------------------------------
QXTK$END DS    0H
         BRAS  R7,QXTKGLBL             ESTABLISH ADDRESS OF GLOBAL AREA 64MODE
         SGR   R2,R2                                                    64MODE
         IC    R2,TAPEGLOB             GET GLOBAL INDEX VALUE
         MGHI  R2,L'@TD2FLD            MULTIPLY TO GET TRUE INDEX       64MODE
         LA    R1,0(R2,R1)             CALCULATE GLOBAL ENTRY ADDRESS
         SPACE 2
         MVC   @TD2FTCC,ACTCMD         SET COMMAND STARTED
         SPACE 2
         J     QXTKBACK                RETURN TO CALLER                 64MODE
         EJECT 1
*----------------------------------------------------------------------
*
* SUBROUTINE: QXTKHLPF
*
*  DISPLAY THE FUNCTION TEST HELP DISPLAY
*----------------------------------------------------------------------
QXTKHLPF DS    0H
         SGR   R2,R2                   MESSAGE LIST TABLE INDEX         64MODE
QXTKHPF1 DS    0H
         LGF   R6,HLPFLIST(R2)         GET ADCON OF CURRENT MSG         64MODE
         LTGR  R6,R6                   IS IT 0?                         64MODE
         JE    QXTKHLPX                YES, SEND OUT COMPLETION MSG     64MODE
         LA    R1,PARTIAL              CONTINUE CHAINING THIS MSG
         LA    R6,QXTK(R6)             CALCULATE MSG ADDRESS
         BRAS  R7,QXTKWTP2             DISPLAY TEXT                     64MODE
         LA    R2,L'HLPFLIST(R2)       INCREMENT TO NEXT LIST ENTRY
         J     QXTKHPF1                REPEAT UNTIL FINISHED            64MODE
         SPACE 5
QXTKHLPX DS    0H
         LA    R1,FULLMSG              LAST MESSAGE IN CHAIN
         LA    R6,HLPX999              POINT TO COMPLETION MSG
         BRAS  R7,QXTKWTP2             DISPLAY TEXT                     64MODE
         SPACE
         J     QXTKBACK                RETURN TO CALLER                 64MODE
         EJECT 1
*----------------------------------------------------------------------
*
* SUBROUTINE: QXTKHLPS
*
*  DISPLAY THE SYSTEM TEST HELP DISPLAY
*----------------------------------------------------------------------
QXTKHLPS DS    0H
         SGR   R2,R2                   MESSAGE LIST TABLE INDEX         64MODE
QXTKHPS1 DS    0H
         LGF   R6,HLPSLIST(R2)         GET ADCON OF CURRENT MSG         64MODE
         LTGR  R6,R6                   IS IT 0?                         64MODE
         JE    QXTKHLPX                YES, SEND OUT COMPLETION MSG     64MODE
         LA    R1,PARTIAL              CONTINUE CHAINING THIS MSG
         LA    R6,QXTK(R6)             CALCULATE MSG ADDRESS
         BRAS  R7,QXTKWTP2             DISPLAY TEXT                     64MODE
         LA    R2,L'HLPSLIST(R2)       INCREMENT TO NEXT LIST ENTRY
         J     QXTKHPS1                REPEAT UNTIL FINISHED            64MODE
         EJECT 1
*----------------------------------------------------------------------
*
* SUBROUTINE: QXTKHLPO
*
*  DISPLAY THE OTHER TESTS HELP DISPLAY
*
*----------------------------------------------------------------------
QXTKHLPO DS    0H                                             @TEDMLIN
         SGR   R2,R2                   MESSAGE TABLE INDEX    @TEDMLIN  64MODE
QXTKHPO1 DS    0H                                             @TEDMLIN
         LGF   R6,HLPOLIST(R2)         GET ADCON OF CURR MSG  @TEDMLIN  64MODE
         LTGR  R6,R6                   IS IT 0?               @TEDMLIN  64MODE
         JE    QXTKHLPX                YES, SEND CMPLTION MSG @TEDMLIN  64MODE
         LA    R1,PARTIAL              CONTINUE CHAINING MSG  @TEDMLIN
         LA    R6,QXTK(R6)             CALCULATE MSG ADDRESS  @TEDMLIN
         BRAS  R7,QXTKWTP2             DISPLAY TEXT           @TEDMLIN  64MODE
         LA    R2,L'HLPOLIST(R2)       INCREMENT TO NEXT MSG  @TEDMLIN
         J     QXTKHPO1                REPEAT UNTIL FINISHED  @TEDMLIN  64MODE
         EJECT 1                                              @TEDMLIN
*----------------------------------------------------------------------
*
*  ERROR HANDLERS -- DISPLAY AN APPROPRIATE MESSAGE AND EXIT
*----------------------------------------------------------------------
QXTKERR1 DS    0H
         SGR   R3,R3                                                    64MODE
         IC    R3,ACTCMD               INCLUDE VALUE IN MESSAGE
         LA    R6,QXTKM05A
         BRAS  R7,QXTKWTP1             SEND OUT MESSAGE                 64MODE
         J     QXTKEXIT                                                 64MODE
         SPACE 2
QXTKERR2 DS    0H
         SGR   R3,R3                                                    64MODE
         IC    R3,TAPEFUNC             INCLUDE VALUE IN MESSAGE
         LA    R6,QXTKM05B
         BRAS  R7,QXTKWTP1             SEND OUT MESSAGE                 64MODE
         J     QXTKEXIT                                                 64MODE
         EJECT 1
QXTKBACK DS    0H
         LM    R0,R7,CE1UR0            RESTORE REGS BEFORE RETURN
         BACKC ,
         SPACE 5
QXTKEXIT DS    0H
         EXITC ,                       TAKE AN ERROR EXIT
         EJECT 1
*----------------------------------------------------------------------
*   WTOPC MESSAGES TO LET USER KNOW WHAT'S GOING ON
*----------------------------------------------------------------------
QXTKM05A GENMSG NUM=05,LET=E,                                          X
               TEXT='Internal error - invalid command indicator ..'
QXTKM05B GENMSG NUM=05,LET=E,                                          X
               TEXT='Internal error during processing - TAPEFUNC = ..'
         EJECT 1
*----------------------------------------------------------------------
*   TEXT/POINTER TABLE FOR HELP MESSAGES
*----------------------------------------------------------------------
HLPFLIST DC    A(HLPF001-QXTK)         ADDRESS OF FIRST BLOCK
         DC    A(HLPF002-QXTK)         ADDRESS OF NEXT BLOCK
         DC    A(HLPF003-QXTK)         ADDRESS OF NEXT BLOCK
         DC    A(HLPF004-QXTK)         ADDRESS OF NEXT BLOCK
         DC    A(HLPF005-QXTK)         ADDRESS OF NEXT BLOCK
         DC    A(HLPF006-QXTK)         ADDRESS OF NEXT BLOCK
         DC    A(0)                    NO MORE BLOCKS, FINISH OFF MSG
         SPACE 2
HLPF001  DC    AL1(HLPF001E-*-1)       CALCULATE LENGTH OF THIS BLOCK
         DC    C'TAPE99I Function Test Interface Syntax:',AL1(#CAR)
         DC    AL1(#CAR)
         DC    C' ZTEST TA64   HELPFUNC | HELPSYS | HELPOTH'  @TEDMLIN
HLPF001E EQU   *
HLPF002  DC    AL1(HLPF002E-*-1)
         DC    C'              ABORT | BACKSPACE | CCW | DUMP'
         DC    AL1(#CAR)
         DC    C'              INPUT | OUTPUT | QUERY | REWIND'
         DC    AL1(#CAR)
         DC    C'              SWITCH | SYNC | VERIFY | WRITE'
         DC    AL1(#CAR)
HLPF002E EQU   *
HLPF003  DC    AL1(HLPF003E-*-1)
         DC    C' Opts are --  '
         DC    C'AUTO/NOAUTO'
         DC    AL1(#CAR)
         DC    C'              '
         DC    C'BLOCK-127/381/1055/4K'
         DC    AL1(#CAR)
         DC    C'              '
         DC    C'BUFFERED/NOBUFFERED'
         DC    AL1(#CAR)
         DC    C'              '
         DC    C'CLOSE-YES/RESERVE/EXIT'
HLPF003E EQU   *
HLPF004  DC    AL1(HLPF004E-*-1)
         DC    C'              '
         DC    C'COUNT-nnnnnnnn'
         DC    AL1(#CAR)
         DC    C'              '
         DC    C'CYCLE/NOCYCLE'
         DC    AL1(#CAR)
         DC    C'              '
         DC    C'FILLCHAR-xx'
         DC    AL1(#CAR)
         DC    C'              '
         DC    C'MSG-PARMLIST/WTOPC/OFF/ALL'
HLPF004E EQU   *
HLPF005  DC    AL1(HLPF005E-*-1)
         DC    C'              '
         DC    C'OPCODE-WRT/RDF/NOP/SNS/REW/RUN/RDB/RBL/ERG/WTM'
         DC    AL1(#CAR)
         DC    C'              '
         DC    C'       BID/BSB/BSF/FSB/FSF/SYN/DD0/DD1/DD2/SID'
HLPF005E EQU   *
HLPF006  DC    AL1(HLPF006E-*-1)
         DC    C'              '
         DC    C'OPEN-INPUT/OUTPUT/ASSIGN/NO'
         DC    AL1(#CAR)
         DC    C'              '
         DC    C'TAPE-xxx'
HLPF006E EQU   *
         SPACE 2
HLPX999  DC    AL1(L'HLPX999T)
HLPX999T DC    C'END OF DISPLAY'
         EJECT 1
HLPSLIST DC    A(HLPS001-QXTK)         ADDRESS OF FIRST BLOCK
         DC    A(HLPS002-QXTK)         ADDRESS OF NEXT BLOCK
         DC    A(HLPS003-QXTK)         ADDRESS OF NEXT BLOCK
         DC    A(HLPS004-QXTK)         ADDRESS OF NEXT BLOCK
         DC    A(HLPS005-QXTK)         ADDRESS OF NEXT BLOCK
         DC    A(HLPS006-QXTK)         ADDRESS OF NEXT BLOCK
         DC    A(HLPS007-QXTK)         ADDRESS OF NEXT BLOCK
         DC    A(HLPS008-QXTK)         ADDRESS OF NEXT BLOCK
         DC    A(0)                    NO MORE BLOCKS, FINISH OFF MSG
         SPACE 2
HLPS001  DC    AL1(HLPS001E-*-1)       CALCULATE LENGTH OF THIS BLOCK
         DC    C'TAPE99I SYSTEM TEST interface syntax:',AL1(#CAR)
         DC    AL1(#CAR)
         DC    C' ZTEST TA64    '
         DC    C'START    tapename'
HLPS001E EQU   *
HLPS002  DC    AL1(HLPS002E-*-1)       CALCULATE LENGTH OF THIS BLOCK
         DC    C'               '
         DC    C'STOP     tapename   (BP)'
         DC    AL1(#CAR)
         DC    C'               '
         DC    C'tapename -- ZA1, ZA2, ZA3, ZA4, ZA5'
HLPS002E EQU   *
HLPS003  DC    AL1(HLPS003E-*-1)       CALCULATE LENGTH OF THIS BLOCK
         DC    C'               '
         DC    C'            ZB1, ZB2, ZB3, ZB4, ZB5'
         DC    AL1(#CAR)
         DC    C'               '
         DC    C'            ZC1, ZC2, ZC3, ZC4, ZC5'
         DC    AL1(#CAR)
         DC    C'         OR    '
         DC    C'STOP        ALL     (BP)'
HLPS003E EQU   *
HLPS004  DC    AL1(HLPS004E-*-1)       CALCULATE LENGTH OF THIS BLOCK
         DC    C'               '
         DC    C'SETUP    -- Define tape labels'
         DC    AL1(#CAR)
         DC    C'               '
         DC    C'STATUS   -- Display driver status'
HLPS004E EQU   *
HLPS005  DC    AL1(HLPS005E-*-1)       CALCULATE LENGTH OF THIS BLOCK
         DC    C'               '
         DC    C'PAUSE    -- Pause driver activity'
         DC    AL1(#CAR)
         DC    C'               '
         DC    C'RESTART  -- Restart driver activity'
HLPS005E EQU   *
HLPS006  DC    AL1(HLPS006E-*-1)       CALCULATE LENGTH OF THIS BLOCK
         DC    C'               '
         DC    C'CONTROL  -- Change/display global control values'
         DC    AL1(#CAR)
         DC    C'               '
         DC    C'            CRETC    -- 1 to 99'
         DC    AL1(#CAR)
         DC    C'               '
         DC    C'            RTA/NORTA'
HLPS006E EQU   *
HLPS007  DC    AL1(HLPS007E-*-1)       CALCULATE LENGTH OF THIS BLOCK
         DC    C'               '
         DC    C'            MSG      -- usual MSG values'
         DC    AL1(#CAR)
         DC    C'               '
         DC    C'            MINCOUNT -- minimum random COUNT value'
         DC    AL1(#CAR)
         DC    C'               '
         DC    C'            MAXCOUNT -- maximum random COUNT value'
HLPS007E EQU   *
HLPS008  DC    AL1(HLPS008E-*-1)       CALCULATE LENGTH OF THIS BLOCK
         DC    C'               '
         DC    C'HELPFUNC -- Show function test help'
         DC    AL1(#CAR)
         DC    C'               '
         DC    C'HELPSYS  -- Show this message'
         DC    AL1(#CAR)                                      @TEDMLIN
         DC    C'               '                             @TEDMLIN
         DC    C'HELPOTH  -- Show other tests help'           @TEDMLIN
HLPS008E EQU   *
         EJECT 1
HLPOLIST DC    A(HLPO001-QXTK)         ADDRESS OF FIRST BLOCK @TEDMLIN
         DC    A(HLPO990-QXTK)         ADDRESS OF NEXT BLOCK  @TEDMLIN
         DC    A(0)                    NO MORE BLOCKS, FINISH @TEDMLIN
         SPACE 2                                              @TEDMLIN
HLPO001  DC    AL1(HLPO001E-*-1)       CALCULATE LEN OF BLK   @TEDMLIN
         DC    C'TAPE99I Other Test Interface Syntax:',AL1(#CAR)
         DC    AL1(#CAR)                                      @TEDMLIN
         DC    C' ZTEST TA64    '                             @TEDMLIN
         DC    C'OTEST1   -- Test tape_cntl, tsync, tdsp_q'   @TEDMLIN
HLPO001E EQU   *                                              @TEDMLIN
HLPO990  DC    AL1(HLPO990E-*-1)       CALCULATE LEN OF BLK   @TEDMLIN
         DC    C'               '                             @TEDMLIN
         DC    C'HELPFUNC -- Show function test help'         @TEDMLIN
         DC    AL1(#CAR)                                      @TEDMLIN
         DC    C'               '                             @TEDMLIN
         DC    C'HELPSYS  -- Show system test help'           @TEDMLIN
         DC    AL1(#CAR)                                      @TEDMLIN
         DC    C'               '                             @TEDMLIN
         DC    C'HELPOTH  -- Show this message'               @TEDMLIN
HLPO990E EQU   *                                              @TEDMLIN
         EJECT 1                                              @TEDMLIN
*----------------------------------------------------------------------
*   SKELETON WTOPC MACROS
*----------------------------------------------------------------------
         DCTMSG
         USING DCTMSG,R6
QXTKWTP1 DS    0H
         CLI   MSGLET,C'E'    ERROR MESSAGE?
         JNE   WTP1$010       NO, CHECK FOR OTHERS                      64MODE
         TM    MSGDEF,MSGEMSG PROCESS ERROR MESSAGES?
         BNOR  R7             NO, RETURN
         J     WTP1$090       OK, PROCESS THE MESSAGE                   64MODE
WTP1$010 DS    0H
         CLI   MSGLET,C'I'    INFORMATIVE MESSAGE?
         BNER  R7             NO, I DON'T KNOW WHAT IT IS... RETURN
         TM    MSGDEF,MSGIMSG PROCESS THESE MESSAGES?
         BNOR  R7             NO, RETURN
WTP1$090 DS    0H
         WTOPC PREFIX=TA64,HEADER=NO,TIME=NO,CHAIN=YES,ENDOFM=YES,     X
               DOT=YES,                                                X
               NUMA=MSGNUM,                                            X
               LETA=MSGLET,                                            X
               TEXTA=MSGLEN,                                           X
               COMP=YES,                                               X
               EBROUT=YES,                                             X
               ROUT=(TAPE),            ROUTE TO PRC IF NOT ALREADY     X
               SUB=(DEC,(R3)),                                         X
               MF=(E,WTOPCXXX)         AVOID USE OF EBW000-003
         BR    R7
         DROP  R6
QXTKWTP2 DS    0H
         WTOPC PREFIX=TA64,HEADER=NO,TIME=NO,CHAIN=YES,ENDOFMR=(R1),   X
               DOT=NO,                                                 X
               NUM=99,                                                 X
               LET=I,                                                  X
               TEXTA=(R6),                                             X
               COMP=NO,                                                X
               MF=(E,WTOPCXXX)         AVOID USE OF EBW000-003
         BR    R7
         EJECT 1
MINAVAIL DC    F'40'                   MINIMUM % BLOCK AVAILABILITY
*
REALTIME DC    AL(L'CP0RTT)(CP0RTT)    REAL TIME LOGGING TAPE NAME
REALPRFX EQU   REALTIME,2              REAL TIME TAPE NAME PREFIX
*
         LTORG
         PRINT GEN
         TA0EQ GLOB=YES
         PRINT NOGEN
         FINIS QXTK
         END  ,
